@@ -4,7 +4,7 @@
 # Copyright 2010 Johan Samyn <johan.samyn@gmail.com>
 #
 # This software may be used and distributed according to the terms of the
-# GNU General Public License version 2, incorporated herein by reference.
+# GNU General Public License version 2 or any later version.
 
 from __future__ import absolute_import
 
@@ -44,9 +44,10 @@ from . import (
 
 if hglib.TYPE_CHECKING:
     from typing import (
+        Dict,
+        List,
         Optional,
         Text,
-        Union,
     )
 
     from . import (
@@ -79,7 +80,7 @@ class ArchiveWidget(cmdui.AbstractCmdWidget):
     _archive_content_touched_since = 2
 
     def __init__(self, repoagent, rev, parent=None, minrev=None):
-        # type: (thgrepo.RepoAgent, Optional[Union[Text]], Optional[QWidget], Optional[Union[Text]]) -> None
+        # type: (thgrepo.RepoAgent, Optional[Text], Optional[QWidget], Optional[Text]) -> None
         super(ArchiveWidget, self).__init__(parent)
         self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
         self._repoagent = repoagent
@@ -91,8 +92,8 @@ class ArchiveWidget(cmdui.AbstractCmdWidget):
 
         possibleroots = []
         if minrev:
-            parents = scmutil.revsymbol(self.repo,
-                                        hglib.fromunicode(minrev)).parents()
+            parents = hglib.revsymbol(self.repo,
+                                      hglib.fromunicode(minrev)).parents()
             if parents:
                 for p in parents:
                     possibleroots.append(u'%d' % p.rev())
@@ -206,10 +207,12 @@ class ArchiveWidget(cmdui.AbstractCmdWidget):
         return self._repoagent.rawRepo()
 
     def rev_combo_changed(self):
+        # type: () -> None
         self.subrepos_chk.setChecked(self.get_subrepos_present())
         self.update_path()
 
     def browse_clicked(self):
+        # type: () -> None
         """Select the destination directory or file"""
         dest = pycompat.unicode(self.dest_edit.text())
         if not os.path.exists(dest):
@@ -231,12 +234,13 @@ class ArchiveWidget(cmdui.AbstractCmdWidget):
             self.update_path()
 
     def get_subrepos_present(self):
+        # type: () -> bool
         rev = self.get_selected_rev()
         try:
-            ctx = scmutil.revsymbol(self.repo, rev)
+            ctx = hglib.revsymbol(self.repo, rev)
         except (error.LookupError, error.RepoLookupError):
             return False
-        return '.hgsubstate' in ctx
+        return b'.hgsubstate' in ctx
 
     def get_selected_rev(self):
         # type: () -> bytes
@@ -248,20 +252,27 @@ class ArchiveWidget(cmdui.AbstractCmdWidget):
         return rev
 
     def get_selected_rootrev(self):
+        # type: () -> bytes
         rev = self.rootrev_combo.currentText()
         return hglib.fromunicode(rev)
 
     def get_selected_archive_type(self):
+        # type: () -> Dict[Text, Text]
         """Return a dictionary describing the selected archive type"""
         return _ARCHIVE_TYPES[self._typesradios.checkedId()]
 
     def update_path(self):
+        # type: () -> None
+
+        # pytype: disable=redundant-function-type-comment
         def remove_ext(path):
+            # type: (Text) -> Text
             for ext in ('.tar', '.tar.bz2', '.tar.gz', '.zip'):
                 if path.endswith(ext):
                     return path.replace(ext, '')
             return path
         def remove_rev(path):
+            # type: (Text) -> Text
             l = ''
             for i in pycompat.xrange(self.rev_combo.count() - 1):
                 l += pycompat.unicode(self.rev_combo.itemText(i))
@@ -274,12 +285,16 @@ class ArchiveWidget(cmdui.AbstractCmdWidget):
                     return path.replace(rev, '')
             return path
         def add_rev(path, rev):
+            # type: (Text, Text) -> Text
             return '%s_%s' % (path, rev)
         def add_ext(path):
+            # type: (Text) -> Text
             select = self.get_selected_archive_type()
             if select['type'] != 'files':
                 path += select['ext']
             return path
+        # pytype: enable=redundant-function-type-comment
+
         text = pycompat.unicode(self.rev_combo.currentText())
         if len(text) == 0:
             self.commandChanged.emit()
@@ -289,7 +304,7 @@ class ArchiveWidget(cmdui.AbstractCmdWidget):
             text = wdrev
         else:
             try:
-                scmutil.revsymbol(self.repo, hglib.fromunicode(text))
+                hglib.revsymbol(self.repo, hglib.fromunicode(text))
             except (error.RepoError, error.LookupError):
                 self.commandChanged.emit()
                 return
@@ -304,6 +319,7 @@ class ArchiveWidget(cmdui.AbstractCmdWidget):
 
     @pyqtSlot()
     def compose_command(self):
+        # type: () -> List[Text]
         content = self.content_mode.checkedId()
         targetrev = hglib.tounicode(self.get_selected_rev())
         if content == self._archive_content_all_files:
@@ -325,6 +341,7 @@ class ArchiveWidget(cmdui.AbstractCmdWidget):
         return cmdline
 
     def canRunCommand(self):
+        # type: () -> bool
         rev = self.get_selected_rev()
         if not rev or not self.dest_edit.text():
             return False
@@ -335,6 +352,7 @@ class ArchiveWidget(cmdui.AbstractCmdWidget):
             return False
 
     def runCommand(self):
+        # type: () -> cmdcore.CmdSession
         # verify input
         type = self.get_selected_archive_type()['type']
         dest = pycompat.unicode(self.dest_edit.text())
@@ -369,7 +387,7 @@ class ArchiveWidget(cmdui.AbstractCmdWidget):
 
 
 def createArchiveDialog(repoagent, rev=None, parent=None, minrev=None):
-    # type: (thgrepo.RepoAgent, Optional[Union[Text]], Optional[QWidget], Optional[Union[Text]]) -> cmdui.CmdControlDialog
+    # type: (thgrepo.RepoAgent, Optional[Text], Optional[QWidget], Optional[Text]) -> cmdui.CmdControlDialog
     dlg = cmdui.CmdControlDialog(parent)
     dlg.setWindowTitle(_('Archive - %s') % repoagent.displayName())
     dlg.setWindowIcon(qtlib.geticon('hg-archive'))
