@@ -3,7 +3,7 @@
 # Copyright 2010 Yuki KODAMA <endflow.net@gmail.com>
 #
 # This software may be used and distributed according to the terms of the
-# GNU General Public License version 2, incorporated herein by reference.
+# GNU General Public License version 2 or any later version.
 
 from __future__ import absolute_import
 
@@ -46,7 +46,28 @@ from . import (
     wctxcleaner,
 )
 
+if hglib.TYPE_CHECKING:
+    from typing import (
+        Any,
+        Optional,
+        Text,
+    )
+    from mercurial import (
+        localrepo,
+    )
+    from .qtgui import (
+        QWidget,
+    )
+    from .thgrepo import (
+        RepoAgent,
+    )
+    from ..util.typelib import (
+        HgContext,
+    )
+
+
 def checkrev(repo, rev):
+    # type: (localrepo.localrepository, int) -> Optional[Text]
     op1, op2 = repo.dirstate.parents()
     if op1 is None:
         return _('Backout requires a parent revision')
@@ -60,6 +81,7 @@ def checkrev(repo, rev):
 class BackoutDialog(QWizard):
 
     def __init__(self, repoagent, rev, parent=None):
+        # type: (RepoAgent, int, Optional[QWidget]) -> None
         super(BackoutDialog, self).__init__(parent)
         self._repoagent = repoagent
         f = self.windowFlags()
@@ -88,6 +110,7 @@ class BackoutDialog(QWizard):
         self._readSettings()
 
     def _readSettings(self):
+        # type: () -> None
         qs = QSettings()
         qs.beginGroup('backout')
         for n in ['autoadvance', 'skiplast']:
@@ -100,6 +123,7 @@ class BackoutDialog(QWizard):
         qs.endGroup()
 
     def _writeSettings(self):
+        # type: () -> None
         qs = QSettings()
         qs.beginGroup('backout')
         for n in ['autoadvance', 'autoresolve', 'skiplast']:
@@ -108,60 +132,74 @@ class BackoutDialog(QWizard):
 
     @pyqtSlot()
     def repositoryChanged(self):
+        # type: () -> None
         self.currentPage().repositoryChanged()
 
     @pyqtSlot()
     def configChanged(self):
+        # type: () -> None
         self.currentPage().configChanged()
 
     def pageChanged(self, id):
+        # type: (int) -> None
         if id != -1:
             self.currentPage().currentPage()
 
     def reject(self):
+        # type: () -> None
         if self.currentPage().canExit():
             super(BackoutDialog, self).reject()
 
     def done(self, r):
+        # type: (int) -> None
         self._writeSettings()
         super(BackoutDialog, self).done(r)
 
 
 class BasePage(QWizardPage):
     def __init__(self, repoagent, parent):
+        # type: (RepoAgent, Optional[QWidget]) -> None
         super(BasePage, self).__init__(parent)
         self._repoagent = repoagent
 
     @property
     def repo(self):
+        # type: () -> localrepo.localrepository
         return self._repoagent.rawRepo()
 
     def validatePage(self):
+        # type: () -> bool
         'user pressed NEXT button, can we proceed?'
         return True
 
     def isComplete(self):
+        # type: () -> bool
         'should NEXT button be sensitive?'
         return True
 
     def repositoryChanged(self):
+        # type: () -> None
         'repository has detected a change to changelog or parents'
         pass
 
     def configChanged(self):
+        # type: () -> None
         'repository has detected a change to config files'
         pass
 
     def currentPage(self):
+        # type: () -> None
         pass
 
     def canExit(self):
+        # type: () -> bool
         return True
 
 
 class SummaryPage(BasePage):
 
     def __init__(self, repoagent, backoutrev, parentbackout, parent):
+        # type: (RepoAgent, int, bool, Optional[QWidget]) -> None
         super(SummaryPage, self).__init__(repoagent, parent)
         self._wctxcleaner = wctxcleaner.WctxCleaner(repoagent, self)
         self._wctxcleaner.checkStarted.connect(self._onCheckStarted)
@@ -239,32 +277,39 @@ class SummaryPage(BasePage):
         self.groups.set_visible(False, 'dirty')
 
     def isComplete(self):
+        # type: () -> bool
         'should Next button be sensitive?'
         return self._wctxcleaner.isClean()
 
     def repositoryChanged(self):
+        # type: () -> None
         'repository has detected a change to changelog or parents'
         pctx = self.repo[b'.']
         self.localCsInfo.update(pctx)
 
     def canExit(self):
+        # type: () -> bool
         'can backout tool be closed?'
         if self._wctxcleaner.isChecking():
             self._wctxcleaner.cancelCheck()
         return True
 
     def currentPage(self):
+        # type: () -> None
         self.refresh()
 
     def refresh(self):
+        # type: () -> None
         self._wctxcleaner.check()
 
     @pyqtSlot()
     def _onCheckStarted(self):
+        # type: () -> None
         self.groups.set_visible(True, 'prog')
 
     @pyqtSlot(bool)
     def _onCheckFinished(self, clean):
+        # type: (bool) -> None
         self.groups.set_visible(False, 'prog')
         if self._wctxcleaner.isCheckCanceled():
             return
@@ -280,6 +325,7 @@ class SummaryPage(BasePage):
 
 class BackoutPage(BasePage):
     def __init__(self, repoagent, backoutrev, parentbackout, parent):
+        # type: (RepoAgent, int, bool, Optional[QWidget]) -> None
         super(BackoutPage, self).__init__(repoagent, parent)
         self._backoutrev = backoutrev
         self._parentbackout = parentbackout
@@ -304,6 +350,7 @@ class BackoutPage(BasePage):
         self.layout().addWidget(autonext)
 
     def currentPage(self):
+        # type: () -> None
         if self._parentbackout:
             self.wizard().next()
             return
@@ -316,6 +363,7 @@ class BackoutPage(BasePage):
         sess.outputReceived.connect(self._cmdlog.appendLog)
 
     def isComplete(self):
+        # type: () -> bool
         'should Next button be sensitive?'
         if not self.backoutcomplete:
             return False
@@ -335,11 +383,13 @@ class BackoutPage(BasePage):
 
     @pyqtSlot(bool)
     def tryAutoAdvance(self, checked):
+        # type: (bool) -> None
         if checked and self.isComplete():
             self.wizard().next()
 
     @pyqtSlot(int)
     def onCommandFinished(self, ret):
+        # type: (int) -> None
         if ret in (0, 1):
             self.backoutcomplete = True
             if self.field('autoadvance'):
@@ -348,6 +398,7 @@ class BackoutPage(BasePage):
 
     @pyqtSlot(str)
     def onLinkActivated(self, cmd):
+        # type: (Text) -> None
         if cmd == 'resolve':
             dlg = resolve.ResolveDialog(self._repoagent, self)
             dlg.exec_()
@@ -359,6 +410,7 @@ class BackoutPage(BasePage):
 class CommitPage(BasePage):
 
     def __init__(self, repoagent, backoutrev, parentbackout, parent):
+        # type: (RepoAgent, int, bool, Optional[QWidget]) -> None
         super(CommitPage, self).__init__(repoagent, parent)
         self._backoutrev = backoutrev
         self._parentbackout = parentbackout
@@ -371,14 +423,17 @@ class CommitPage(BasePage):
 
         repo = repoagent.rawRepo()
 
+        # pytype: disable=redundant-function-type-comment
         # csinfo
         def label_func(widget, item, ctx):
+            # type: (Any, Text, HgContext) -> Any
             if item == 'rev':
                 return _('Revision:')
             elif item == 'parents':
                 return _('Parents')
             raise csinfo.UnknownItem()
         def data_func(widget, item, ctx):
+            # type: (Any, Text, HgContext) -> Any
             if item == 'rev':
                 return _('Working Directory'), str(ctx)
             elif item == 'parents':
@@ -388,6 +443,11 @@ class CommitPage(BasePage):
                     branch = None
                     if hasattr(pctx, 'branch') and pctx.branch() != cbranch:
                         branch = pctx.branch()
+                    # TODO: convert branch name to str like data_func() in
+                    #    revpanel?  The branch always seems to be None, because
+                    #    ctx always has one parent and is pctx with a '+',
+                    #    unless backing out in the middle of an uncommitted
+                    #    merge.
                     parents.append((str(pctx.rev()), str(pctx), branch, pctx))
                 return parents
             raise csinfo.UnknownItem()
@@ -400,6 +460,7 @@ class CommitPage(BasePage):
                     return '<a href="view">%s</a> (%s)' % (text, rev)
             elif item == 'parents':
                 def branch_markup(branch):
+                    # type: (Text) -> Text
                     opts = dict(fg='black', bg='#aaffaa')
                     return qtlib.markup(' %s ' % branch, **opts)
                 csets = []
@@ -414,6 +475,8 @@ class CommitPage(BasePage):
                     csets.append(line)
                 return csets
             raise csinfo.UnknownItem()
+        # pytype: enable=redundant-function-type-comment
+
         custom = csinfo.custom(label=label_func, data=data_func,
                                markup=markup_func)
         contents = ('rev', 'user', 'dateage', 'branch', 'parents')
@@ -479,13 +542,16 @@ class CommitPage(BasePage):
         self.layout().addWidget(self.engChk)
 
     def refresh(self):
+        # type: () -> None
         pass
 
     def cleanupPage(self):
+        # type: () -> None
         s = QSettings()
         self.msgEntry.saveSettings(s, 'backout/message')
 
     def currentPage(self):
+        # type: () -> None
         engmsg = self._repoagent.configBool('tortoisehg', 'engmsg')
         msgset = i18n.keepgettext()._('Backed out changeset: ')
         msg = engmsg and msgset['id'] or msgset['str']
@@ -494,15 +560,18 @@ class CommitPage(BasePage):
 
     @pyqtSlot(str)
     def onLinkActivated(self, cmd):
+        # type: (Text) -> None
         if cmd == 'view':
             dlg = status.StatusDialog(self._repoagent, [], {}, self)
             dlg.exec_()
             self.refresh()
 
     def isComplete(self):
+        # type: () -> bool
         return len(self.msgEntry.text()) > 0
 
     def validatePage(self):
+        # type: () -> bool
         if self.commitComplete:
             # commit succeeded, repositoryChanged() called wizard().next()
             if self.field('skiplast'):
@@ -511,7 +580,7 @@ class CommitPage(BasePage):
         if not self._cmdsession.isFinished():
             return False
 
-        user = hglib.tounicode(qtlib.getCurrentUsername(self, self.repo))
+        user = qtlib.getCurrentUsername(self, self.repo)
         if not user:
             return False
 
@@ -543,6 +612,7 @@ class CommitPage(BasePage):
 
     @pyqtSlot(int)
     def onCommandFinished(self, ret):
+        # type: (int) -> None
         if ret == 0:
             self.commitComplete = True
             self.wizard().next()
@@ -550,6 +620,7 @@ class CommitPage(BasePage):
 
 class ResultPage(BasePage):
     def __init__(self, repoagent, parent):
+        # type: (RepoAgent, Optional[QWidget]) -> None
         super(ResultPage, self).__init__(repoagent, parent)
         self.setTitle(_('Finished'))
         self.setSubTitle(' ')
@@ -564,5 +635,6 @@ class ResultPage(BasePage):
         self.layout().addStretch(1)
 
     def currentPage(self):
+        # type: () -> None
         self.bkCsInfo.update(self.repo[b'tip'])
         self.wizard().setOption(QWizard.NoCancelButton, True)
