@@ -135,11 +135,11 @@ class BlockList(QWidget):
             color = self.blockTypes[typ]
             p.setPen(color)  # make sure the height is at least 1px
             p.setBrush(color)
-            p.drawRect(1, alo * sy, w - 1, (ahi - alo) * sy)
+            p.drawRect(1, int(alo * sy), w - 1, int((ahi - alo) * sy))
 
         p.setPen(self._vrectbordercolor)
         p.setBrush(self._vrectcolor)
-        p.drawRect(0, self._value * sy, w, self._pagestep * sy)
+        p.drawRect(0, int(self._value * sy), w, int(self._pagestep * sy))
 
     def scrollToPos(self, y):
         # Scroll to the position which specified by Y coodinate.
