@@ -2,8 +2,8 @@
 #
 # Copyright 2010 Steve Borho <steve@borho.org>
 #
-# This software may be used and distributed according to the terms
-# of the GNU General Public License, incorporated herein by reference.
+# This software may be used and distributed according to the terms of the
+# GNU General Public License version 2 or any later version.
 
 from __future__ import absolute_import
 
@@ -714,7 +714,7 @@ class DiffBrowser(QFrame):
 
     def setContext(self, ctx):
         self._ctx = ctx
-        self.sci.setTabWidth(ctx._repo.tabwidth)
+        self.sci.setTabWidth(ctx.repo().tabwidth)
 
     def clearDisplay(self):
         self.sci.clear()
