@@ -5,7 +5,7 @@
 # Copyright 2010 Yuki KODAMA <endflow.net@gmail.com>
 #
 # This software may be used and distributed according to the terms of the
-# GNU General Public License version 2, incorporated herein by reference.
+# GNU General Public License version 2 or any later version.
 
 from __future__ import absolute_import
 
@@ -48,17 +48,30 @@ from . import (
 if hglib.TYPE_CHECKING:
     from typing import (
         Any,
+        List,
         Optional,
         Text,
         Tuple,
     )
+    from mercurial import (
+        ui as uimod,
+    )
+    from .qtcore import (
+        QSettings,
+    )
+    from .hgconfig import (
+        HgConfig,
+    )
+
 
 def _startrev_available():
+    # type: () -> bool
     entry = cmdutil.findcmd(b'clone', commands.table)[1]
     longopts = set(e[1] for e in entry[1])
     return b'startrev' in longopts
 
 def _suggesteddest(src, basedest):
+    # type: (Text, Text) -> Text
     if '://' in basedest:
         return basedest
     try:
@@ -80,6 +93,7 @@ def _suggesteddest(src, basedest):
 class CloneWidget(cmdui.AbstractCmdWidget):
 
     def __init__(self, config, cmdagent, parent=None):
+        # type: (HgConfig, cmdcore.CmdAgent, Optional[QWidget]) -> None
         super(CloneWidget, self).__init__(parent)
         self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
         self._config = config
@@ -246,6 +260,7 @@ class CloneWidget(cmdui.AbstractCmdWidget):
         self._composeCommand()
 
     def readSettings(self, qs):
+        # type: (QSettings) -> None
         for key, combo in [('source', self.src_combo),
                            ('dest', self.dest_combo)]:
             # addItems() can overwrite temporary edit text
@@ -259,6 +274,7 @@ class CloneWidget(cmdui.AbstractCmdWidget):
         self.src_combo.lineEdit().selectAll()
 
     def writeSettings(self, qs):
+        # type: (QSettings) -> None
         for key, combo in [('source', self.src_combo),
                            ('dest', self.dest_combo)]:
             l = [combo.currentText()]
@@ -267,40 +283,50 @@ class CloneWidget(cmdui.AbstractCmdWidget):
             qs.setValue(key, l[:10])
 
     def source(self):
+        # type: () -> Text
         return self.src_combo.currentText().strip()
 
     def setSource(self, url):
+        # type: (Text) -> None
         self.src_combo.setCurrentIndex(self.src_combo.findText(url))
         self.src_combo.setEditText(url)
 
     def destination(self):
+        # type: () -> Text
         return self.dest_combo.currentText().strip()
 
     def setDestination(self, url):
+        # type: (Text) -> None
         self.dest_combo.setCurrentIndex(self.dest_combo.findText(url))
         self.dest_combo.setEditText(url)
 
     @pyqtSlot()
     def _suggestDestination(self):
+        # type: () -> None
         self.setDestination(_suggesteddest(self.source(), self.destination()))
 
     def revSymbol(self):
+        # type: () -> Text
         if not self.rev_chk.isChecked():
             return ''
         return self.rev_text.text().strip()
 
     def setRevSymbol(self, rev):
+        # type: (Text) -> None
         self.rev_chk.setChecked(bool(rev))
         self.rev_text.setText(rev)
 
     def testOption(self, key):
+        # type: (Text) -> bool
         return self._opt_checks[key].isChecked()
 
     def setOption(self, key, on):
+        # type: (Text, bool) -> None
         self._opt_checks[key].setChecked(on)
 
     @pyqtSlot()
     def _composeCommand(self):
+        # type: () -> List[Text]
         opts = {
             'verbose': True,
             'config': [],
@@ -337,15 +363,18 @@ class CloneWidget(cmdui.AbstractCmdWidget):
         return cmdline
 
     def canRunCommand(self):
+        # type: () -> bool
         src, dest = self.source(), self.destination()
         return bool(src and dest and src != dest)
 
     def runCommand(self):
+        # type: () -> cmdcore.CmdSession
         cmdline = self._composeCommand()
         return self._cmdagent.runCommand(cmdline, self)
 
     @pyqtSlot()
     def _browseSource(self):
+        # type: () -> None
         FD = QFileDialog
         caption = _("Select source repository")
         path = FD.getExistingDirectory(self, caption,
@@ -357,6 +386,7 @@ class CloneWidget(cmdui.AbstractCmdWidget):
 
     @pyqtSlot()
     def _browseDestination(self):
+        # type: () -> None
         FD = QFileDialog
         caption = _("Select destination repository")
         path = FD.getExistingDirectory(self, caption,
@@ -368,6 +398,7 @@ class CloneWidget(cmdui.AbstractCmdWidget):
 
     @pyqtSlot()
     def _browsePatchQueue(self):
+        # type: () -> None
         FD = QFileDialog
         caption = _("Select patch folder")
         upatchroot = os.path.join(self.src_combo.currentText(), '.hg')
@@ -379,6 +410,7 @@ class CloneWidget(cmdui.AbstractCmdWidget):
 
     @pyqtSlot()
     def _onSourceChanged(self):
+        # type: () -> None
         self.insecure_chk.setEnabled(self.source().startswith('https://'))
         self._composeCommand()
 
@@ -388,12 +420,10 @@ class CloneDialog(cmdui.CmdControlDialog):
     clonedRepository = pyqtSignal(str, str)
 
     def __init__(self, ui, config, parent=None):
+        # type: (uimod.ui, HgConfig, Optional[QWidget]) -> None
         super(CloneDialog, self).__init__(parent)
 
-        cwd = os.getcwd()
-        ucwd = hglib.tounicode(cwd)
-
-        self.setWindowTitle(_('Clone - %s') % ucwd)
+        self.setWindowTitle(_('Clone - %s') % hglib.getcwdu())
         self.setWindowIcon(qtlib.geticon('hg-clone'))
         self.setObjectName('clone')
         self.setRunButtonText(_('&Clone'))
@@ -403,39 +433,49 @@ class CloneDialog(cmdui.CmdControlDialog):
         self.commandFinished.connect(self._emitCloned)
 
     def source(self):
+        # type: () -> Text
         return self.commandWidget().source()
 
     def setSource(self, url):
+        # type: (Text) -> None
         assert self.isCommandFinished()
         self.commandWidget().setSource(url)
 
     def destination(self):
+        # type: () -> Text
         return self.commandWidget().destination()
 
     def setDestination(self, url):
+        # type: (Text) -> None
         assert self.isCommandFinished()
         self.commandWidget().setDestination(url)
 
     def revSymbol(self):
+        # type: () -> Text
         return self.commandWidget().revSymbol()
 
     def setRevSymbol(self, rev):
+        # type: (Text) -> None
         assert self.isCommandFinished()
         self.commandWidget().setRevSymbol(rev)
 
     def testOption(self, key):
+        # type: (Text) -> bool
         return self.commandWidget().testOption(key)
 
     def setOption(self, key, on):
+        # type: (Text, bool) -> None
         assert self.isCommandFinished()
         self.commandWidget().setOption(key, on)
 
     @pyqtSlot(int)
     def _emitCloned(self, ret):
+        # type: (int) -> None
         if ret == 0:
             self.clonedRepository.emit(self.destination(), self.source())
 
     def done(self, r):
+        # type: (int) -> None
         if self._cmdagent.isServiceRunning():
             self._cmdagent.stopService()
             return  # postponed until serviceStopped
