@@ -5,7 +5,7 @@
 # Copyright 2012 Yuya Nishihara <yuya@tcha.org>
 #
 # This software may be used and distributed according to the terms of the
-# GNU General Public License version 2, incorporated herein by reference.
+# GNU General Public License version 2 or any later version.
 
 from __future__ import absolute_import
 
@@ -398,7 +398,7 @@ class FilectxActions(QObject):
         output = self._cmdsession.readAll()
         mdata = QMimeData()
         mdata.setData('text/x-diff', output)  # for lossless import
-        mdata.setText(hglib.tounicode(str(output)))
+        mdata.setText(hglib.tounicode(bytes(output)))
         QApplication.clipboard().setMimeData(mdata)
 
     @actionSlot(_('Copy Absolute &Path'), None, 'Shift+Ctrl+C',
