@@ -54,7 +54,6 @@ from .qtgui import (
 
 from mercurial import (
     pycompat,
-    scmutil,
 )
 
 from ..util import hglib
@@ -378,8 +377,7 @@ class FileLogDialog(_AbstractFileDialog):
         if ':' in link:
             scheme, param = link.split(':', 1)
             if scheme == 'cset':
-                rev = scmutil.revsymbol(self.repo,
-                                        hglib.fromunicode(param)).rev()
+                rev = hglib.revsymbol(self.repo, hglib.fromunicode(param)).rev()
                 return self.goto(rev)
         QDesktopServices.openUrl(QUrl(link))
 
