@@ -28,7 +28,20 @@ from .qtgui import (
 )
 
 from ..util import hglib
-from . import qtlib
+from . import (
+    manifestmodel,
+    qtlib,
+)
+
+if hglib.TYPE_CHECKING:
+    from typing import (
+        List,
+        Optional,
+    )
+    from .qtgui import (
+        QWidget,
+    )
+
 
 class HgFileListView(QTreeView):
     """Display files and statuses between two revisions or patch"""
@@ -37,6 +50,7 @@ class HgFileListView(QTreeView):
     clearDisplay = pyqtSignal()
 
     def __init__(self, parent):
+        # type: (Optional[QWidget]) -> None
         QTreeView.__init__(self, parent)
         self.setHeaderHidden(True)
         self.setDragDropMode(QAbstractItemView.DragOnly)
@@ -48,33 +62,40 @@ class HgFileListView(QTreeView):
         self.setIconSize(qtlib.smallIconSize())
         self.setUniformRowHeights(True)
 
+    def _model(self):
+        # type: () -> manifestmodel.ManifestModel
+        model = self.model()
+        assert isinstance(model, manifestmodel.ManifestModel)
+        return model
+
     def setModel(self, model):
+        # type: (manifestmodel.ManifestModel) -> None
         QTreeView.setModel(self, model)
         model.layoutChanged.connect(self._onLayoutChanged)
         model.revLoaded.connect(self._onRevLoaded)
         self.selectionModel().currentRowChanged.connect(self._emitFileChanged)
 
     def currentFile(self):
+        # type: () -> bytes
         index = self.currentIndex()
-        model = self.model()
-        assert model is not None
+        model = self._model()
         return hglib.fromunicode(model.filePath(index))
 
     def setCurrentFile(self, path):
-        model = self.model()
-        assert model is not None
+        # type: (bytes) -> None
+        model = self._model()
         model.fetchMore(QModelIndex())  # make sure path is populated
         self.setCurrentIndex(model.indexFromPath(hglib.tounicode(path)))
 
     def getSelectedFiles(self):
-        model = self.model()
-        assert model is not None
+        # type: () -> List[bytes]
+        model = self._model()
         return [hglib.fromunicode(model.filePath(index))
                 for index in self.selectedRows()]
 
     def _initCurrentIndex(self):
-        m = self.model()
-        assert m is not None
+        # type: () -> None
+        m = self._model()
         if m.rowCount() > 0:
             self.setCurrentIndex(m.index(0, 0))
         else:
@@ -82,6 +103,7 @@ class HgFileListView(QTreeView):
 
     @pyqtSlot()
     def _onLayoutChanged(self):
+        # type: () -> None
         index = self.currentIndex()
         if index.isValid():
             self.scrollTo(index)
@@ -90,6 +112,7 @@ class HgFileListView(QTreeView):
 
     @pyqtSlot()
     def _onRevLoaded(self):
+        # type: () -> None
         index = self.currentIndex()
         if index.isValid():
             # redisplay previous row
@@ -99,9 +122,9 @@ class HgFileListView(QTreeView):
 
     @pyqtSlot()
     def _emitFileChanged(self):
+        # type: () -> None
         index = self.currentIndex()
-        m = self.model()
-        assert m is not None
+        m = self._model()
         if index.isValid():
             # TODO: delete status from fileSelected because it isn't primitive
             # pseudo directory node has no status
@@ -111,4 +134,5 @@ class HgFileListView(QTreeView):
             self.clearDisplay.emit()
 
     def selectedRows(self):
+        # type: () -> List[QModelIndex]
         return self.selectionModel().selectedRows()
