@@ -3,7 +3,7 @@
 # Copyright 2010 Steve Borho <steve@borho.org>
 #
 # This software may be used and distributed according to the terms of the
-# GNU General Public License version 2, incorporated herein by reference.
+# GNU General Public License version 2 or any later version.
 
 from __future__ import absolute_import
 
@@ -163,6 +163,7 @@ class HgFileView(QFrame):
         self.sci.cursorPositionChanged.connect(self._updateDiffActions)
         self.sci.setContextMenuPolicy(Qt.CustomContextMenu)
         self.sci.customContextMenuRequested.connect(self._onMenuRequested)
+        self.sci.SCN_ZOOM.connect(self._updateScrollBar)
 
         self.blk.linkScrollBar(self.sci.verticalScrollBar())
         self.blk.setVisible(False)
@@ -490,9 +491,7 @@ class HgFileView(QFrame):
         self.filenamelabel.setText(' ')
         self.extralabel.hide()
         self._updateDiffActions()
-
-        self.maxWidth = 0
-        self.sci.showHScrollBar(False)
+        self._updateScrollBar()
 
     @pyqtSlot()
     def _forceDisplayFile(self):
@@ -558,22 +557,6 @@ class HgFileView(QFrame):
         self.fileDisplayed.emit(fd.filePath(), fd.fileText())
 
         self.blksearch.syncPageStep()
-
-        lexer = self.sci.lexer()
-
-        if lexer:
-            font = self.sci.lexer().font(0)
-        else:
-            font = self.sci.font()
-
-        fm = QFontMetrics(font)
-        self.maxWidth = fm.maxWidth()
-        lines = pycompat.unicode(self.sci.text()).splitlines()
-        if lines:
-            # assume that the longest line has the largest width;
-            # fm.width() is too slow to apply to each line.
-            longestline = max(lines, key=len)
-            self.maxWidth += fm.width(longestline)
         self._updateScrollBar()
 
     @pyqtSlot(str, bool, bool, bool)
@@ -698,15 +681,25 @@ class HgFileView(QFrame):
             c.setupContextMenu(menu, line)
         return menu
 
-    def resizeEvent(self, event):
-        super(HgFileView, self).resizeEvent(event)
-        self._updateScrollBar()
-
+    @pyqtSlot()
     def _updateScrollBar(self):
-        sbWidth = self.sci.verticalScrollBar().width()
-        scrollWidth = self.maxWidth + sbWidth - self.sci.width()
-        self.sci.showHScrollBar(scrollWidth > 0)
-        self.sci.horizontalScrollBar().setRange(0, scrollWidth)
+        lexer = self.sci.lexer()
+        if lexer:
+            font = self.sci.lexer().font(0)
+        else:
+            font = self.sci.font()
+        fm = QFontMetrics(font)
+
+        lines = pycompat.unicode(self.sci.text()).splitlines()
+        if lines:
+            # assume that the longest line has the largest width;
+            # fm.width() is too slow to apply to each line.
+            longestline = max(lines, key=len)
+            maxWidth = fm.width(longestline)
+        else:
+            maxWidth = 0
+        # setScrollWidth() expects the value to be > 0
+        self.sci.setScrollWidth(max(maxWidth, 1))
 
 
 class _AbstractViewControl(QObject):
@@ -1198,7 +1191,7 @@ class _AnnotateViewControl(_AbstractViewControl):
     @util.propertycache
     def _margin_style(self):
         """Style for margin area"""
-        s = Qsci.QsciStyle()
+        s = Qsci.QsciStyle(qscilib.STYLE_FILEVIEW_MARGIN)
         s.setPaper(QApplication.palette().color(QPalette.Window))
         s.setFont(self._sci.font())
         return s
