@@ -4,7 +4,7 @@
 # Copyright 2010 Johan Samyn <johan.samyn@gmail.com>
 #
 # This software may be used and distributed according to the terms of the
-# GNU General Public License version 2, incorporated herein by reference.
+# GNU General Public License version 2 or any later version.
 
 from __future__ import absolute_import
 
@@ -37,9 +37,24 @@ from . import (
     qtlib,
 )
 
+if hglib.TYPE_CHECKING:
+    from typing import (
+        List,
+        Optional,
+        Text,
+    )
+    from mercurial import (
+        ui as uimod,
+    )
+    from .qtgui import (
+        QWidget,
+    )
+
+
 class InitWidget(cmdui.AbstractCmdWidget):
 
     def __init__(self, ui, cmdagent, destdir='.', parent=None):
+        # type: (uimod.ui, cmdcore.CmdAgent, Text, Optional[QWidget]) -> None
         super(InitWidget, self).__init__(parent)
         self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
         self._cmdagent = cmdagent
@@ -92,6 +107,7 @@ class InitWidget(cmdui.AbstractCmdWidget):
 
     @pyqtSlot()
     def _browseDestination(self):
+        # type: () -> None
         """Select the destination directory"""
         caption = _('Select Destination Folder')
         path = QFileDialog.getExistingDirectory(self, caption)
@@ -99,9 +115,11 @@ class InitWidget(cmdui.AbstractCmdWidget):
             self._dest_edit.setText(path)
 
     def destination(self):
+        # type: () -> Text
         return pycompat.unicode(self._dest_edit.text()).strip()
 
     def _buildCommand(self):
+        # type: () -> List[Text]
         cfgs = []
         if self._add_files_chk.isChecked():
             cfgs.append('hooks.post-init.thgskel='
@@ -112,14 +130,17 @@ class InitWidget(cmdui.AbstractCmdWidget):
 
     @pyqtSlot()
     def _composeCommand(self):
+        # type: () -> None
         cmdline = self._buildCommand()
         self._hgcmd_txt.setText('hg ' + hglib.prettifycmdline(cmdline))
         self.commandChanged.emit()
 
     def canRunCommand(self):
+        # type: () -> bool
         return bool(self.destination())
 
     def runCommand(self):
+        # type: () -> cmdcore.CmdSession
         cmdline = self._buildCommand()
         return self._cmdagent.runCommand(cmdline, self)
 
@@ -129,6 +150,7 @@ class InitDialog(cmdui.CmdControlDialog):
     newRepository = pyqtSignal(str)
 
     def __init__(self, ui, destdir='.', parent=None):
+        # type: (uimod.ui, Text, Optional[QWidget]) -> None
         super(InitDialog, self).__init__(parent)
         self.setWindowTitle(_('New Repository'))
         self.setWindowIcon(qtlib.geticon('hg-init'))
@@ -140,15 +162,18 @@ class InitDialog(cmdui.CmdControlDialog):
         self.commandFinished.connect(self._handleNewRepo)
 
     def destination(self):
+        # type: () -> Text
         return self.commandWidget().destination()
 
     @pyqtSlot(int)
     def _handleNewRepo(self, ret):
+        # type: (int) -> None
         if ret != 0:
             return
         self.newRepository.emit(self.destination())
 
     def done(self, r):
+        # type: (int) -> None
         if self._cmdagent.isServiceRunning():
             self._cmdagent.stopService()
             return  # postponed until serviceStopped
