@@ -3,7 +3,7 @@
 # Copyright 2010 Steve Borho <steve@borho.org>
 #
 # This software may be used and distributed according to the terms of the
-# GNU General Public License version 2, incorporated herein by reference.
+# GNU General Public License version 2 or any later version.
 
 from __future__ import absolute_import
 
@@ -228,9 +228,9 @@ class DiffLexerSelector(_ScriptLexerSelector):
     _lexer = Qsci.QsciLexerDiff
     regex = re.compile(br'^@@ [-]\d+,\d+ [+]\d+,\d+ @@$')
     def cfg_lexer(self, lexer):
-        for label, i in (('diff.inserted', 6),
-                         ('diff.deleted', 5),
-                         ('diff.hunk', 4)):
+        for label, i in (('diff.inserted', Qsci.QsciLexerDiff.LineAdded),
+                         ('diff.deleted', Qsci.QsciLexerDiff.LineRemoved),
+                         ('diff.hunk', Qsci.QsciLexerDiff.Position)):
             effect = qtlib.geteffect(label)
             for e in effect.split(';'):
                 if e.startswith('color:'):
