@@ -12,7 +12,23 @@ from tortoisehg.hgqt import qtlib
 from tortoisehg.util import hglib
 from tortoisehg.util.i18n import _
 
+if hglib.TYPE_CHECKING:
+    from typing import (
+        List,
+        Optional,
+        Tuple,
+    )
+    from mercurial import (
+        localrepo,
+        ui as uimod,
+    )
+    from .qtgui import (
+        QWidget,
+    )
+
+
 def _createPrompt(parent, files):
+    # type: (Optional[QWidget], List[bytes]) -> qtlib.CustomPrompt
     return qtlib.CustomPrompt(
         _('Confirm Add'),
         _('Some of the files that you have selected are of a size '
@@ -25,6 +41,7 @@ def _createPrompt(parent, files):
         0, 2, files)
 
 def promptForLfiles(parent, ui, repo, files):
+    # type: (Optional[QWidget], uimod.ui, localrepo.localrepository, List[bytes]) -> Optional[Tuple[List[bytes], List[bytes]]]
     lfiles = []
     section = b'largefiles'
     try:
