@@ -43,6 +43,18 @@ from . import (
     qtlib,
 )
 
+if hglib.TYPE_CHECKING:
+    from typing import (
+        Dict,
+        List,
+        Optional,
+        Text,
+    )
+    from mercurial import localrepo
+    from .qtgui import QWidget
+    from .thgrepo import RepoAgent
+
+
 _FILE_FILTER = ';;'.join([
     _('Word docs (*.doc *.docx)'),
     _('PDF docs (*.pdf)'),
@@ -54,9 +66,11 @@ class LockDialog(QDialog):
 
     @property
     def repo(self):
+        # type: () -> localrepo.localrepository
         return self._repoagent.rawRepo()
 
     def __init__(self, repoagent, parent=None):
+        # type: (RepoAgent, Optional[QWidget]) -> None
         QDialog.__init__(self, parent)
 
         self.setWindowTitle(_('TortoiseHg Lock Tool - %s') % \
@@ -118,10 +132,11 @@ class LockDialog(QDialog):
 
     @pyqtSlot()
     def finishSetup(self):
+        # type: () -> None
         'complete the setup, some of these steps might fail'
         # this code is specific to simplelock
         try:
-            self.sl = extensions.find('simplelock')
+            self.sl = extensions.find(b'simplelock')
         except KeyError:
             qtlib.WarningMsgBox(_('Simplelock extension not enabled'),
                                 _('Please enable and configure simplelock'),
@@ -134,12 +149,13 @@ class LockDialog(QDialog):
         self.reload()
 
     def refillModel(self):
+        # type: () -> Dict[Text, List[Text]]
         # this code is specific to simplelock
-        locks = self.sl.parseLocks(self.repo)
-        lockables = self.sl.readlockables(self.repo)
+        locks = self.sl.parseLocks(self.repo)  # type: Dict[bytes, List[bytes]]
+        lockables = self.sl.readlockables(self.repo)  # type: List[bytes]
         for wfile in lockables:
             if wfile not in locks:
-                locks[wfile] = ['', '']
+                locks[wfile] = [b'', b'']
         # this code is specific to simplelock
 
         self.locktw.clear()
@@ -151,7 +167,7 @@ class LockDialog(QDialog):
             upurpose = hglib.tounicode(purpose)
             rows.append(QTreeWidgetItem([uwfile, uuser, upurpose]))
         self.locktw.addTopLevelItems(rows)
-        return locks
+        return pycompat.rapply(hglib.tounicode, locks)
 
     def reject(self):
         s = QSettings()
@@ -169,12 +185,13 @@ class LockDialog(QDialog):
 
     @pyqtSlot()
     def lockany(self):
+        # type: () -> None
         wfile, _filter = QFileDialog.getOpenFileName(
             self, _('Open a (nonmergable) file you wish to be locked'),
-            self.repo.root, _FILE_FILTER)
+            hglib.tounicode(self.repo.root), _FILE_FILTER)
 
-        wfile = util.normpath(pycompat.unicode(wfile))
-        pathprefix = util.normpath(hglib.tounicode(self.repo.root)) + '/'
+        wfile = hglib.normpath(wfile)
+        pathprefix = hglib.tounicode(util.normpath(self.repo.root)) + '/'
         if not os.path.normcase(wfile).startswith(os.path.normcase(pathprefix)):
             self.showMessage.emit(_('File was not within current repository'))
         wfile = wfile[len(pathprefix):]
@@ -183,26 +200,31 @@ class LockDialog(QDialog):
         self.lockrun(['lock', wfile])
 
     def unlock(self, wfile):
+        # type: (Text) -> None
         self.showMessage.emit(_('Unlocking %s') % wfile)
         self.lockrun(['unlock', wfile])
 
     def lock(self, wfile, user):
+        # type: (Text, Text) -> None
         self.showMessage.emit(_('Locking %s') % wfile)
         self.lockrun(['lock', wfile])
 
     @pyqtSlot()
     def reload(self):
+        # type: () -> None
         'update list of locks, then update UI'
         self.showMessage.emit(_('Refreshing locks...'))
         self.lockrun(['locks']) # has side-effect of refreshing locks
 
     def lockrun(self, ucmdline):
+        # type: (List[Text]) -> None
         self.operation = ucmdline + [None]
         self._cmdsession = sess = self._repoagent.runCommand(ucmdline, self)
         sess.commandFinished.connect(self.operationComplete)
         self._updateUi()
 
     def operationComplete(self):
+        # type: () -> None
         locks = self.refillModel()
         self._updateUi()
 
@@ -230,17 +252,19 @@ class LockDialog(QDialog):
 
     @pyqtSlot(QModelIndex)
     def rowDoubleClicked(self, index):
+        # type: (QModelIndex) -> None
         wfile, user, purpose = self.rawrows[index.row()]
-        curuser = qtlib.getCurrentUsername(self, self.repo, {})
+        curuser = hglib.fromunicode(qtlib.getCurrentUsername(self, self.repo))
         if user or purpose:
             if user != curuser:
                 self.showMessage.emit(_('You can only release your own locks'))
             else:
-                self.unlock(wfile)
+                self.unlock(hglib.tounicode(wfile))
         else:
-            self.lock(wfile, curuser)
+            self.lock(hglib.tounicode(wfile), hglib.tounicode(curuser))
 
     def canExit(self):
+        # type: () -> bool
         return self._cmdsession.isFinished()
 
     def keyPressEvent(self, event):
