@@ -47,6 +47,31 @@ from . import (
     visdiff,
 )
 
+if hglib.TYPE_CHECKING:
+    from typing import (
+        Any,
+        Dict,
+        List,
+        Optional,
+        Sequence,
+        Text,
+        Tuple,
+        Union,
+    )
+    from mercurial import (
+        localrepo,
+    )
+    from .qtcore import (
+        QObject,
+    )
+    from .thgrepo import (
+        RepoAgent,
+    )
+    from ..util.typelib import (
+        ThgContext,
+    )
+
+
 _subrepoType2IcoMap = {
     'hg': 'hg',
     'hgsubversion': 'thg-svn-subrepo',
@@ -74,6 +99,7 @@ class ManifestModel(QAbstractItemModel):
 
     def __init__(self, repoagent, parent=None, rev=None, namefilter=None,
                  statusfilter='MASC', flat=False):
+        # type: (RepoAgent, Optional[QObject], Optional[int], Optional[Text], Text, bool) -> None
         QAbstractItemModel.__init__(self, parent)
 
         self._fileiconprovider = QFileIconProvider()
@@ -91,6 +117,7 @@ class ManifestModel(QAbstractItemModel):
         self._rootpopulated = False
 
     def data(self, index, role=Qt.DisplayRole):
+        # type: (QModelIndex, Qt.ItemDataRole) -> Optional[Any]
         if not index.isValid():
             return
 
@@ -99,11 +126,12 @@ class ManifestModel(QAbstractItemModel):
         if role == self.StatusRole:
             return self.fileStatus(index)
 
-        e = index.internalPointer()
+        e = index.internalPointer()  # type: "_Entry"
         if role in (Qt.DisplayRole, Qt.EditRole):
             return e.name
 
     def filePath(self, index):
+        # type: (QModelIndex) -> Text
         """Return path at the given index [unicode]"""
         if not index.isValid():
             return ''
@@ -116,7 +144,7 @@ class ManifestModel(QAbstractItemModel):
         if not index.isValid():
             return filedata.createNullData(repo)
 
-        f = index.internalPointer()
+        f = index.internalPointer()  # type: "_Entry"
         e = f.parent
         while e and e.ctx is None:
             e = e.parent
@@ -132,16 +160,18 @@ class ManifestModel(QAbstractItemModel):
         return filedata.createFileData(e.ctx, e.pctx, wfile, f.status, rpath)
 
     def subrepoType(self, index):
+        # type: (QModelIndex) -> Optional[Text]
         """Return the subrepo type the specified index"""
         if not index.isValid():
             return
-        e = index.internalPointer()
+        e = index.internalPointer()  # type: "_Entry"
         return e.subkind
 
     def fileIcon(self, index):
+        # type: (QModelIndex) -> QIcon
         if not index.isValid():
             return QIcon()
-        e = index.internalPointer()
+        e = index.internalPointer()  # type: "_Entry"
         k = (e.path, e.status, e.subkind)
         try:
             return self._iconcache[k]
@@ -150,6 +180,7 @@ class ManifestModel(QAbstractItemModel):
             return ic
 
     def _makeFileIcon(self, e):
+        # type: ("_Entry") -> QIcon
         if e.subkind in _subrepoType2IcoMap:
             ic = qtlib.geticon(_subrepoType2IcoMap[e.subkind])
             # use fine-tuned status overlay if any
@@ -175,10 +206,11 @@ class ManifestModel(QAbstractItemModel):
         return ic
 
     def fileStatus(self, index):
+        # type: (QModelIndex) -> Optional[Text]
         """Return the change status of the specified file"""
         if not index.isValid():
             return
-        e = index.internalPointer()
+        e = index.internalPointer()  # type: "_Entry"
         # TODO: 'S' should not be a status
         if e.subkind:
             return 'S'
@@ -186,21 +218,24 @@ class ManifestModel(QAbstractItemModel):
 
     # TODO: this should be merged to fileStatus()
     def subrepoStatus(self, index):
+        # type: (QModelIndex) -> Optional[Text]
         """Return the change status of the specified subrepo"""
         if not index.isValid():
             return
-        e = index.internalPointer()
+        e = index.internalPointer()  # type: "_Entry"
         if not e.subkind:
             return
         return e.status
 
     def isDir(self, index):
+        # type: (QModelIndex) -> bool
         if not index.isValid():
             return True  # root entry must be a directory
-        e = index.internalPointer()
+        e = index.internalPointer()  # type: "_Entry"
         return e.isdir
 
     def mimeData(self, indexes):
+        # type: (Sequence[QModelIndex]) -> QMimeData
         files = [self.filePath(i) for i in indexes if i.isValid()]
         ctx = self._rootentry.ctx
         if ctx.rev() is not None:
@@ -217,9 +252,11 @@ class ManifestModel(QAbstractItemModel):
         return m
 
     def mimeTypes(self):
+        # type: () -> List[Text]
         return ['text/uri-list']
 
     def flags(self, index):
+        # type: (QModelIndex) -> Qt.ItemFlags
         f = super(ManifestModel, self).flags(index)
         if not index.isValid():
             return f
@@ -229,11 +266,13 @@ class ManifestModel(QAbstractItemModel):
         return f
 
     def index(self, row, column, parent=QModelIndex()):
+        # type: (int, int, QModelIndex) -> QModelIndex
         if row < 0 or self.rowCount(parent) <= row or column != 0:
             return QModelIndex()
         return self.createIndex(row, column, self._parententry(parent).at(row))
 
     def indexFromPath(self, path, column=0):
+        # type: (Text, int) -> QModelIndex
         """Return index for the specified path if found [unicode]
 
         If not found, returns invalid index.
@@ -249,28 +288,33 @@ class ManifestModel(QAbstractItemModel):
         return self.createIndex(e.parent.index(e.name), column, e)
 
     def parent(self, index):
+        # type: (QModelIndex) -> QModelIndex
         if not index.isValid():
             return QModelIndex()
 
-        e = index.internalPointer()
+        e = index.internalPointer()  # type: "_Entry"
         if e.path:
             return self.indexFromPath(e.parent.path, index.column())
         else:
             return QModelIndex()
 
     def _parententry(self, parent):
+        # type: (QModelIndex) -> "_Entry"
         if parent.isValid():
             return parent.internalPointer()
         else:
             return self._rootentry
 
     def rowCount(self, parent=QModelIndex()):
+        # type: (QModelIndex) -> int
         return len(self._parententry(parent))
 
     def columnCount(self, parent=QModelIndex()):
+        # type: (QModelIndex) -> int
         return 1
 
     def rev(self, parent=QModelIndex()):
+        # type: (QModelIndex) -> Optional[int]
         """Revision number of the current changectx"""
         e = self._parententry(parent)
         if e.ctx is None or not _isreporev(e.ctx.rev()):
@@ -278,13 +322,17 @@ class ManifestModel(QAbstractItemModel):
         return e.ctx.rev()
 
     def baseRev(self, parent=QModelIndex()):
+        # type: (QModelIndex) -> int
         """Revision of the base changectx where status is calculated from"""
         e = self._parententry(parent)
         if e.pctx is None or not _isreporev(e.pctx.rev()):
             return -1
-        return e.pctx.rev()
+        rev = e.pctx.rev()
+        assert rev is not None  # help pytype
+        return rev
 
     def setRev(self, rev, prev=FirstParent):
+        # type: (Optional[int], int) -> None
         """Change to the specified repository revision; None for working-dir"""
         roote = self._rootentry
         newroote = self._newRevNode(rev, prev)
@@ -297,6 +345,7 @@ class ManifestModel(QAbstractItemModel):
             self.revLoaded.emit(self.rev())
 
     def setRawContext(self, ctx):
+        # type: (ThgContext) -> None
         """Change to the specified changectx in place of repository revision"""
         if _samectx(self._rootentry.ctx, ctx):
             return
@@ -315,11 +364,13 @@ class ManifestModel(QAbstractItemModel):
             self.revLoaded.emit(self.rev())
 
     def nameFilter(self):
+        # type: () -> Text
         """Return the current name filter"""
         return self._namefilter
 
     @pyqtSlot(str)
     def setNameFilter(self, pattern):
+        # type: (Text) -> None
         """Filter file name by partial match of glob pattern"""
         pattern = pycompat.unicode(pattern)
         if self._namefilter == pattern:
@@ -328,12 +379,14 @@ class ManifestModel(QAbstractItemModel):
         self._repopulateNodes()
 
     def statusFilter(self):
+        # type: () -> Text
         """Return the current status filter"""
         return self._statusfilter
 
     # TODO: split or remove 'S' which causes several design flaws
     @pyqtSlot(str)
     def setStatusFilter(self, status):
+        # type: (Text) -> None
         """Filter file tree by change status 'MARSC'"""
         status = str(status)
         assert all(c in 'MARSC' for c in status), repr(status)
@@ -343,6 +396,7 @@ class ManifestModel(QAbstractItemModel):
         self._repopulateNodes()
 
     def isChangedFilesOnly(self):
+        # type: () -> bool
         """Whether or not to filter by ctx.files, i.e. to exclude files not
         changed in the current revision.
 
@@ -352,16 +406,19 @@ class ManifestModel(QAbstractItemModel):
         return self._changedfilesonly
 
     def setChangedFilesOnly(self, changedonly):
+        # type: (bool) -> None
         if self._changedfilesonly == bool(changedonly):
             return
         self._changedfilesonly = bool(changedonly)
         self._repopulateNodes()
 
     def isFlat(self):
+        # type: () -> bool
         """Whether all entries are listed in the same level or per directory"""
         return self._nodeop is _listnodeop
 
     def setFlat(self, flat):
+        # type: (bool) -> None
         if self.isFlat() == bool(flat):
             return
         # self._nodeop must be changed after layoutAboutToBeChanged; otherwise
@@ -369,11 +426,13 @@ class ManifestModel(QAbstractItemModel):
         self._repopulateNodes(newnodeop=_nodeopmap[bool(flat)])
 
     def canFetchMore(self, parent):
+        # type: (QModelIndex) -> bool
         if parent.isValid():
             return False
         return not self._rootpopulated
 
     def fetchMore(self, parent):
+        # type: (QModelIndex) -> None
         if parent.isValid() or self._rootpopulated:
             return
         assert len(self._rootentry) == 0, self._rootentry
@@ -389,6 +448,7 @@ class ManifestModel(QAbstractItemModel):
         self.revLoaded.emit(self.rev())
 
     def _repopulateNodes(self, newnodeop=None, newroote=None):
+        # type: (Optional[Any], Optional["_Entry"]) -> None
         """Recreate populated nodes if any"""
         if not self._rootpopulated:
             # no stale nodes
@@ -414,6 +474,7 @@ class ManifestModel(QAbstractItemModel):
             self.layoutChanged.emit()
 
     def _newRevNode(self, rev, prev=FirstParent):
+        # type: (Optional[int], int) -> "_Entry"
         """Create empty root node for the specified revision"""
         if not _isreporev(rev):
             raise ValueError('unacceptable revision number: %r' % rev)
@@ -431,6 +492,7 @@ class ManifestModel(QAbstractItemModel):
         return roote
 
     def _populateNodes(self, roote):
+        # type: ("_Entry") -> None
         repo = self._repoagent.rawRepo()
         lpat = hglib.fromunicode(self._namefilter)
         match = _makematcher(repo, roote.ctx, lpat, self._changedfilesonly)
@@ -445,16 +507,22 @@ class _Entry(object):
                  '_child', '_nameindex')
 
     def __init__(self, name='', parent=None):
+        # type: (Text, Optional["_Entry"]) -> None
         self._name = name
         self._parent = parent
-        self.status = None
-        self.ctx = None
-        self.pctx = None
-        self.subkind = None
-        self._child = {}
-        self._nameindex = []
+        self.status = None  # type: Optional[Text]
+        # TODO: check this with newer pytype versions periodically.  It was fine
+        #       when it was `Optional[context.basectx]`.
+        # pytype: disable=not-supported-yet
+        self.ctx = None  # type: Optional[ThgContext]
+        self.pctx = None  # type: Optional[ThgContext]
+        # pytype: enable=not-supported-yet
+        self.subkind = None  # type: Optional[Text]
+        self._child = {}  # type: Dict[Text, "_Entry"]
+        self._nameindex = []  # type: List[Text]
 
     def copyskel(self):
+        # type: () -> "_Entry"
         """Create unpopulated copy of this entry"""
         e = self.__class__()
         e.status = self.status
@@ -465,10 +533,12 @@ class _Entry(object):
 
     @property
     def parent(self):
+        # type: () -> Optional["_Entry"]
         return self._parent
 
     @property
     def path(self):
+        # type: () -> Text
         if self.parent is None or not self.parent.name:
             return self.name
         else:
@@ -476,31 +546,38 @@ class _Entry(object):
 
     @property
     def name(self):
+        # type: () -> Text
         return self._name
 
     @property
     def isdir(self):
+        # type: () -> bool
         return bool(self.subkind or self._child)
 
     def __len__(self):
+        # type: () -> int
         return len(self._child)
 
     def __nonzero__(self):
+        # type: () -> bool
         # leaf node should not be False because of len(node) == 0
         return True
 
     __bool__ = __nonzero__
 
     def __getitem__(self, name):
+        # type: (Text) -> "_Entry"
         return self._child[name]
 
     def makechild(self, name):
+        # type: (Text) -> "_Entry"
         if name not in self._child:
             self._nameindex.append(name)
         self._child[name] = e = self.__class__(name, parent=self)
         return e
 
     def putchild(self, name, e):
+        # type: (Text, "_Entry") -> None
         assert not e.name and not e.parent, (e.name, e.parent)
         e._name = name
         e._parent = self
@@ -509,15 +586,19 @@ class _Entry(object):
         self._child[name] = e
 
     def __contains__(self, item):
+        # type: (Text) -> bool
         return item in self._child
 
     def at(self, index):
+        # type: (int) -> "_Entry"
         return self._child[self._nameindex[index]]
 
     def index(self, name):
+        # type: (Text) -> int
         return self._nameindex.index(name)
 
     def sort(self, reverse=False):
+        # type: (bool) -> None
         """Sort the entries recursively; directories first"""
         for e in self._child.values():
             e.sort(reverse=reverse)
@@ -527,10 +608,12 @@ class _Entry(object):
 
 
 def _isreporev(rev):
+    # type: (Optional[Union[bytes, int]]) -> bool
     # patchctx.rev() returns str, which isn't a valid repository revision
     return rev is None or isinstance(rev, int)
 
 def _samectx(ctx1, ctx2):
+    # type: (ThgContext, ThgContext) -> bool
     # no fast way to detect changes in uncommitted ctx, just assumes different
     if ctx1.rev() is None or not _isreporev(ctx1.rev()):
         return False
@@ -539,6 +622,7 @@ def _samectx(ctx1, ctx2):
 
 # TODO: visual feedback to denote query type and error as in repofilter
 def _makematcher(repo, ctx, pat, changedonly):
+    # type: (localrepo.localrepository, ThgContext, bytes, bool) -> matchmod.basematcher
     cwd = b''  # always relative to repo root
     patterns = []
     if pat and b':' not in pat and b'*' not in pat:
@@ -567,14 +651,17 @@ class _listnodeop(object):
 
     @staticmethod
     def findpath(e, path):
+        # type: (_Entry, Text) -> _Entry
         return e[path]
 
     @staticmethod
     def makepath(e, path):
+        # type: (_Entry, Text) -> _Entry
         return e.makechild(path)
 
     @staticmethod
     def putpath(e, path, c):
+        # type: (_Entry, Text, _Entry) -> None
         e.putchild(path, c)
 
 class _treenodeop(object):
@@ -582,12 +669,14 @@ class _treenodeop(object):
 
     @staticmethod
     def findpath(e, path):
+        # type: (_Entry, Text) -> _Entry
         for p in path.split('/'):
             e = e[p]
         return e
 
     @staticmethod
     def makepath(e, path):
+        # type: (_Entry, Text) -> _Entry
         for p in path.split('/'):
             if p not in e:
                 e.makechild(p)
@@ -596,6 +685,7 @@ class _treenodeop(object):
 
     @staticmethod
     def putpath(e, path, c):
+        # type: (_Entry, Text, _Entry) -> None
         rp = path.rfind('/')
         if rp >= 0:
             e = _treenodeop.makepath(e, path[:rp])
@@ -608,6 +698,8 @@ _nodeopmap = {
 
 
 def _populaterepo(roote, repo, nodeop, statusfilter, match):
+    # type: (_Entry, localrepo.localrepository, Any, Text, matchmod.basematcher) -> None
+    # TODO: replace 'Any' with structural typing for _treenodeop/_listnodeop
     if 'S' in statusfilter:
         _populatesubrepos(roote, repo, nodeop, statusfilter, match)
 
@@ -625,6 +717,7 @@ def _populaterepo(roote, repo, nodeop, statusfilter, match):
             e.status = st
 
 def _comparesubstate(state1, state2):
+    # type: (Tuple[bytes, bytes, bytes], Tuple[bytes, bytes, bytes]) -> Text
     if state1 == state2:
         return 'C'
     elif state1 == hglib.nullsubrepostate:
@@ -635,6 +728,8 @@ def _comparesubstate(state1, state2):
         return 'M'
 
 def _populatesubrepos(roote, repo, nodeop, statusfilter, match):
+    # type: (_Entry, localrepo.localrepository, Any, Text, matchmod.basematcher) -> None
+    # TODO: replace 'Any' with structural typing for _treenodeop/_listnodeop
     ctx = roote.ctx
     pctx = roote.pctx
     subpaths = set(pctx.substate)
@@ -656,9 +751,15 @@ def _populatesubrepos(roote, repo, nodeop, statusfilter, match):
             and os.path.isdir(repo.wjoin(path))):
             smatch = matchmod.subdirmatcher(path, match)
             try:
-                srepo = ctx.sub(path)._repo
-                e.ctx = scmutil.revsymbol(srepo, substate[1])
-                e.pctx = scmutil.revsymbol(srepo, psubstate[1] or b'null')
+                # pytype: disable=attribute-error
+                srepo = ctx.sub(path)._repo  # type: localrepo.localrepository
+                # pytype: enable=attribute-error
+                e.ctx = hglib.revsymbol(srepo, substate[1])
+
+                # This will always be a changectx, so the type warning is bogus.
+                # pytype: disable=annotation-type-mismatch
+                e.pctx = hglib.revsymbol(srepo, psubstate[1] or b'null')
+                # pytype: enable=annotation-type-mismatch
                 _populaterepo(e, srepo, nodeop, statusfilter, smatch)
             except (error.RepoError, EnvironmentError):
                 pass
@@ -669,8 +770,10 @@ def _populatesubrepos(roote, repo, nodeop, statusfilter, match):
             nodeop.putpath(roote, hglib.tounicode(path), e)
 
 def _populatepatch(roote, repo, nodeop, statusfilter, match):
+    # type: (_Entry, localrepo.localrepository, Any, Text, matchmod.basematcher) -> None
+    # TODO: replace 'Any' with structural typing for _treenodeop/_listnodeop
     ctx = roote.ctx
-    stat = ctx.changesToParent(0)
+    stat = ctx.changesToParent(0)  # pytype: disable=attribute-error
     for st, files in zip('MAR', stat):
         if st not in statusfilter:
             continue
@@ -685,6 +788,7 @@ class ManifestCompleter(QCompleter):
     """QCompleter for ManifestModel"""
 
     def splitPath(self, path):
+        # type: (Text) -> List[Text]
         """
         >>> c = ManifestCompleter()
         >>> c.splitPath(u'foo/bar')
@@ -698,6 +802,7 @@ class ManifestCompleter(QCompleter):
         return pycompat.unicode(path).split('/')
 
     def pathFromIndex(self, index):
+        # type: (QModelIndex) -> Text
         if not index.isValid():
             return ''
         m = self.model()
