@@ -4,7 +4,7 @@
 # Copyright 2011 Steve Borho <steve@borho.org>
 #
 # This software may be used and distributed according to the terms of the
-# GNU General Public License version 2, incorporated herein by reference.
+# GNU General Public License version 2 or any later version.
 
 from __future__ import absolute_import
 
@@ -608,8 +608,7 @@ class CommitPage(BasePage):
         # username will be prompted as necessary by hg if ui.askusername
         user = self.opts.get('user')
         if not self.repo.ui.configbool(b'ui', b'askusername'):
-            user = hglib.tounicode(qtlib.getCurrentUsername(self, self.repo,
-                                                            self.opts))
+            user = qtlib.getCurrentUsername(self, self.repo, self.opts)
             if not user:
                 return False
 
