@@ -48,7 +48,6 @@ from .qtgui import (
 from mercurial import (
     error,
     pycompat,
-    scmutil,
 )
 
 from ..util import hglib
@@ -547,14 +546,15 @@ class PatchQueueModel(QAbstractListModel):
             if patch in repo.thgmqunappliedpatches:
                 ctx = repo[patch]
             else:
-                ctx = scmutil.revsymbol(repo, patch)
+                ctx = hglib.revsymbol(repo, patch)
         except error.RepoLookupError:
             # cache not updated after qdelete or qfinish
             return
         guards = self.patchGuards(index)
+        # longsummary() is on the thgrepo wrapped context
         return '%s: %s\n%s' % (self.patchName(index),
                                guards and ', '.join(guards) or _('no guards'),
-                               ctx.longsummary())
+                               ctx.longsummary())  # pytype: disable=attribute-error
 
     def topAppliedIndex(self, column=0):
         """Index of the last applied, i.e. qtip, patch"""
@@ -605,8 +605,9 @@ class PatchQueueModel(QAbstractListModel):
 class MQPatchesWidget(QDockWidget):
     patchSelected = pyqtSignal(str)
 
-    def __init__(self, parent):
+    def __init__(self, actionregistry, parent):
         QDockWidget.__init__(self, parent)
+        self._actionregistry = actionregistry
         self._repoagent = None
 
         self.setFeatures(QDockWidget.DockWidgetClosable |
@@ -630,39 +631,43 @@ class MQPatchesWidget(QDockWidget):
         mainlayout.addWidget(w)
 
         # TODO: move QAction instances to PatchQueueActions
-        self._qpushAct = a = QAction(
-            qtlib.geticon('hg-qpush'), _('Push', 'MQ QPush'), self)
+        self._qpushAct = a = QAction(qtlib.geticon('hg-qpush'), '', self)
         a.setToolTip(_('Apply one patch'))
-        self._qpushAllAct = a = QAction(
-            qtlib.geticon('hg-qpush-all'), _('Push all', 'MQ QPush'), self)
+        self._qpushAllAct = a = QAction(qtlib.geticon('hg-qpush-all'), '', self)
         a.setToolTip(_('Apply all patches'))
-        self._qpopAct = a = QAction(
-            qtlib.geticon('hg-qpop'), _('Pop'), self)
+        self._qpopAct = a = QAction(qtlib.geticon('hg-qpop'), '', self)
         a.setToolTip(_('Unapply one patch'))
-        self._qpopAllAct = a = QAction(
-            qtlib.geticon('hg-qpop-all'), _('Pop all'), self)
+        self._qpopAllAct = a = QAction(qtlib.geticon('hg-qpop-all'), '', self)
         a.setToolTip(_('Unapply all patches'))
-        self._qgotoAct = QAction(
-            qtlib.geticon('hg-qgoto'), _('Go &to Patch'), self)
-        self._qpushMoveAct = a = QAction(
-            qtlib.geticon('hg-qpush-move'), _('&Apply Only This Patch'), self)
+        self._qgotoAct = QAction(qtlib.geticon('hg-qgoto'), '', self)
+        self._qpushMoveAct = a = QAction(qtlib.geticon('hg-qpush-move'), '',
+                                         self)
         a.setToolTip(_('Apply only the selected patch'))
-        qtlib.setContextMenuShortcut(a, 'Ctrl+Return')
-        a.setShortcutContext(Qt.WidgetWithChildrenShortcut)
-        self._qfinishAct = a = QAction(
-            qtlib.geticon('qfinish'), _('&Finish Patch'), self)
+        self._qfinishAct = a = QAction(qtlib.geticon('qfinish'), '', self)
         a.setToolTip(_('Move applied patches into repository history'))
-        self._qdeleteAct = a = QAction(
-            qtlib.geticon('hg-qdelete'), _('&Delete Patches...'), self)
-        qtlib.setContextMenuShortcut(a, 'Del')
-        a.setShortcutContext(Qt.WidgetWithChildrenShortcut)
+        self._qdeleteAct = a = QAction(qtlib.geticon('hg-qdelete'), '', self)
         a.setToolTip(_('Delete selected patches'))
-        self._qrenameAct = a = QAction(_('Re&name Patch...'), self)
-        qtlib.setContextMenuShortcut(a, 'F2')
-        a.setShortcutContext(Qt.WidgetWithChildrenShortcut)
-        self._setGuardsAct = a = QAction(
-            qtlib.geticon('hg-qguard'), _('Set &Guards...'), self)
+        self._qrenameAct = a = QAction(self)
+        self._setGuardsAct = a = QAction(qtlib.geticon('hg-qguard'), '', self)
         a.setToolTip(_('Configure guards for selected patch'))
+
+        namedActions = {
+            'PatchQueue.pushOnePatch': self._qpushAct,
+            'PatchQueue.pushAllPatches': self._qpushAllAct,
+            'PatchQueue.popOnePatch': self._qpopAct,
+            'PatchQueue.popAllPatches': self._qpopAllAct,
+            'PatchQueue.goToPatch': self._qgotoAct,
+            'PatchQueue.pushMovePatch': self._qpushMoveAct,
+            'PatchQueue.finishRevision': self._qfinishAct,
+            'PatchQueue.deletePatches': self._qdeleteAct,
+            'PatchQueue.renamePatch': self._qrenameAct,
+            'PatchQueue.guardPatch': self._setGuardsAct,
+        }
+        for n, a in namedActions.items():
+            self.addAction(a)
+            a.setShortcutContext(Qt.WidgetWithChildrenShortcut)
+            self._actionregistry.registerAction(n, a)
+
         tbar = QToolBar(_('Patch Queue Actions Toolbar'), self)
         tbar.setIconSize(qtlib.smallIconSize())
         tbarhbox.addWidget(tbar)
@@ -673,13 +678,10 @@ class MQPatchesWidget(QDockWidget):
         tbar.addAction(self._qpopAllAct)
         tbar.addSeparator()
         tbar.addAction(self._qpushMoveAct)
-        self.addAction(self._qpushMoveAct)
         tbar.addSeparator()
         tbar.addAction(self._qfinishAct)
         tbar.addAction(self._qdeleteAct)
-        self.addAction(self._qdeleteAct)
         tbar.addSeparator()
-        self.addAction(self._qrenameAct)
         tbar.addAction(self._setGuardsAct)
 
         self._queueFrame = w = QFrame()
@@ -891,7 +893,11 @@ class MQPatchesWidget(QDockWidget):
 
     @pyqtSlot()
     def reload(self):
-        self.widget().setEnabled(bool(self._repoagent))
+        # Repository mutation should be disabled while incoming bundle is
+        # applied. Some MQ operation could be allowed, but let's simply
+        # disable the widget at all.
+        self.widget().setEnabled(bool(self._repoagent)
+                                 and not self._repoagent.overlayUrl())
         if not self._repoagent:
             return
 
