@@ -3,7 +3,7 @@
 # Copyright 2010 Steve Borho <steve@borho.org>
 #
 # This software may be used and distributed according to the terms of the
-# GNU General Public License version 2, incorporated herein by reference.
+# GNU General Public License version 2 or any later version.
 
 from __future__ import absolute_import
 
@@ -83,17 +83,17 @@ class PerforcePending(QDialog):
         try:
             hashes = self.pending[curcl]
             revs = [repo[hash] for hash in hashes]
-        except (error.Abort, error.RepoLookupError), e:
+        except (error.Abort, error.RepoLookupError) as e:
             revs = []
         self.cslist.clear()
         self.cslist.update(revs)
-        sensitive = not curcl.endswith('(submitted)')
+        sensitive = not curcl.endswith(b'(submitted)')
         self.bb.button(QDialogButtonBox.Ok).setEnabled(sensitive)
         self.bb.button(QDialogButtonBox.Discard).setEnabled(sensitive)
         self.curcl = curcl
 
     def submit(self):
-        assert(self.curcl.endswith('(pending)'))
+        assert(self.curcl.endswith(b'(pending)'))
         cmdline = ['p4submit', '--verbose',
                    '--config', 'extensions.perfarce=',
                    '--repository', hglib.tounicode(self.url),
@@ -105,7 +105,7 @@ class PerforcePending(QDialog):
         sess.commandFinished.connect(self.commandFinished)
 
     def revert(self):
-        assert(self.curcl.endswith('(pending)'))
+        assert(self.curcl.endswith(b'(pending)'))
         cmdline = ['p4revert', '--verbose',
                    '--config', 'extensions.perfarce=',
                    '--repository', hglib.tounicode(self.url),
