@@ -3,7 +3,7 @@
 # Copyright 2018 Matt Harbison <mharbison72@gmail.com>
 #
 # This software may be used and distributed according to the terms of the
-# GNU General Public License version 2, incorporated herein by reference.
+# GNU General Public License version 2 or any later version.
 
 from __future__ import absolute_import
 
@@ -40,30 +40,56 @@ from . import (
 from .hgemail import _ChangesetsModel
 from .phabreview_ui import Ui_PhabReviewDialog
 
+if hglib.TYPE_CHECKING:
+    from typing import (
+        Any,
+        Dict,
+        List,
+        Optional,
+        Sequence,
+        Text,
+    )
+    from mercurial import (
+        localrepo,
+        ui as uimod,
+    )
+    from .qtgui import (
+        QWidget,
+    )
+    from .thgrepo import (
+        RepoAgent,
+    )
+
+
 class user(tuple):
     '''Named tuple with user properties.'''
 
     __slots__ = ()
 
     def __new__(cls, username, realname, roles):
+        # type: (Any, Text, Text, List[Text]) -> user
         return tuple.__new__(cls, (username, realname, roles))
 
     @property
     def username(self):
+        # type: () -> Text
         '''username used by phabsend'''
         return self[0]
 
     @property
     def realname(self):
+        # type: () -> Text
         '''friendly name of this user'''
         return self[1]
 
     @property
     def roles(self):
+        # type: () -> List[Text]
         '''the roles filled by this user'''
         return self[2]
 
     def __repr__(self):
+        # type: () -> Text
         return '%s (%s)' % (self.realname, self.username)
 
 
@@ -71,6 +97,7 @@ class PhabReviewDialog(QDialog):
     """Dialog for posting patches to Phabricator"""
 
     def __init__(self, repoagent, revs, parent=None):
+        # type: (RepoAgent, Sequence[int], Optional[QWidget]) -> None
         """Create PhabReviewDialog for the given repo and revs
 
         :revs: List of revisions to be sent.
@@ -133,14 +160,17 @@ class PhabReviewDialog(QDialog):
         super(PhabReviewDialog, self).closeEvent(event)
 
     def _readsettings(self):
+        # type: () -> None
         s = QSettings()
         self.restoreGeometry(qtlib.readByteArray(s, 'phabsend/geom'))
 
     def _writesettings(self):
+        # type: () -> None
         s = QSettings()
         s.setValue('phabsend/geom', self.saveGeometry())
 
     def _reviewerhistorypath(self, withcallsign=False):
+        # type: (bool) -> Optional[Text]
         '''Fetches the path in the settings used to store reviewer history.
 
         If no reviewers are stored or the configuration isn't present to find
@@ -165,6 +195,7 @@ class PhabReviewDialog(QDialog):
         return 'phabsend/%s/%s/reviewers' % (hostpath, callsign)
 
     def _getreviewerhistory(self):
+        # type: () -> List[user]
         '''Returns a fully populated list of the users previously selected to
         perform a review for any repository on the configured server.
         '''
@@ -182,12 +213,14 @@ class PhabReviewDialog(QDialog):
             realname = qtlib.readString(s, "realname")
             username = qtlib.readString(s, "username")
             roles = qtlib.readStringList(s, "roles")
-            reviewers.append(user(username, realname, roles))
+            # https://github.com/google/pytype/issues/500
+            reviewers.append(user(username, realname, roles))  # pytype: disable=wrong-arg-count
         s.endArray()
 
         return reviewers
 
     def _readreviewerhistory(self):
+        # type: () -> None
         '''Pre-populates the reviewer lists.
 
         The available reviewer list is populated with all reviewers selected
@@ -223,6 +256,7 @@ class PhabReviewDialog(QDialog):
             s.endArray()
 
     def _writereviewerhistory(self):
+        # type: () -> None
         '''Stores the selected reviewers persistently.
 
         The currently selected reviewers are combined with the existing reviewer
@@ -233,7 +267,7 @@ class PhabReviewDialog(QDialog):
         '''
         path = self._reviewerhistorypath()
         if not path:
-            return []
+            return
 
         # Avoid set() when eliminating duplicates, which hashes the role list
         reviewers = {r.username: r for r in self._getreviewerhistory()}
@@ -267,6 +301,8 @@ class PhabReviewDialog(QDialog):
             s.endArray()
 
     def _initchangesets(self, revs):
+        # type: (Sequence[int]) -> None
+
         # Posting a review amends the commits, making them hidden.  That in
         # turn would raise a FilteredRepoLookupError each time the table tries
         # to repaint after posting the review.  And that makes it almost
@@ -280,18 +316,22 @@ class PhabReviewDialog(QDialog):
 
     @property
     def _repo(self):
+        # type: () -> localrepo.localrepository
         return self._repoagent.rawRepo()
 
     @property
     def _ui(self):
+        # type: () -> uimod.ui
         return self._repo.ui
 
     @property
     def _revs(self):
+        # type: () -> List[int]
         """Returns list of revisions to be sent"""
         return self._changesets.selectedrevs
 
     def _phabsendopts(self, **opts):
+        # type: (...) -> Dict[Text, Any]
         """Generate opts for phabsend by form values"""
         opts['rev'] = hglib.compactrevs(self._revs)
 
@@ -302,6 +342,7 @@ class PhabReviewDialog(QDialog):
         return opts
 
     def _isvalid(self):
+        # type: () -> bool
         """Filled all required values?"""
         if not self._revs:
             return False
@@ -310,12 +351,14 @@ class PhabReviewDialog(QDialog):
 
     @pyqtSlot()
     def _updateforms(self):
+        # type: () -> None
         """Update availability of form widgets"""
         valid = self._isvalid()
         self._qui.send_button.setEnabled(valid)
         self._qui.main_tabs.setTabEnabled(self._previewtabindex(), valid)
 
     def accept(self):
+        # type: () -> None
         opts = self._phabsendopts()
         cmdline = hglib.buildcmdargs('phabsend', **opts)
         cmd = cmdui.CmdSessionDialog(self)
@@ -326,6 +369,7 @@ class PhabReviewDialog(QDialog):
             self._writereviewerhistory()
 
     def _initpreviewtab(self):
+        # type: () -> None
         def initqsci(w):
             w.setUtf8(True)
             w.setReadOnly(True)
@@ -348,6 +392,7 @@ class PhabReviewDialog(QDialog):
 
     @pyqtSlot(int)
     def _refreshpreviewtab(self, index):
+        # type: (int) -> None
         """Generate preview text if current tab is preview"""
         if self._previewtabindex() != index:
             return
@@ -362,6 +407,7 @@ class PhabReviewDialog(QDialog):
 
     @pyqtSlot()
     def _updatepreview(self):
+        # type: () -> None
         preview = self._qui.preview_edit
         exported = hglib.tounicode(bytes(self._cmdsession.readAll()))
 
@@ -390,6 +436,7 @@ class PhabReviewDialog(QDialog):
         preview.append(exported)
 
     def _previewtabindex(self):
+        # type: () -> None
         """Index of preview tab"""
         return self._qui.main_tabs.indexOf(self._qui.preview_tab)
 
@@ -437,7 +484,8 @@ class PhabReviewDialog(QDialog):
             if not realname or not username or username == self._my_username:
                 continue
 
-            u = user(username, realname, roles)
+            # https://github.com/google/pytype/issues/500
+            u = user(username, realname, roles)  # pytype: disable=wrong-arg-count
             item = QStandardItem(pycompat.unicode(u))
             item.setData(u)
 
@@ -445,6 +493,7 @@ class PhabReviewDialog(QDialog):
             availablereviewers.appendRow(item)
 
     def _queryreviewers(self, after):
+        # type: (Optional[bytes]) -> None
         """Issues the command to query the users, and arranges to have
         _updateavailablereviewers() called on completion.  ``after`` is the
         cursor value from the result of the last command, or None at the start
@@ -481,6 +530,7 @@ class PhabReviewDialog(QDialog):
 
     @pyqtSlot()
     def on_rescan_button_clicked(self):
+        # type: () -> None
         availablereviewers = self.availablereviewersmodel.sourceModel()
 
         availablereviewers.clear()
@@ -494,6 +544,7 @@ class PhabReviewDialog(QDialog):
         self._queryreviewers(None)
 
     def _get_my_username(self):
+        # type: () -> None
         """Spawns the phabricator command to return the current user's info."""
         buf = QBuffer()
 
@@ -508,6 +559,7 @@ class PhabReviewDialog(QDialog):
 
     @pyqtSlot()
     def _update_self_username(self):
+        # type: () -> None
         """Handles the completion of the current user info fetch."""
         exitcode = self._namesession.exitCode()
 
@@ -519,7 +571,7 @@ class PhabReviewDialog(QDialog):
 
     @pyqtSlot()
     def _updateavailablereviewers(self):
-
+        # type: () -> None
         exitcode = self._rescansession.exitCode()
 
         if exitcode == 0:
@@ -548,16 +600,19 @@ class PhabReviewDialog(QDialog):
 
     @pyqtSlot()
     def on_available_reviewer_selection_changed(self):
+        # type: () -> None
         view = self._qui.available_reviewer_list
         self._qui.addreviewer_button.setEnabled(bool(view.selectedIndexes()))
 
     @pyqtSlot()
     def on_reviewer_selection_changed(self):
+        # type: () -> None
         view = self._qui.selected_reviewers_list
         self._qui.removereviewer_button.setEnabled(bool(view.selectedIndexes()))
 
     @pyqtSlot()
     def on_addreviewer_button_clicked(self):
+        # type: () -> None
         """Populates the selected reviewers list when the ">" button is clicked.
         """
         reviewers = self._qui.selected_reviewers_list
@@ -573,6 +628,7 @@ class PhabReviewDialog(QDialog):
 
     @pyqtSlot()
     def on_removereviewer_button_clicked(self):
+        # type: () -> None
         """Removes items from the selected reviewers list when the "<" button is
         clicked.
         """
@@ -582,8 +638,10 @@ class PhabReviewDialog(QDialog):
 
     @pyqtSlot()
     def on_selectall_button_clicked(self):
+        # type: () -> None
         self._changesets.selectAll()
 
     @pyqtSlot()
     def on_selectnone_button_clicked(self):
+        # type: () -> None
         self._changesets.selectNone()
