@@ -3,7 +3,7 @@
 # Copyright 2011 Michael De Wildt <michael.dewildt@gmail.com>
 #
 # This software may be used and distributed according to the terms of the
-# GNU General Public License version 2, incorporated herein by reference.
+# GNU General Public License version 2 or any later version.
 
 """A dialog to allow users to post a review to reviewboard
 
@@ -49,26 +49,49 @@ from . import (
 from .hgemail import _ChangesetsModel
 from .postreview_ui import Ui_PostReviewDialog
 
+if hglib.TYPE_CHECKING:
+    from typing import (
+        Dict,
+        List,
+        Optional,
+        Sequence,
+        Text,
+        Union,
+    )
+    from mercurial import (
+        ui as uimod,
+    )
+    from .qtgui import (
+        QCloseEvent,
+        QWidget,
+    )
+    from .thgrepo import (
+        RepoAgent,
+    )
+
+
 class LoadReviewDataThread(QThread):
     def __init__ (self, dialog):
+        # type: ("PostReviewDialog") -> None
         super(LoadReviewDataThread, self).__init__(dialog)
         self.dialog = dialog
 
     def run(self):
+        # type: () -> None
         msg = None
         if not self.dialog.server:
             msg = _("Invalid Settings - The ReviewBoard server is not setup")
         elif not self.dialog.user:
             msg = _("Invalid Settings - Please provide your ReviewBoard username")
         else:
-            rb = extensions.find("reviewboard")
+            rb = extensions.find(b"reviewboard")
             try:
                 pwd = self.dialog.password
                 #if we don't have a password send something here to skip
                 #the cli getpass in the extension. We will set the password
                 #later
                 if not pwd:
-                    pwd = "None"
+                    pwd = b"None"
 
                 self.reviewboard = rb.make_rbclient(self.dialog.server,
                                                     self.dialog.user,
@@ -86,6 +109,7 @@ class LoadReviewDataThread(QThread):
         self.dialog.error_message = msg
 
     def loadCombos(self):
+        # type: () -> None
         #Get the index of a users previously selected repo id
         index = 0
         count = 0
@@ -111,6 +135,7 @@ class LoadReviewDataThread(QThread):
 class PostReviewDialog(QDialog):
     """Dialog for sending patches to reviewboard"""
     def __init__(self, ui, repoagent, revs, parent=None):
+        # type: (uimod.ui, RepoAgent, Sequence[Union[bytes, int]], Optional[QWidget]) -> None
         super(PostReviewDialog, self).__init__(parent)
         self.setWindowFlags(self.windowFlags() & ~Qt.WindowContextHelpButtonHint)
         self.ui = ui
@@ -137,12 +162,13 @@ class PostReviewDialog(QDialog):
 
     @pyqtSlot()
     def passwordPrompt(self):
+        # type: () -> bool
         pwd, ok = qtlib.getTextInput(self,
                                      _('Review Board'),
                                      _('Password:'),
                                      mode=QLineEdit.Password)
         if ok and pwd:
-            self.password = pwd
+            self.password = hglib.fromunicode(pwd)
             return True
         else:
             self.password = None
@@ -150,6 +176,7 @@ class PostReviewDialog(QDialog):
 
     @pyqtSlot()
     def errorPrompt(self):
+        # type: () -> None
         self.qui.progress_bar.hide()
         self.qui.progress_label.hide()
 
@@ -161,6 +188,7 @@ class PostReviewDialog(QDialog):
             self.qui.post_review_button.setEnabled(True)
 
     def closeEvent(self, event):
+        # type: (QCloseEvent) -> None
         if not self._cmdsession.isFinished():
             self._cmdsession.abort()
             event.ignore()
@@ -174,6 +202,7 @@ class PostReviewDialog(QDialog):
         super(PostReviewDialog, self).closeEvent(event)
 
     def readSettings(self):
+        # type: () -> None
         s = QSettings()
 
         self.restoreGeometry(qtlib.readByteArray(s, 'reviewboard/geom'))
@@ -203,6 +232,7 @@ class PostReviewDialog(QDialog):
         self.browser = self.repo.ui.config(b'reviewboard', b'browser')
 
     def writeSettings(self):
+        # type: () -> None
         s = QSettings()
         s.setValue('reviewboard/geom', self.saveGeometry())
         s.setValue('reviewboard/publish_immediately_check',
@@ -226,8 +256,10 @@ class PostReviewDialog(QDialog):
                    list(itercombo(self.qui.summary_edit))[:10])
 
     def initChangesets(self, revs, selected_revs=None):
+        # type: (Sequence[Union[bytes, int]], Optional[Sequence[int]]) -> None
         def purerevs(revs):
-            return scmutil.revrange(self.repo, iter(str(e) for e in revs))
+            # type: (Sequence[Union[bytes, int]]) -> Sequence[int]
+            return [r for r in scmutil.revrange(self.repo, revs)]
         if selected_revs:
             selectedrevs = purerevs(selected_revs)
         else:
@@ -235,7 +267,7 @@ class PostReviewDialog(QDialog):
 
         self._changesets = _ChangesetsModel(self.repo,
                                             # TODO: [':'] is inefficient
-                                            revs=purerevs(revs or [':']),
+                                            revs=purerevs(revs or [b':']),
                                             selectedrevs=selectedrevs,
                                             parent=self)
 
@@ -243,27 +275,33 @@ class PostReviewDialog(QDialog):
 
     @property
     def selectedRevs(self):
+        # type: () -> List[int]
         """Returns list of revisions to be sent"""
         return self._changesets.selectedrevs
 
     @property
     def allRevs(self):
+        # type: () -> List[int]
         """Returns list of revisions to be sent"""
         return self._changesets.revs
 
     def getRepoId(self):
+        # type: () -> Text
         comboText = self.qui.repo_id_combo.currentText().split(":")
         return str(comboText[0])
 
     def getReviewId(self):
+        # type: () -> Text
         comboText = self.qui.review_id_combo.currentText().split(":")
         return str(comboText[0])
 
     def getSummary(self):
+        # type: () -> Text
         comboText = self.qui.review_id_combo.currentText().split(":")
-        return hglib.fromunicode(comboText[1])
+        return comboText[1]
 
     def postReviewOpts(self, **opts):
+        # type: (...) -> Dict[Text, Union[bool, Text]]
         """Generate opts for reviewboard by form values"""
         opts['outgoingchanges'] = self.qui.outgoing_changes_check.isChecked()
         opts['branch'] = self.qui.branch_check.isChecked()
@@ -275,7 +313,7 @@ class PostReviewDialog(QDialog):
             opts['summary'] = self.getSummary()
         else:
             opts['repoid'] = self.getRepoId()
-            opts['summary'] = hglib.fromunicode(self.qui.summary_edit.currentText())
+            opts['summary'] = self.qui.summary_edit.currentText()
 
         if len(self.selectedRevs) > 1:
             #Set the parent to the revision below the last one on the list
@@ -288,11 +326,12 @@ class PostReviewDialog(QDialog):
         opts['outgoing'] = True
 
         #Set the password just in  case the user has opted to not save it
-        opts['password'] = str(self.password)
+        opts['password'] = hglib.tounicode(self.password)
 
         return opts
 
     def isValid(self):
+        # type: () -> bool
         """Filled all required values?"""
         if not self.qui.repo_id_combo.currentText():
             return False
@@ -308,10 +347,12 @@ class PostReviewDialog(QDialog):
 
     @pyqtSlot()
     def tabChanged(self):
+        # type: () -> None
         self.qui.post_review_button.setEnabled(self.isValid())
 
     @pyqtSlot()
     def branchCheckToggle(self):
+        # type: () -> None
         if self.qui.branch_check.isChecked():
             self.qui.outgoing_changes_check.setChecked(False)
 
@@ -319,12 +360,14 @@ class PostReviewDialog(QDialog):
 
     @pyqtSlot()
     def outgoingChangesCheckToggle(self):
+        # type: () -> None
         if self.qui.outgoing_changes_check.isChecked():
             self.qui.branch_check.setChecked(False)
 
         self.toggleOutgoingChangesets()
 
     def toggleOutgoingChangesets(self):
+        # type: () -> None
         branch = self.qui.branch_check.isChecked()
         outgoing = self.qui.outgoing_changes_check.isChecked()
         if branch or outgoing:
@@ -335,9 +378,11 @@ class PostReviewDialog(QDialog):
             self.qui.changesets_view.setEnabled(True)
 
     def close(self):
+        # type: () -> None
         super(PostReviewDialog, self).close()
 
     def accept(self):
+        # type: () -> None
         if not self.isValid():
             return
         if not self.password and not self.passwordPrompt():
@@ -348,6 +393,7 @@ class PostReviewDialog(QDialog):
         self.qui.progress_label.show()
 
         def cmdargs(opts):
+            # type: (Dict[Text, Union[bool, Text]]) -> List[Text]
             args = []
             for k, v in opts.items():
                 if isinstance(v, bool):
@@ -366,8 +412,7 @@ class PostReviewDialog(QDialog):
         self.qui.post_review_button.setEnabled(False)
         self.qui.close_button.setEnabled(False)
 
-        cmdline = pycompat.maplist(hglib.tounicode,
-                                   ['postreview'] + cmdargs(opts) + [revstr])
+        cmdline = ['postreview'] + cmdargs(opts) + [revstr]
         self._cmdsession = sess = self._repoagent.runCommand(cmdline, self)
         del self._cmdoutputs[:]
         sess.commandFinished.connect(self.onCompletion)
@@ -375,6 +420,7 @@ class PostReviewDialog(QDialog):
 
     @pyqtSlot()
     def onCompletion(self):
+        # type: () -> None
         self.qui.progress_bar.hide()
         self.qui.progress_label.hide()
 
@@ -413,11 +459,13 @@ class PostReviewDialog(QDialog):
 
     @pyqtSlot(str, str)
     def _captureOutput(self, msg, label):
+        # type: (Text, Text) -> None
         if label != 'control':
             self._cmdoutputs.append(pycompat.unicode(msg))
 
     @pyqtSlot()
     def onSettingsButtonClicked(self):
+        # type: () -> None
         from tortoisehg.hgqt import settings
         if settings.SettingsDialog(parent=self, focus='reviewboard.server').exec_():
             # not use repo.configChanged because it can clobber user input
