@@ -4,7 +4,7 @@
 # Copyright 2010 Johan Samyn <johan.samyn@gmail.com>
 #
 # This software may be used and distributed according to the terms of the
-# GNU General Public License version 2, incorporated herein by reference.
+# GNU General Public License version 2 or any later version.
 
 from __future__ import absolute_import
 
@@ -29,8 +29,8 @@ from .qtgui import (
 
 from hgext import mq
 from mercurial import (
+    context,
     pycompat,
-    scmutil,
 )
 
 from ..util import hglib
@@ -139,7 +139,7 @@ class QFoldDialog(QDialog):
         self.summ.setText(hglib.tounicode(txt))
 
     def composeMsg(self, patches):
-        descs = [scmutil.revsymbol(self.repo, b'qtip').description()]
+        descs = [hglib.revsymbol(self.repo, b'qtip').description()]
         # lookup of unapplied patches is handled by thgrepo hack
         descs.extend(self.repo[p].description() for p in patches)
         return u'\n* * *\n'.join(map(hglib.tounicode, descs))
