@@ -38,11 +38,9 @@ from .qtgui import (
     QCheckBox,
     QDialog,
     QDialogButtonBox,
-    QFont,
     QInputMethodEvent,
     QKeyEvent,
     QKeySequence,
-    QLabel,
     QLineEdit,
     QMenu,
     QToolBar,
@@ -59,6 +57,9 @@ _IM_PREEDIT_INDIC_ID = QsciScintilla.INDIC_MAX
 # indicator for keyword highlighting
 _HIGHLIGHT_INDIC_ID = _IM_PREEDIT_INDIC_ID - 1
 
+STYLE_FILEVIEW_MARGIN = QsciScintilla.STYLE_LASTPREDEFINED + 1
+
+
 class _SciImSupport(object):
     """Patch for QsciScintilla to implement improved input method support
 
@@ -549,13 +550,7 @@ class SearchToolBar(QToolBar):
         self.addWidget(qtlib.Spacer(2, 2))
 
         self._le = QLineEdit()
-        if hasattr(self._le, 'setPlaceholderText'): # Qt >= 4.7
-            self._le.setPlaceholderText(_('### regular expression ###'))
-        else:
-            self._lbl = QLabel(_('Regexp:'),
-                               toolTip=_('Regular expression search pattern'))
-            self.addWidget(self._lbl)
-            self._lbl.setBuddy(self._le)
+        self._le.setPlaceholderText(_('### regular expression ###'))
         self._le.returnPressed.connect(self._emitSearchRequested)
         self.addWidget(self._le)
         self.addWidget(qtlib.Spacer(4, 4))
@@ -813,7 +808,7 @@ def writeFile(editor, filename, encoding=None):
     return True
 
 def fileEditor(filename, **opts):
-    'Open a simple modal file editing dialog'
+    """Open a simple modal file editing dialog"""
     dialog = QDialog()
     dialog.setWindowFlags(dialog.windowFlags()
                           & ~Qt.WindowContextHelpButtonHint
@@ -827,7 +822,7 @@ def fileEditor(filename, **opts):
     editor.setMarginWidth(1, '000')
 
     lexer = QsciLexerProperties()
-    lexer.setFont(QFont('Monospace', 10), -1)
+    lexer.setFont(qtlib.getfont('fonteditor').font(), -1)
 
     editor.setLexer(lexer)
 
