@@ -4,7 +4,7 @@
 # Copyright 2008 TK Soh <teekaysoh@gmail.com>
 #
 # This software may be used and distributed according to the terms of the
-# GNU General Public License version 2, incorporated herein by reference.
+# GNU General Public License version 2 or any later version.
 
 from __future__ import absolute_import
 
@@ -63,6 +63,7 @@ from . import (
     bugreport,
     hgconfig,
     qtlib,
+    shortcutregistry,
     thgrepo,
     workbench,
 )
@@ -74,8 +75,8 @@ if os.name == 'nt' and getattr(sys, 'frozen', False):
     _mod = __import__(QT_API, globals(), locals(), ['QtSvg', 'QtXml'])
     _mod.QtSvg.__name__, _mod.QtXml.__name__  # no demandimport
 
-if PYQT_VERSION < 0x40705 or QT_VERSION < 0x40600:
-    sys.stderr.write('TortoiseHg requires at least Qt 4.6 and PyQt 4.7.5\n')
+if PYQT_VERSION < 0x50000 or QT_VERSION < 0x50000:
+    sys.stderr.write('TortoiseHg requires at least Qt 5.0 and PyQt 5.0\n')
     sys.stderr.write('You have Qt %s and PyQt %s\n' %
                      (QT_VERSION_STR, PYQT_VERSION_STR))
     sys.exit(-1)
@@ -236,7 +237,7 @@ class ExceptionCatcher(QObject):
 
         def _releaseWakeup(self):
             self._wakeupsn.setEnabled(False)
-            rfd = self._wakeupsn.socket()
+            rfd = int(self._wakeupsn.socket())
             wfd = signal.set_wakeup_fd(self._origwakeupfd)
             self._origwakeupfd = -1
             os.close(rfd)
@@ -246,7 +247,7 @@ class ExceptionCatcher(QObject):
         def _handleWakeup(self):
             # here Python signal handler will be invoked
             self._wakeupsn.setEnabled(False)
-            rfd = self._wakeupsn.socket()
+            rfd = int(self._wakeupsn.socket())
             try:
                 os.read(rfd, 1)
             except OSError as inst:
@@ -406,6 +407,7 @@ class QtRunner(QObject):
         self._config = None
         self._mainapp = None
         self._exccatcher = None
+        self._actionregistry = None
         self._server = None
         self._repomanager = None
         self._reporeleaser = None
@@ -419,24 +421,13 @@ class QtRunner(QObject):
 
         QSettings.setDefaultFormat(QSettings.IniFormat)
 
-        # fixes font placement on OSX 10.9 with QT <= 4.8.5
-        # see QTBUG-32789 (https://bugreports.qt-project.org/browse/QTBUG-32789)
-        if sys.platform == 'darwin' and QT_VERSION <= 0x040805:
-            version = platform.mac_ver()[0]
-            version = '.'.join(version.split('.')[:2])
-            if version == '10.9':
-                # needs to replace the font created in the constructor of
-                # QApplication, which is invalid use of QFont but works on Mac
-                QFont.insertSubstitution('.Lucida Grande UI', 'Lucida Grande')
-
         self._ui = ui
         self._config = hgconfig.HgConfig(ui)
         self._mainapp = QApplication(sys.argv)
 
-        if QT_VERSION >= 0x50000:
-            self._mainapp.setAttribute(Qt.AA_UseHighDpiPixmaps, True)
-            if sys.platform == 'darwin':
-                self._mainapp.setAttribute(Qt.AA_DontShowIconsInMenus, True)
+        self._mainapp.setAttribute(Qt.AA_UseHighDpiPixmaps, True)
+        if sys.platform == 'darwin':
+            self._mainapp.setAttribute(Qt.AA_DontShowIconsInMenus, True)
 
         self._exccatcher = ExceptionCatcher(ui, self._mainapp, self)
         self._gc = GarbageCollector(ui, self)
@@ -446,6 +437,8 @@ class QtRunner(QObject):
         self._mainapp.setOrganizationName('TortoiseHg')
         self._mainapp.setOrganizationDomain('tortoisehg.org')
         self._mainapp.setApplicationVersion(thgversion.version())
+        if hasattr(self._mainapp, 'setDesktopFileName'):  # introduced in Qt 5.7
+            self._mainapp.setDesktopFileName('thg.desktop')
         self._fixlibrarypaths()
         self._installtranslator()
         QFont.insertSubstitutions('monospace', ['monaco', 'courier new'])
@@ -454,6 +447,9 @@ class QtRunner(QObject):
         qtlib.initfontcache(ui)
         self._mainapp.setWindowIcon(qtlib.geticon('thg'))
 
+        self._actionregistry = shortcutregistry.ActionRegistry()
+        self._actionregistry.readSettings()
+
         self._repomanager = thgrepo.RepoManager(ui, self)
         self._reporeleaser = releaser = QSignalMapper(self)
         releaser.mapped[pycompat.unicode].connect(self._repomanager.releaseRepoAgent)
@@ -555,12 +551,18 @@ class QtRunner(QObject):
             dlg.setParent(self._workbench, dlg.windowFlags())
         dlg.show()
 
+    def actionRegistry(self):
+        # type: () -> shortcutregistry.ActionRegistry
+        assert self._actionregistry
+        return self._actionregistry
+
     def createWorkbench(self):
         """Create Workbench window and keep single reference"""
         assert self._ui and self._config and self._mainapp and self._repomanager
+        assert self._actionregistry
         assert not self._workbench
         self._workbench = workbench.Workbench(
-            self._ui, self._config, self._repomanager)
+            self._ui, self._config, self._actionregistry, self._repomanager)
         return self._workbench
 
     @pyqtSlot(str)
