@@ -692,7 +692,7 @@ def geticon(name):
     named as 'name.(svg|png|ico)'.
     """
     try:
-        return _iconcache[name]  # pytype: disable=key-error
+        return _iconcache[name]
     except KeyError:
         _iconcache[name] = (_findthemeicon(name)
                             or _findscalableicon(name)
@@ -797,6 +797,7 @@ class ThgFont(QObject):
 _fontdefaults = {
     'fontcomment': 'monospace,10',
     'fontdiff': 'monospace,10',
+    'fonteditor': 'monospace,10',
     'fontlog': 'monospace,10',
     'fontoutputlog': 'sans,8'
 }
