@@ -3,12 +3,10 @@
 # Copyright 2010 Steve Borho <steve@borho.org>
 #
 # This software may be used and distributed according to the terms of the
-# GNU General Public License version 2, incorporated herein by reference.
+# GNU General Public License version 2 or any later version.
 
 from __future__ import absolute_import
 
-import os
-
 from .qtcore import (
     QSettings,
     QTimer,
@@ -39,6 +37,21 @@ from . import (
     wctxcleaner,
 )
 
+if hglib.TYPE_CHECKING:
+    from typing import (
+        Any,
+        List,
+        Optional,
+        Text,
+    )
+    from .qtgui import (
+        QWidget,
+    )
+    from .thgrepo import (
+        RepoAgent,
+    )
+
+
 BB = QDialogButtonBox
 
 _SOURCE_SPECS = [
@@ -51,6 +64,7 @@ _SOURCE_SPECS = [
 class RebaseDialog(QDialog):
 
     def __init__(self, repoagent, parent, **opts):
+        # type: (RepoAgent, Optional[QWidget], Any) -> None
         super(RebaseDialog, self).__init__(parent)
         self.setWindowIcon(qtlib.geticon('hg-rebase'))
         self.setWindowFlags(self.windowFlags()
@@ -169,6 +183,7 @@ class RebaseDialog(QDialog):
         return self._repoagent.rawRepo()
 
     def _readSettings(self):
+        # type: () -> None
         qs = QSettings()
         qs.beginGroup('rebase')
         self.autoresolvechk.setChecked(
@@ -177,6 +192,7 @@ class RebaseDialog(QDialog):
         qs.endGroup()
 
     def _writeSettings(self):
+        # type: () -> None
         qs = QSettings()
         qs.beginGroup('rebase')
         qs.setValue('autoresolve', self.autoresolvechk.isChecked())
@@ -184,6 +200,7 @@ class RebaseDialog(QDialog):
 
     @pyqtSlot(bool)
     def _onCheckFinished(self, clean):
+        # type: (bool) -> None
         if not clean:
             self.rebasebtn.setEnabled(False)
             txt = _('Before rebase, you must '
@@ -197,10 +214,12 @@ class RebaseDialog(QDialog):
 
     @pyqtSlot()
     def _updateSourceSelector(self):
+        # type: () -> None
         _name, title, _oplabel = _SOURCE_SPECS[self._sourcegroup.checkedId()]
         self._sourcebox.setTitle(title)
 
     def rebase(self):
+        # type: () -> None
         self.rebasebtn.setEnabled(False)
         self.cancelbtn.setVisible(False)
         self.keepchk.setEnabled(False)
@@ -210,9 +229,9 @@ class RebaseDialog(QDialog):
         self.collapsechk.setEnabled(False)
         self.swaplabel.setVisible(False)
 
-        itool = self.autoresolvechk.isChecked() and 'merge' or 'fail'
+        itool = 'merge' if self.autoresolvechk.isChecked() else 'fail'
         opts = {'config': 'ui.merge=internal:%s' % itool}
-        if os.path.exists(self.repo.vfs.join(b'rebasestate')):
+        if hglib.rebase_in_progress(self.repo):
             opts['continue'] = True
         else:
             opts.update({
@@ -231,6 +250,7 @@ class RebaseDialog(QDialog):
         sess.commandFinished.connect(self._rebaseFinished)
 
     def swap(self):
+        # type: () -> None
         oldsource = self.opts.get('source', '.')
         olddest = self.opts.get('dest', '.')
 
@@ -241,11 +261,13 @@ class RebaseDialog(QDialog):
         self.opts['dest'] = oldsource
 
     def abort(self):
+        # type: () -> None
         cmdline = hglib.buildcmdargs('rebase', abort=True)
         sess = self._runCommand(cmdline)
         sess.commandFinished.connect(self._abortFinished)
 
     def _runCommand(self, cmdline):
+        # type: (List[Text]) -> cmdcore.CmdSession
         assert self._cmdsession.isFinished()
         self._cmdsession = sess = self._repoagent.runCommand(cmdline, self)
         sess.commandFinished.connect(self._stbar.clearProgress)
@@ -256,6 +278,7 @@ class RebaseDialog(QDialog):
 
     @pyqtSlot(int)
     def _rebaseFinished(self, ret):
+        # type: (int) -> None
         # TODO since hg 2.6, rebase will end with ret=1 in case of "unresolved
         # conflicts", so we can fine-tune checkResolve() later.
         if self.checkResolve() is False:
@@ -268,17 +291,20 @@ class RebaseDialog(QDialog):
 
     @pyqtSlot()
     def _abortFinished(self):
+        # type: () -> None
         if self.checkResolve() is False:
             self._stbar.showMessage(_('Rebase aborted'))
             self._makeCloseButton()
 
     def _makeCloseButton(self):
+        # type: () -> None
         self.rebasebtn.setEnabled(True)
         self.rebasebtn.setText(_('Close'))
         self.rebasebtn.clicked.disconnect(self.rebase)
         self.rebasebtn.clicked.connect(self.accept)
 
     def checkResolve(self):
+        # type: () -> bool
         for root, path, status in thgrepo.recursiveMergeStatus(self.repo):
             if status == b'u':
                 txt = _('Rebase generated merge <b>conflicts</b> that must '
@@ -290,7 +316,7 @@ class RebaseDialog(QDialog):
             txt = _('You may continue the rebase')
         self._stbar.showMessage(txt)
 
-        if os.path.exists(self.repo.vfs.join(b'rebasestate')):
+        if hglib.rebase_in_progress(self.repo):
             self.swaplabel.setVisible(False)
             self.abortbtn.setEnabled(True)
             self.rebasebtn.setText('Continue')
@@ -300,6 +326,7 @@ class RebaseDialog(QDialog):
             return False
 
     def linkActivated(self, cmd):
+        # type: (Text) -> None
         if cmd == 'resolve':
             dlg = resolve.ResolveDialog(self._repoagent, self)
             dlg.exec_()
@@ -308,7 +335,8 @@ class RebaseDialog(QDialog):
             self._wctxcleaner.runCleaner(cmd)
 
     def reject(self):
-        if os.path.exists(self.repo.vfs.join(b'rebasestate')):
+        # type: () -> None
+        if hglib.rebase_in_progress(self.repo):
             main = _('Exiting with an unfinished rebase is not recommended.')
             text = _('Consider aborting the rebase first.')
             labels = ((QMessageBox.Yes, _('&Exit')),
@@ -319,5 +347,6 @@ class RebaseDialog(QDialog):
         super(RebaseDialog, self).reject()
 
     def done(self, r):
+        # type: (int) -> None
         self._writeSettings()
         super(RebaseDialog, self).done(r)
