@@ -4,7 +4,7 @@
 # Copyright 2010 Johan Samyn <johan.samyn@gmail.com>
 #
 # This software may be used and distributed according to the terms of the
-# GNU General Public License version 2, incorporated herein by reference.
+# GNU General Public License version 2 or any later version.
 
 from __future__ import absolute_import
 
@@ -40,10 +40,20 @@ from . import (
     qtlib,
 )
 
+if hglib.TYPE_CHECKING:
+    from typing import (
+        List,
+        Optional,
+        Text,
+    )
+    from .qtgui import QWidget
+    from .thgrepo import RepoAgent
+
 class RenameWidget(cmdui.AbstractCmdWidget):
 
     def __init__(self, repoagent, parent=None, source=None, destination=None,
                  iscopy=False):
+        # type: (RepoAgent, Optional[QWidget], Optional[Text], Optional[Text], bool) -> None
         super(RenameWidget, self).__init__(parent)
         self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
         self._repoagent = repoagent
@@ -95,6 +105,7 @@ class RenameWidget(cmdui.AbstractCmdWidget):
         self.setRenameCopy()
 
     def setRenameCopy(self):
+        # type: () -> None
         if self.copy_chk.isChecked():
             self.msgTitle = _('Copy')
             self.errTitle = _('Copy Error')
@@ -107,24 +118,30 @@ class RenameWidget(cmdui.AbstractCmdWidget):
         return self._repoagent.rawRepo()
 
     def source(self):
+        # type: () -> Text
         return pycompat.unicode(self.src_txt.text())
 
     def destination(self):
+        # type: () -> Text
         return pycompat.unicode(self.dest_txt.text())
 
     def _sourceFile(self):
+        # type: () -> Text
         root = self._repoagent.rootPath()
         return os.path.normpath(os.path.join(root, self.source()))
 
     def _destinationFile(self):
+        # type: () -> Text
         root = self._repoagent.rootPath()
         return os.path.normpath(os.path.join(root, self.destination()))
 
     def src_dest_edited(self):
+        # type: () -> None
         self.show_command(self.compose_command())
         self.commandChanged.emit()
 
     def src_btn_clicked(self):
+        # type: () -> None
         """Select the source file of folder"""
         FD = QFileDialog
         if os.path.isfile(self._sourceFile()):
@@ -141,6 +158,7 @@ class RenameWidget(cmdui.AbstractCmdWidget):
         self.src_txt.setText(relpath)
 
     def dest_btn_clicked(self):
+        # type: () -> None
         """Select the destination file of folder"""
         FD = QFileDialog
         if os.path.isfile(self._sourceFile()):
@@ -153,42 +171,50 @@ class RenameWidget(cmdui.AbstractCmdWidget):
             return
         self.dest_txt.setText(relpath)
 
-    def to_relative_path(self, fullpath):  # unicode or QString
+    def to_relative_path(self, fullpath):
+        # type: (Text) -> Optional[Text]
         if not fullpath:
             return
-        fullpath = util.normpath(pycompat.unicode(fullpath))
-        pathprefix = util.normpath(hglib.tounicode(self.repo.root)) + '/'
+        fullpath = hglib.normpath(fullpath)
+        pathprefix = hglib.tounicode(util.normpath(self.repo.root)) + '/'
         if not os.path.normcase(fullpath).startswith(os.path.normcase(pathprefix)):
             return
         return fullpath[len(pathprefix):]
 
     def isCopyCommand(self):
+        # type: () -> bool
         return self.copy_chk.isChecked()
 
     def copy_chk_toggled(self):
+        # type: () -> None
         self.setRenameCopy()
         self.show_command(self.compose_command())
         self.commandChanged.emit()
 
     def isCaseFoldingOnWin(self):
+        # type: () -> bool
         fullsrc, fulldest = self._sourceFile(), self._destinationFile()
         return fullsrc.upper() == fulldest.upper() and sys.platform == 'win32'
 
     def compose_command(self):
+        # type: () -> List[Text]
         name = self.isCopyCommand() and 'copy' or 'rename'
         return hglib.buildcmdargs(name, self.source(), self.destination(),
                                   v=True, f=True)
 
     def show_command(self, cmdline):
+        # type: (List[Text]) -> None
         self.hgcmd_txt.setText('hg %s' % hglib.prettifycmdline(cmdline))
 
     def canRunCommand(self):
+        # type: () -> bool
         src, dest = self.source(), self.destination()
         return bool(src and dest and src != dest
                     and not (self.isCopyCommand()
                              and self.isCaseFoldingOnWin()))
 
     def runCommand(self):
+        # type: () -> cmdcore.CmdSession
         # check inputs
         fullsrc, fulldest = self._sourceFile(), self._destinationFile()
         if not os.path.exists(fullsrc):
@@ -219,6 +245,7 @@ class RenameDialog(cmdui.CmdControlDialog):
 
     def __init__(self, repoagent, parent=None, source=None, destination=None,
                  iscopy=False):
+        # type: (RepoAgent, Optional[QWidget], Optional[Text], Optional[Text], bool) -> None
         super(RenameDialog, self).__init__(parent)
         self._repoagent = repoagent
 
@@ -232,12 +259,14 @@ class RenameDialog(cmdui.CmdControlDialog):
 
     @pyqtSlot(int)
     def _checkKnownError(self, ret):
+        # type: (int) -> None
         if ret == 1:
             # occurs if _some_ of the files cannot be copied
             cmdui.errorMessageBox(self.lastFinishedSession(), self)
 
     @pyqtSlot()
     def _updateUi(self):
+        # type: () -> None
         if self.commandWidget().isCopyCommand():
             bt = _('Copy')
             wt = _('Copy - %s')
