@@ -3,8 +3,8 @@
 # Copyright (C) 2007-2010 Logilab. All rights reserved.
 # Copyright (C) 2010 Yuya Nishihara <yuya@tcha.org>
 #
-# This software may be used and distributed according to the terms
-# of the GNU General Public License, incorporated herein by reference.
+# This software may be used and distributed according to the terms of the
+# GNU General Public License version 2 or any later version.
 
 from __future__ import absolute_import
 
@@ -47,11 +47,31 @@ from . import (
     revset,
 )
 
+if hglib.TYPE_CHECKING:
+    from typing import (
+        List,
+        Optional,
+        Text,
+    )
+    from mercurial import (
+        localrepo,
+    )
+    from .qtcore import (
+        QSettings,
+    )
+    from .qtgui import (
+        QWidget,
+    )
+    from . import (
+        thgrepo,
+    )
+
 _permanent_queries = ('head()', 'merge()',
                       'tagged()', 'bookmark()',
                       'file(".hgsubstate") or file(".hgsub")')
 
 def _firstword(query):
+    # type: (Text) -> Optional[bytes]
     lquery = hglib.fromunicode(query)
     try:
         for token, value, _pos in hglib.tokenizerevspec(lquery):
@@ -61,6 +81,7 @@ def _firstword(query):
         pass
 
 def _querytype(repo, query):
+    # type: (localrepo.localrepository, Text) -> Optional[Text]
     r"""
     >>> # TODO: maybe replace with real repo
     >>> origisrevsymbol = scmutil.isrevsymbol
@@ -106,6 +127,7 @@ def _querytype(repo, query):
 
 class SelectAllLineEdit(QLineEdit):
     def __init__(self, parent=None):
+        # type: (Optional[QWidget]) -> None
         super(SelectAllLineEdit, self).__init__(parent)
         self.just_got_focus = False
 
@@ -136,6 +158,7 @@ class RepoFilterBar(QToolBar):
     _allBranchesLabel = u'\u2605 ' + _('Show all') + u' \u2605'
 
     def __init__(self, repoagent, parent=None):
+        # type: (thgrepo.RepoAgent, Optional[QWidget]) -> None
         super(RepoFilterBar, self).__init__(parent)
         self.layout().setContentsMargins(0, 0, 0, 0)
         self.setIconSize(qtlib.smallIconSize())
@@ -174,8 +197,7 @@ class RepoFilterBar(QToolBar):
         le = combo.lineEdit()
         le.returnPressed.connect(self.runQuery)
         le.selectionChanged.connect(self.selectionChanged)
-        if hasattr(le, 'setPlaceholderText'): # Qt >= 4.7
-            le.setPlaceholderText(_('### revision set query ###'))
+        le.setPlaceholderText(_('### revision set query ###'))
         combo.activated.connect(self.runQuery)
 
         self._revsettypelabel = QLabel(le)
@@ -269,16 +291,6 @@ class RepoFilterBar(QToolBar):
         self.revsetcombo.addItems(full)
         self.revsetcombo.setCurrentIndex(-1)
 
-    if not util.safehasattr(QToolBar, 'visibilityChanged'):
-        # Qt < 4.7
-        visibilityChanged = pyqtSignal(bool)
-
-        def event(self, event):
-            etype = event.type()
-            if etype == QEvent.Show or etype == QEvent.Hide and self.isHidden():
-                self.visibilityChanged.emit(etype == QEvent.Show)
-            return super(RepoFilterBar, self).event(event)
-
     def eventFilter(self, watched, event):
         if watched is self.revsetcombo.lineEdit():
             if event.type() == QEvent.Resize:
@@ -294,10 +306,12 @@ class RepoFilterBar(QToolBar):
         self.entrydlg.setVisible(True)
 
     def queryIssued(self, query):
+        # type: (Text) -> None
         self.revsetcombo.setEditText(query)
         self.runQuery()
 
     def _prepareQuery(self):
+        # type: () -> Text
         query = pycompat.unicode(self.revsetcombo.currentText()).strip()
         if _querytype(self._repo, query) == 'keyword':
             return hglib.formatrevspec('keyword(%s)', query)
@@ -305,6 +319,7 @@ class RepoFilterBar(QToolBar):
             return query
 
     def _isUnsavedQuery(self):
+        # type: () -> bool
         return pycompat.unicode(self.revsetcombo.currentText()).startswith(' ')
 
     @pyqtSlot()
@@ -344,6 +359,7 @@ class RepoFilterBar(QToolBar):
         le.setContentsMargins(*margins)
 
     def setQuery(self, query):
+        # type: (Text) -> None
         self.revsetcombo.setCurrentIndex(self.revsetcombo.findText(query))
         self.revsetcombo.setEditText(query)
 
@@ -371,6 +387,7 @@ class RepoFilterBar(QToolBar):
         self.revsetcombo.setCurrentIndex(self.revsetcombo.findText(query))
 
     def loadSettings(self, s):
+        # type: (QSettings) -> None
         repoid = hglib.shortrepoid(self._repo)
         s.beginGroup('revset/' + repoid)
         self.entrydlg.restoreGeometry(qtlib.readByteArray(s, 'geom'))
@@ -388,6 +405,7 @@ class RepoFilterBar(QToolBar):
         s.endGroup()
 
     def saveSettings(self, s):
+        # type: (QSettings) -> None
         try:
             repoid = hglib.shortrepoid(self._repo)
         except EnvironmentError:
@@ -438,6 +456,7 @@ class RepoFilterBar(QToolBar):
         self.addWidget(self._branchCombo)
 
     def _loadBranchFilterSettings(self, s):
+        # type: (QSettings) -> None
         branch = qtlib.readString(s, 'branch')
         if branch == '.':
             branch = hglib.tounicode(self._repo.dirstate.branch())
@@ -448,6 +467,7 @@ class RepoFilterBar(QToolBar):
         self._allparAction.setChecked(qtlib.readBool(s, 'branch_allparents'))
 
     def _saveBranchFilterSettings(self, s):
+        # type: (QSettings) -> None
         branch = self.branch()
         if branch == hglib.tounicode(self._repo.dirstate.branch()):
             # special case for working branch: it's common to have multiple
@@ -466,7 +486,7 @@ class RepoFilterBar(QToolBar):
                 for n in self._repo.heads()
                 if not self._repo[n].extra().get(b'close')]))
         elif self._cbranchAction.isChecked():
-            branches = sorted(self._repo.branchmap())
+            branches = sorted(self._repo.branchmap())  # type: List[bytes]
         else:
             branches = hglib.namedbranches(self._repo)
 
@@ -493,24 +513,29 @@ class RepoFilterBar(QToolBar):
 
     @pyqtSlot(str)
     def setBranch(self, branch):
+        # type: (Text) -> None
         """Change the current branch by name [unicode]"""
         index = self._branchCombo.findData(branch)
         if index >= 0:
             self._branchCombo.setCurrentIndex(index)
 
     def branch(self):
+        # type: () -> Text
         """Return the current branch name [unicode]"""
         index = self._branchCombo.currentIndex()
         branch = self._branchCombo.itemData(index)
         return pycompat.unicode(branch)
 
     def branchAncestorsIncluded(self):
+        # type: () -> bool
         return self._allparAction.isChecked()
 
     def getShowHidden(self):
+        # type: () -> bool
         return self.showHiddenBtn.isChecked()
 
     def getShowGraftSource(self):
+        # type: () -> bool
         return self.showGraftSourceBtn.isChecked()
 
     @pyqtSlot()
