@@ -53,6 +53,27 @@ from . import (
     graphopt,
 )
 
+if hglib.TYPE_CHECKING:
+    from typing import (
+        Any,
+        FrozenSet,
+        List,
+        Optional,
+        Text,
+        Tuple,
+        Union,
+    )
+    from mercurial import context
+    from .qtcore import QObject
+    from . import (
+        thgrepo,
+        filedialogs,
+    )
+
+    # _UnfilteredRepoAgentProxy is a short term hack, so don't use this type
+    # for RepoAgent everywhere.
+    RepoAgent = Union[thgrepo.RepoAgent, filedialogs._UnfilteredRepoAgentProxy]
+
 mqpatchmimetype = 'application/thg-mqunappliedpatch'
 
 # pick names from "hg help templating" if any
@@ -103,6 +124,7 @@ GraphNodeRole = Qt.UserRole + 0
 LabelsRole = Qt.UserRole + 1  # [(text, style), ...]
 
 def _parsebranchcolors(value):
+    # type: (Text) -> List[Tuple[Text, Text]]
     r"""Parse tortoisehg.branchcolors setting
 
     >>> _parsebranchcolors('foo:#123456  bar:#789abc ')
@@ -160,12 +182,13 @@ class HgRepoListModel(QAbstractTableModel):
     _mqtags = (b'qbase', b'qtip', b'qparent')
 
     def __init__(self, repoagent, parent=None):
+        # type: (RepoAgent, Optional[QObject]) -> None
         QAbstractTableModel.__init__(self, parent)
         self._cache = []
         self._timerhandle = None
         self._rowcount = 0
         self._repoagent = repoagent
-        self._selectedrevs = frozenset([])
+        self._selectedrevs = frozenset([])  # type: FrozenSet[int]
         self._revspec = ''
         self._filterbyrevset = True
         self.unicodestar = True
@@ -196,6 +219,7 @@ class HgRepoListModel(QAbstractTableModel):
         return self._repoagent.rawRepo()
 
     def _initBranchColors(self):
+        # type: () -> None
         # Always assign the first color to the default branch
         self._branch_colors['default'] = graph.COLORS[0]
 
@@ -204,6 +228,7 @@ class HgRepoListModel(QAbstractTableModel):
             self._repoagent.configString('tortoisehg', 'branchcolors')))
 
     def setBranch(self, branch, allparents=False):
+        # type: (Text, bool) -> None
         branchchanged = (branch != self._filterbranch)
         parentchanged = (allparents != self._allparents)
         self._filterbranch = branch
@@ -212,6 +237,7 @@ class HgRepoListModel(QAbstractTableModel):
             self._rebuildGraph()
 
     def setShowGraftSource(self, visible):
+        # type: (bool) -> None
         if self._showgraftsource == visible:
             return
         self._showgraftsource = visible
@@ -245,6 +271,7 @@ class HgRepoListModel(QAbstractTableModel):
 
     @pyqtSlot()
     def _reloadGraph(self):
+        # type: () -> None
         self._branchheads.clear()
         self._latesttags = {-1: self._latesttags[-1]}  # clear
         if self._revspec:
@@ -252,6 +279,7 @@ class HgRepoListModel(QAbstractTableModel):
         self._rebuildGraph()
 
     def _rebuildGraph(self):
+        # type: () -> None
         if not self._querysess.isFinished():
             self._pendingrebuild = True
             return
@@ -320,9 +348,11 @@ class HgRepoListModel(QAbstractTableModel):
         self.revsUpdated.emit()
 
     def revset(self):
+        # type: () -> Text
         return self._revspec
 
     def setRevset(self, revspec):
+        # type: (Text) -> None
         revspec = pycompat.unicode(revspec)
         if revspec == self._revspec:
             return
@@ -334,6 +364,7 @@ class HgRepoListModel(QAbstractTableModel):
         self._runQuery()
 
     def _runQuery(self):
+        # type: () -> None
         self.showMessage.emit(_('Searching...'))
 
         self._querysess.abort()
@@ -344,6 +375,7 @@ class HgRepoListModel(QAbstractTableModel):
 
     @pyqtSlot(int)
     def _onQueryFinished(self, ret):
+        # type: (int) -> None
         sess = self._querysess
         if not sess.isFinished():
             # new query is already running
@@ -361,6 +393,7 @@ class HgRepoListModel(QAbstractTableModel):
             self._rebuildGraph()  # invalid revspec
 
     def _applyRevsetResult(self, revset):
+        # type: (List[int]) -> None
         self._selectedrevs = frozenset(revset)
         if self._filterbyrevset or self._pendingrebuild:
             self._rebuildGraph()
@@ -369,6 +402,7 @@ class HgRepoListModel(QAbstractTableModel):
             self.revsUpdated.emit()  # some revisions may be disabled
 
     def setFilterByRevset(self, filtered):
+        # type: (bool) -> None
         if self._filterbyrevset == filtered:
             return
         self._filterbyrevset = filtered
@@ -376,6 +410,7 @@ class HgRepoListModel(QAbstractTableModel):
             self._rebuildGraph()
 
     def _reloadConfig(self):
+        # type: () -> None
         self._fill_step = self._repoagent.configInt('tortoisehg', 'graphlimit')
         self._authorcolor = self._repoagent.configBool(
             'tortoisehg', 'authorcolor')
@@ -386,11 +421,13 @@ class HgRepoListModel(QAbstractTableModel):
 
     @pyqtSlot()
     def _invalidate(self):
+        # type: () -> None
         self._reloadConfig()
         self._cache = []
         self._emitAllDataChanged()
 
     def _emitAllDataChanged(self):
+        # type: () -> None
         if self._rowcount <= 0:
             return
         # optimize range if necessary
@@ -398,20 +435,24 @@ class HgRepoListModel(QAbstractTableModel):
         self.dataChanged.emit(self.index(0, 0), bottomright)
 
     def branch(self):
+        # type: () -> Text
         return self._filterbranch
 
     def canFetchMore(self, parent):
+        # type: (QModelIndex) -> bool
         if parent.isValid():
             return False
         return not self.graph.isfilled()
 
     def fetchMore(self, parent):
+        # type: (QModelIndex) -> None
         if parent.isValid() or self.graph.isfilled():
             return
         self.graph.build_nodes(self._fill_step)
         self._expandRowCount()
 
     def _ensureBuilt(self, rev):
+        # type: (Optional[int]) -> None
         """
         Make sure rev data is available (graph element created).
 
@@ -422,6 +463,7 @@ class HgRepoListModel(QAbstractTableModel):
         # caller should do _expandRowCount() or _shrinkRowCount() by itself
 
     def loadall(self):
+        # type: () -> None
         self._timerhandle = self.startTimer(1)
 
     def timerEvent(self, event):
@@ -438,6 +480,7 @@ class HgRepoListModel(QAbstractTableModel):
                 self.showMessage.emit('')
 
     def _expandRowCount(self):
+        # type: () -> None
         newlen = len(self.graph)
         if newlen > self._rowcount:
             self.beginInsertRows(QModelIndex(), self._rowcount, newlen - 1)
@@ -445,6 +488,7 @@ class HgRepoListModel(QAbstractTableModel):
             self.endInsertRows()
 
     def _shrinkRowCount(self):
+        # type: () -> None
         newlen = len(self.graph)
         if newlen < self._rowcount:
             self.beginRemoveRows(QModelIndex(), newlen, self._rowcount - 1)
@@ -452,22 +496,25 @@ class HgRepoListModel(QAbstractTableModel):
             self.endRemoveRows()
 
     def rowCount(self, parent=QModelIndex()):
+        # type: (QModelIndex) -> int
         if parent.isValid():
             return 0
         return self._rowcount
 
     def columnCount(self, parent=QModelIndex()):
+        # type: (QModelIndex) -> int
         if parent.isValid():
             return 0
         return len(self.allColumns())
 
     def maxWidthValueForColumn(self, column):
+        # type: (int) -> Optional[Text]
         if column == RevColumn:
             return '8' * len(str(len(self.repo))) + '+'
         if column in (NodeColumn, GitNodeColumn):
             return '8' * 12 + '+'
         if column in (LocalDateColumn, UtcDateColumn):
-            return hglib.displaytime(dateutil.makedate())
+            return hglib.tounicode(hglib.displaytime(dateutil.makedate()))
         if column in (TagsColumn, LatestTagColumn):
             try:
                 return sorted(self.repo.tags().keys(), key=lambda x: len(x))[-1][:10]
@@ -486,6 +533,7 @@ class HgRepoListModel(QAbstractTableModel):
         return None
 
     def rev(self, index):
+        # type: (QModelIndex) -> int
         """Revision number of the specified row; None for working-dir"""
         if not index.isValid():
             return -1
@@ -496,6 +544,7 @@ class HgRepoListModel(QAbstractTableModel):
         return gnode.rev
 
     def _user_color(self, user):
+        # type: (Text) -> Text
         'deprecated, please replace with hgtk color scheme'
         if user not in self._user_colors:
             idx = graph.hashcolor(user)
@@ -503,6 +552,7 @@ class HgRepoListModel(QAbstractTableModel):
         return self._user_colors[user]
 
     def _namedbranch_color(self, branch):
+        # type: (Text) -> Text
         'deprecated, please replace with hgtk color scheme'
         if branch not in self._branch_colors:
             idx = graph.hashcolor(branch)
@@ -510,6 +560,7 @@ class HgRepoListModel(QAbstractTableModel):
         return self._branch_colors[branch]
 
     def data(self, index, role=Qt.DisplayRole):
+        # type: (QModelIndex, int) -> Any
         if not index.isValid():
             return None
         gnode = self.graph[index.row()]
@@ -542,6 +593,7 @@ class HgRepoListModel(QAbstractTableModel):
                 return None
 
     def _safedata(self, index, role):
+        # type: (QModelIndex, int) -> Any
         row = index.row()
         graphlen = len(self.graph)
         cachelen = len(self._cache)
@@ -563,6 +615,7 @@ class HgRepoListModel(QAbstractTableModel):
         return data[idx]
 
     def _rawdata(self, index, role):
+        # type: (QModelIndex, int) -> Any
         row = index.row()
         column = index.column()
         gnode = self.graph[row]
@@ -572,10 +625,7 @@ class HgRepoListModel(QAbstractTableModel):
             textfunc = self._columnmap.get(column)
             if textfunc is None:
                 return None
-            text = textfunc(self, ctx)
-            if not isinstance(text, pycompat.unicode):
-                text = hglib.tounicode(text)
-            return text
+            return textfunc(self, ctx)
         elif role == Qt.ForegroundRole:
             color = None
             if gnode.instabilities:
@@ -599,6 +649,7 @@ class HgRepoListModel(QAbstractTableModel):
         return None
 
     def flags(self, index):
+        # type: (QModelIndex) -> int
         flags = super(HgRepoListModel, self).flags(index)
         if not index.isValid():
             return flags
@@ -616,6 +667,7 @@ class HgRepoListModel(QAbstractTableModel):
         return flags
 
     def isActiveRev(self, rev):
+        # type: (int) -> bool
         """True if the specified rev is not excluded by revset"""
         return (not self._revspec
                 or rev in self._selectedrevs
@@ -624,12 +676,14 @@ class HgRepoListModel(QAbstractTableModel):
                     and not self._querysess.isFinished()))
 
     def mimeTypes(self):
+        # type: () -> List[Text]
         return [mqpatchmimetype]
 
     def supportedDropActions(self):
         return Qt.MoveAction
 
     def mimeData(self, indexes):
+        # type: (List[QModelIndex]) -> QMimeData
         data = set()
         for index in indexes:
             row = b'%d' % index.row()
@@ -641,6 +695,7 @@ class HgRepoListModel(QAbstractTableModel):
         return qmd
 
     def dropMimeData(self, data, action, row, column, parent):
+        # type: (QMimeData, int, int, int, QModelIndex) -> bool
         if mqpatchmimetype not in data.formats():
             return False
         dragrows = [int(r)
@@ -665,10 +720,12 @@ class HgRepoListModel(QAbstractTableModel):
         return True
 
     def headerData(self, section, orientation, role=Qt.DisplayRole):
+        # type: (int, int, int) -> Optional[Text]
         if orientation == Qt.Horizontal and role == Qt.DisplayRole:
             return self.allColumnHeaders()[section][1]
 
     def defaultIndex(self):
+        # type: () -> QModelIndex
         """Index that should be selected when the model is initially loaded
         or the row previously selected is gone"""
         repo = self.repo
@@ -678,7 +735,7 @@ class HgRepoListModel(QAbstractTableModel):
                     'tip': b'tip',
                     'workingdir': None,
                     }.get(initialsel, b'.')
-        rev = repo[changeid].rev()
+        rev = repo[changeid].rev()  # type: int
         if self._selectedrevs and rev not in self._selectedrevs:
             rev = max(self._selectedrevs)
         index = self.indexFromRev(rev)
@@ -698,6 +755,7 @@ class HgRepoListModel(QAbstractTableModel):
         return QModelIndex()
 
     def indexFromRev(self, rev):
+        # type: (int) -> QModelIndex
         self._ensureBuilt(rev)
         self._expandRowCount()
         try:
@@ -707,8 +765,10 @@ class HgRepoListModel(QAbstractTableModel):
         return self.index(row, 0)
 
     def _getbranch(self, ctx):
+        # type: (context.changectx) -> Text
         b = hglib.tounicode(ctx.branch())
-        if ctx.extra().get('close'):
+        assert b is not None
+        if ctx.extra().get(b'close'):
             if self.unicodexinabox:
                 b += u' \u2327'
             else:
@@ -716,6 +776,7 @@ class HgRepoListModel(QAbstractTableModel):
         return b
 
     def _getlatesttags(self, ctx):
+        # type: (context.changectx) -> Text
         rev = ctx.rev()
         todo = [rev]
         repo = self.repo
@@ -746,12 +807,14 @@ class HgRepoListModel(QAbstractTableModel):
         return self._latesttags[rev][2]
 
     def _gettags(self, ctx):
+        # type: (context.changectx) -> Text
         if ctx.rev() is None:
             return ''
         tags = [hglib.tounicode(t) for t in ctx.tags() if t not in self._mqtags]
         return ','.join(tags)
 
     def _getrev(self, ctx):
+        # type: (context.changectx) -> Text
         rev = ctx.rev()
         if isinstance(rev, int):
             return str(rev)
@@ -761,15 +824,17 @@ class HgRepoListModel(QAbstractTableModel):
             return ''
 
     def _getauthor(self, ctx):
+        # type: (context.changectx) -> Text
         try:
             user = ctx.user()
             if not self._fullauthorname:
                 user = hglib.username(user)
-            return user
+            return hglib.tounicode(user)
         except error.Abort:
             return _('Mercurial User')
 
     def _getlog(self, ctx):
+        # type: (context.changectx) -> Text
         if ctx.rev() is None:
             if self.unicodestar:
                 # The Unicode symbol is a black star:
@@ -783,6 +848,7 @@ class HgRepoListModel(QAbstractTableModel):
         return hglib.longsummary(ctx.description(), limit)
 
     def _getrevlabels(self, ctx):
+        # type: (context.changectx) -> List[Tuple[Text, Text]]
         labels = []
 
         # as of hg 4.4.2, repo.branchheads() can be slow because of
@@ -806,9 +872,10 @@ class HgRepoListModel(QAbstractTableModel):
             if ctx.node() in branchheads:
                 labels.append((hglib.tounicode(branch), 'log.branch'))
 
-        if ctx.thgmqunappliedpatch():
+        # ctx is really thgrepo.thgchangectx, but not accessible here
+        if ctx.thgmqunappliedpatch():  # pytype: disable=attribute-error
             style = 'log.unapplied_patch'
-            labels.append((hglib.tounicode(ctx._patchname), style))
+            labels.append((hglib.tounicode(ctx._patchname), style))  # pytype: disable=attribute-error
 
         for mark in ctx.bookmarks():
             style = 'log.bookmark'
@@ -818,7 +885,7 @@ class HgRepoListModel(QAbstractTableModel):
                     style = 'log.curbookmark'
             labels.append((hglib.tounicode(mark), style))
 
-        for tag in ctx.thgtags():
+        for tag in ctx.thgtags():  # pytype: disable=attribute-error
             if self.repo.thgmqtag(tag):
                 style = 'log.patch'
             else:
@@ -839,9 +906,12 @@ class HgRepoListModel(QAbstractTableModel):
         return labels
 
     def _getchanges(self, ctx):
+        # type: (context.changectx) -> List[Tuple[Text, Text]]
         """Return the MAR status for the given ctx."""
         labels = []
-        M, A, R = ctx.changesToParent(0)
+
+        # ctx is really thgrepo.thgchangectx, but not accessible here
+        M, A, R = ctx.changesToParent(0)  # pytype: disable=attribute-error
         if A:
             labels.append((str(len(A)), 'log.added'))
         if M:
@@ -851,38 +921,43 @@ class HgRepoListModel(QAbstractTableModel):
         return labels
 
     def _getconv(self, ctx):
+        # type: (context.changectx) -> Text
         if ctx.rev() is not None:
             extra = ctx.extra()
-            cvt = extra.get('convert_revision', '')
+            cvt = extra.get(b'convert_revision', b'')
             if cvt:
-                if cvt.startswith('svn:'):
-                    return cvt.split('@')[-1]
+                if cvt.startswith(b'svn:'):
+                    return hglib.tounicode(cvt.split(b'@')[-1])
                 if len(cvt) == 40:
                     try:
                         binascii.unhexlify(cvt)
-                        return cvt[:12]
+                        return hglib.tounicode(cvt[:12])
                     except TypeError:
                         pass
-            cvt = extra.get('p4', '')
+            cvt = extra.get(b'p4', b'')
             if cvt:
-                return cvt
+                return hglib.tounicode(cvt)
         return ''
 
     def _getphase(self, ctx):
+        # type: (context.changectx) -> Text
         if ctx.rev() is None:
             return ''
         try:
-            return ctx.phasestr()
+            return hglib.tounicode(ctx.phasestr())
         except:
             return 'draft'
 
     def _gettopic(self, ctx):
+        # type: (context.changectx) -> Optional[Text]
         return hglib.tounicode(ctx.extra().get(b'topic'))
 
     def _hasFileColumn(self):
+        # type: () -> bool
         return False # no FileColumn
 
     def allColumns(self):
+        # type: () -> Tuple[Text, ...]
         if self._hasFileColumn():
             return ALLCOLUMNS
         else:
@@ -902,9 +977,9 @@ class HgRepoListModel(QAbstractTableModel):
         TagsColumn: _gettags,
         LatestTagColumn: _getlatesttags,
         NodeColumn: lambda self, ctx: str(ctx),
-        GitNodeColumn: lambda self, ctx: hglib.gitcommit(ctx),
+        GitNodeColumn: lambda self, ctx: hglib.gitcommit_short(ctx),
         AgeColumn: lambda self, ctx: hglib.age(ctx.date()).decode('utf-8'),
-        LocalDateColumn: lambda self, ctx: hglib.displaytime(ctx.date()),
+        LocalDateColumn: lambda self, ctx: hglib.displaytime(ctx.date()).decode('utf-8'),
         UtcDateColumn: lambda self, ctx: hglib.utctime(ctx.date()),
         ConvertedColumn: _getconv,
         PhaseColumn: _getphase,
@@ -922,6 +997,7 @@ class FileRevModel(HgRepoListModel):
                        'Age', 'Filename')
 
     def __init__(self, repoagent, filename, parent=None):
+        # type: (RepoAgent, bytes, Optional[QObject]) -> None
         self._filename = filename
         HgRepoListModel.__init__(self, repoagent, parent)
 
@@ -933,6 +1009,7 @@ class FileRevModel(HgRepoListModel):
         return graph.Graph(self.repo, grapher)
 
     def indexLinkedFromRev(self, rev):
+        # type: (Optional[int]) -> QModelIndex
         """Index for the last changed revision before the specified revision
 
         This does not follow renames.
@@ -948,6 +1025,7 @@ class FileRevModel(HgRepoListModel):
         return self.indexFromRev(fctx.linkrev())
 
     def fileData(self, index, baseindex=QModelIndex()):
+        # type: (QModelIndex, QModelIndex) -> Union[filedata.FileData, filedata.PatchFileData]
         """Displayable file data at the given index; baseindex specifies the
         revision where status is calculated from"""
         row = index.row()
