@@ -10,6 +10,7 @@ from __future__ import absolute_import
 import os
 
 from .qtcore import (
+    QFile,
     QFileSystemWatcher,
     QModelIndex,
     QPoint,
@@ -263,7 +264,7 @@ class RepoRegistryView(QDockWidget):
         self._updateSettingActions()
 
         sfile = settingsfilename()
-        model = repotreemodel.RepoTreeModel(sfile, repomanager, self,
+        model = repotreemodel.RepoTreeModel(QFile(sfile), repomanager, self,
             showShortPaths=self._isSettingEnabled('showShortPaths'))
         tv.setModel(model)
 
@@ -383,7 +384,7 @@ class RepoRegistryView(QDockWidget):
         oldmodel = self.tview.model()
         assert oldmodel is not None
         activeroot = oldmodel.repoRoot(oldmodel.activeRepoIndex())
-        newmodel = repotreemodel.RepoTreeModel(settingsfilename(),
+        newmodel = repotreemodel.RepoTreeModel(QFile(settingsfilename()),
             self._repomanager, self,
             self._isSettingEnabled('showShortPaths'))
         self.tview.setModel(newmodel)
@@ -645,9 +646,8 @@ class RepoRegistryView(QDockWidget):
                     hasHgsub = os.path.exists(repo.wjoin(b'.hgsub'))
                     if hasHgsub:
                         try:
-                            fsub = repo.wvfs(b'.hgsub', b'r')
-                            lines = fsub.readlines()
-                            fsub.close()
+                            with repo.wvfs(b'.hgsub', b'r') as fsub:
+                                lines = fsub.readlines()
                         except:
                             qtlib.WarningMsgBox(
                                 _('Failed to add subrepository'),
@@ -684,9 +684,9 @@ class RepoRegistryView(QDockWidget):
 
                     # and update the .hgsub file
                     try:
-                        fsub = repo.wvfs(b'.hgsub', b'w')
-                        fsub.write(linesep.join(lines) + linesep)
-                        fsub.close()
+                        with repo.wvfs(b'.hgsub', b'w') as fsub:
+                            fsub.write(linesep.join(lines) + linesep)
+
                         if not hasHgsub:
                             commands.add(hglib.loadui(),
                                          repo, repo.wjoin(b'.hgsub'))
@@ -875,12 +875,12 @@ class RepoRegistryView(QDockWidget):
         repo = hg.repository(hglib.loadui(),
                              hglib.fromunicode(model.repoRoot(index)))
         ctx = repo[b'.']
-        wfile = '.hgsub'
+        wfile = b'.hgsub'
         if wfile not in ctx:
             return self.sortbypath()
         data = ctx[wfile].data().strip()
-        data = data.split('\n')
-        getsubpath = lambda x: x.split('=')[0].strip()
+        data = data.split(b'\n')
+        getsubpath = lambda x: x.split(b'=')[0].strip()
         abspath = lambda x: util.normpath(repo.wjoin(x))
         hgsuborder = [abspath(getsubpath(x)) for x in data]
         def keyfunc(x):
