@@ -70,9 +70,10 @@ class RepoTabWidget(QWidget):
     #
     # tab-index is the master, so do not use stack.setCurrentIndex().
 
-    def __init__(self, config, repomanager, parent=None):
+    def __init__(self, config, actionregistry, repomanager, parent=None):
         super(RepoTabWidget, self).__init__(parent)
         self._config = config
+        self._actionregistry = actionregistry
         self._repomanager = repomanager
         # delay until the next event loop so that the current tab won't be
         # gone in the middle of switching tabs (issue #4253)
@@ -402,7 +403,8 @@ class RepoTabWidget(QWidget):
             qtlib.WarningMsgBox(_('Failed to open repository'),
                                 hglib.tounicode(bytes(e)), parent=self)
             return
-        rw = repowidget.RepoWidget(repoagent, self, bundle=bundle)
+        rw = repowidget.RepoWidget(self._actionregistry, repoagent, self,
+                                   bundle=bundle)
         rw.currentTaskTabChanged.connect(self.currentTaskTabChanged)
         rw.makeLogVisible.connect(self.makeLogVisible)
         rw.progress.connect(self._mapProgressReceived)
