@@ -533,7 +533,7 @@ class HgRepoViewStyle(QProxyStyle):
             if option.rect.height() != 0:
                 vp = widget.viewport().rect()
                 painter.drawRect(vp.x(), option.rect.y(),
-                                 vp.width() - 1, 0.5)
+                                 vp.width() - 1, 0)
         else:
             super(HgRepoViewStyle, self).drawPrimitive(element, option, painter,
                                                        widget)
@@ -566,11 +566,11 @@ class GraphDelegate(QStyledItemDelegate):
         self._rowheight = 16  # updated to the actual height on paint()
 
     def _col2x(self, col):
-        maxradius = max(self._rowheight / 2, 1)
+        maxradius = max(self._rowheight // 2, 1)
         return maxradius * (col + 1)
 
     def _colcount(self, width):
-        maxradius = max(self._rowheight / 2, 1)
+        maxradius = max(self._rowheight // 2, 1)
         return (width + maxradius - 1) // maxradius
 
     def _dotradius(self):
@@ -595,7 +595,7 @@ class GraphDelegate(QStyledItemDelegate):
 
     def _drawEdges(self, painter, index, gnode, visibleend):
         h = self._rowheight
-        dot_y = h / 2
+        dot_y = h // 2
 
         def isactive(e):
             m = index.model()
@@ -605,9 +605,9 @@ class GraphDelegate(QStyledItemDelegate):
 
         for y1, y4, lines in ((dot_y, dot_y + h, gnode.bottomlines),
                               (dot_y - h, dot_y, gnode.toplines)):
-            y2 = y1 + 1 * (y4 - y1)/4
-            ymid = (y1 + y4)/2
-            y3 = y1 + 3 * (y4 - y1)/4
+            y2 = y1 + 1 * (y4 - y1) // 4
+            ymid = (y1 + y4) // 2
+            y3 = y1 + 3 * (y4 - y1) // 4
 
             # omit invisible lines
             lines = [((start, end), e) for (start, end), e in lines
@@ -633,7 +633,7 @@ class GraphDelegate(QStyledItemDelegate):
                     path.moveTo(x1, y1)
                     path.cubicTo(x1, y2,
                                  x1, y2,
-                                 (x1 + x2) / 2, ymid)
+                                 (x1 + x2) // 2, ymid)
                     path.cubicTo(x2, y3,
                                  x2, y3,
                                  x2, y4)
@@ -664,7 +664,7 @@ class GraphDelegate(QStyledItemDelegate):
         painter.setPen(pen)
 
         centre_x = self._col2x(gnode.x)
-        centre_y = self._rowheight / 2
+        centre_y = self._rowheight // 2
 
         def circle(r):
             rect = QRectF(centre_x - r,
