@@ -3,7 +3,7 @@
 # Copyright 2010 Steve Borho <steve@borho.org>
 #
 # This software may be used and distributed according to the terms of the
-# GNU General Public License version 2, incorporated herein by reference.
+# GNU General Public License version 2 or any later version.
 
 from __future__ import absolute_import
 
@@ -52,6 +52,17 @@ from . import (
     visdiff,
 )
 
+if hglib.TYPE_CHECKING:
+    from typing import (
+        List,
+        Optional,
+        Text,
+        Tuple,
+    )
+    from .qtgui import (
+        QWidget,
+    )
+
 MARGINS = (8, 0, 0, 0)
 
 class ResolveDialog(QDialog):
@@ -69,6 +80,8 @@ class ResolveDialog(QDialog):
         self.setLayout(QVBoxLayout())
         self.layout().setSpacing(5)
 
+        self.mergeState = hglib.readmergestate(self.repo)
+
         hbox = QHBoxLayout()
         self.layout().addLayout(hbox)
 
@@ -94,7 +107,7 @@ class ResolveDialog(QDialog):
             localrevtitle = qtlib.LabeledSeparator(_('Local revision '
                                                      'information'))
             localrevinfo = csinfo.create(repo)
-            localrevinfo.update(repo[repo[None].p1().rev()])
+            localrevinfo.update(self.mergeState.localctx)
             vbox.addWidget(localrevtitle)
             vbox.addWidget(localrevinfo)
             vbox.addStretch()
@@ -105,7 +118,7 @@ class ResolveDialog(QDialog):
             otherrevtitle = qtlib.LabeledSeparator(_('Other revision '
                                                      'information'))
             otherrevinfo = csinfo.create(repo)
-            otherrevinfo.update(repo[repo[None].p2().rev()])
+            otherrevinfo.update(self.mergeState.otherctx)
 
             vbox.addWidget(otherrevtitle)
             vbox.addWidget(otherrevinfo)
@@ -113,7 +126,7 @@ class ResolveDialog(QDialog):
 
             return hbox
 
-        if len(self.repo[None].parents()) > 1:
+        if self.mergeState.active():
             self.layout().addLayout(revisionInfoLayout(self.repo))
 
         unres = qtlib.LabeledSeparator(_('Unresolved conflicts'))
@@ -222,6 +235,8 @@ class ResolveDialog(QDialog):
         self.layout().addLayout(hbox)
 
         self.tcombo = ToolsCombo(self.repo, self)
+        self.tcombo.setSizeAdjustPolicy(QComboBox.AdjustToContents)
+
         hbox.addWidget(QLabel(_('Detected merge/diff tools:')))
         hbox.addWidget(self.tcombo)
         hbox.addStretch(1)
@@ -251,17 +266,19 @@ class ResolveDialog(QDialog):
         return self._repoagent.rawRepo()
 
     def getSelectedPaths(self, tree):
+        # type: (QTreeView) -> List[Tuple[bytes, bytes]]
         paths = []
         if not tree.selectionModel():
             return paths
         model = tree.model()
-        assert model is not None
+        assert isinstance(model, PathsModel)  # help pytype
         for idx in tree.selectionModel().selectedRows():
             root, wfile = model.getPathForIndex(idx)
             paths.append((root, wfile))
         return paths
 
     def runCommand(self, tree, cmdline):
+        # type: (QTreeView, List[Text]) -> None
         cmdlines = []
         selected = self.getSelectedPaths(tree)
         while selected:
@@ -279,7 +296,7 @@ class ResolveDialog(QDialog):
             sess.outputReceived.connect(self._cmdlog.appendLog)
             self._updateActions()
 
-    def merge(self, tool=False):
+    def merge(self, tool=''):
         if not tool:
             tool = self.tcombo.readValue()
             if tool:
@@ -306,6 +323,7 @@ class ResolveDialog(QDialog):
             qtlib.editfiles(self.repo, abspaths, parent=self)
 
     def getVdiffFiles(self, tree):
+        # type: (QTreeView) -> List[bytes]
         paths = self.getSelectedPaths(tree)
         if not paths:
             return []
@@ -324,34 +342,39 @@ class ResolveDialog(QDialog):
     def v3way(self):
         paths = self.getVdiffFiles(self.rtree)
         if paths:
-            opts = {'rev': [],
-                    'tool': self.tcombo.readValue()}
-            dlg = visdiff.visualdiff(self.repo.ui, self.repo, paths, opts)
+            p1 = self.mergeState.localctx
+            p2 = self.mergeState.otherctx
+
+            dlg = visdiff.visual_diff(self.repo.ui, self.repo, paths, p1, p2,
+                                      self.repo[None], self.tcombo.readValue())
             if dlg:
                 dlg.exec_()
 
     def vp0(self):
         paths = self.getVdiffFiles(self.rtree)
         if paths:
-            opts = {'rev': ['p1()'],
-                    'tool': self.tcombo.readValue()}
-            dlg = visdiff.visualdiff(self.repo.ui, self.repo, paths, opts)
+            p1 = self.mergeState.localctx
+            dlg = visdiff.visual_diff(self.repo.ui, self.repo, paths, p1, None,
+                                      self.repo[None], self.tcombo.readValue())
             if dlg:
                 dlg.exec_()
 
     def vp1(self):
         paths = self.getVdiffFiles(self.rtree)
         if paths:
-            opts = {'rev': ['p2()'],
-                    'tool': self.tcombo.readValue()}
-            dlg = visdiff.visualdiff(self.repo.ui, self.repo, paths, opts)
+            p2 = self.mergeState.otherctx
+            dlg = visdiff.visual_diff(self.repo.ui, self.repo, paths, p2, None,
+                                      self.repo[None], self.tcombo.readValue())
             if dlg:
                 dlg.exec_()
 
     def diffLocToAnc(self):
         paths = self.getVdiffFiles(self.utree)
         if paths:
-            opts = {'rev': ['ancestor(p1(),p2())..p1()'],
+            p1 = self.mergeState.localctx.hex()
+            p2 = self.mergeState.otherctx.hex()
+            revs = hglib.tounicode(b'ancestor(%s,%s)..%s' % (p1, p2, p1))
+            opts = {'rev': [revs],
                     'tool': self.tcombo.readValue()}
             dlg = visdiff.visualdiff(self.repo.ui, self.repo, paths, opts)
             if dlg:
@@ -360,7 +383,10 @@ class ResolveDialog(QDialog):
     def diffOthToAnc(self):
         paths = self.getVdiffFiles(self.utree)
         if paths:
-            opts = {'rev': ['ancestor(p1(),p2())..p2()'],
+            p1 = self.mergeState.localctx.hex()
+            p2 = self.mergeState.otherctx.hex()
+            revs = hglib.tounicode(b'ancestor(%s,%s)..%s' % (p1, p2, p2))
+            opts = {'rev': [revs],
                     'tool': self.tcombo.readValue()}
             dlg = visdiff.visualdiff(self.repo.ui, self.repo, paths, opts)
             if dlg:
@@ -374,6 +400,9 @@ class ResolveDialog(QDialog):
                 u.append((root, path))
             else:
                 r.append((root, path))
+
+        self.mergeState = hglib.readmergestate(self.repo)
+
         paths = self.getSelectedPaths(self.utree)
         oldmodel = self.utree.model()
         self.utree.setModel(PathsModel(u, self))
@@ -451,16 +480,18 @@ class ResolveDialog(QDialog):
                   and self._cmdsession.isFinished())
         for c in self.rmenuitems:
             c.setEnabled(enable)
-        merge = len(self.repo[None].parents()) > 1
+
         for c in self.rmmenuitems:
-            c.setEnabled(enable and merge)
+            c.setEnabled(enable and self.mergeState.active())
 
     @pyqtSlot(QPoint)
     def utreeMenuRequested(self, point):
+        # type: (QPoint) -> None
         self.utreecmenu.popup(self.utree.viewport().mapToGlobal(point))
 
     @pyqtSlot(QPoint)
     def rtreeMenuRequested(self, point):
+        # type: (QPoint) -> None
         self.rtreecmenu.popup(self.rtree.viewport().mapToGlobal(point))
 
     def utreeDoubleClicked(self):
@@ -472,24 +503,28 @@ class ResolveDialog(QDialog):
 
 class PathsModel(QAbstractTableModel):
     def __init__(self, pathlist, parent):
+        # type: (List[Tuple[bytes, bytes]], Optional[QWidget]) -> None
         QAbstractTableModel.__init__(self, parent)
         self.headers = (_('Path'), _('Ext'), _('Repository'))
-        self.rows = []
+        self.rows = []  # type: List[Tuple[bytes, bytes, bytes]]
         for root, path in pathlist:
             name, ext = os.path.splitext(path)
-            self.rows.append([path, ext, root])
+            self.rows.append((path, ext, root))
 
     def rowCount(self, parent=QModelIndex()):
+        # type: (QModelIndex) -> int
         if parent.isValid():
             return 0 # no child
         return len(self.rows)
 
     def columnCount(self, parent=QModelIndex()):
+        # type: (QModelIndex) -> int
         if parent.isValid():
             return 0 # no child
         return len(self.headers)
 
     def data(self, index, role=Qt.DisplayRole):
+        # type: (QModelIndex, int) -> Optional[Text]
         if not index.isValid():
             return None
         if role == Qt.DisplayRole:
@@ -498,6 +533,7 @@ class PathsModel(QAbstractTableModel):
         return None
 
     def flags(self, index):
+        # type: (QModelIndex) -> Qt.ItemFlags
         flags = super(PathsModel, self).flags(index)
         if not index.isValid():
             return flags
@@ -505,20 +541,24 @@ class PathsModel(QAbstractTableModel):
         return flags
 
     def headerData(self, col, orientation, role=Qt.DisplayRole):
+        # type: (int, int, int) -> Optional[Text]
         if role != Qt.DisplayRole or orientation != Qt.Horizontal:
             return None
         else:
             return self.headers[col]
 
     def getPathForIndex(self, index):
+        # type: (QModelIndex) -> Tuple[bytes, bytes]
         'return root, wfile for the given row'
         row = index.row()
         return self.rows[row][2], self.rows[row][0]
 
     def mimeTypes(self):
+        # type: () -> List[Text]
         return ['text/uri-list']
 
     def mimeData(self, indexes):
+        # type: (List[QModelIndex]) -> QMimeData
         paths = [hglib.tounicode(os.path.join(*self.getPathForIndex(i)))
                  for i in indexes if i.column() == 0]
         data = QMimeData()
@@ -530,30 +570,19 @@ class ToolsCombo(QComboBox):
     def __init__(self, repo, parent):
         QComboBox.__init__(self, parent)
         self.setEditable(False)
-        self.loaded = False
         self.default = _('<default>')
-        self.addItem(self.default)
         self.repo = repo
+        self.reset()
 
     @pyqtSlot()
     def reset(self):
-        self.loaded = False
         self.clear()
         self.addItem(self.default)
-
-    def showPopup(self):
-        if not self.loaded:
-            self.loaded = True
-            self.clear()
-            self.addItem(self.default)
-            for t in self.repo.mergetools:
-                self.addItem(hglib.tounicode(t))
-        QComboBox.showPopup(self)
+        for t in self.repo.mergetools:
+            self.addItem(hglib.tounicode(t))
 
     def readValue(self):
-        if self.loaded:
-            text = self.currentText()
-            if text != self.default:
-                return hglib.fromunicode(text)
-        else:
-            return None
+        # type: () -> Optional[bytes]
+        text = self.currentText()
+        if text != self.default:
+            return hglib.fromunicode(text)
