@@ -3,7 +3,7 @@
 # Copyright 2010 Yuya Nishihara <yuya@tcha.org>
 #
 # This software may be used and distributed according to the terms of the
-# GNU General Public License version 2, incorporated herein by reference.
+# GNU General Public License version 2 or any later version.
 
 from __future__ import absolute_import
 
@@ -39,9 +39,29 @@ from . import (
 from .serve_ui import Ui_ServeDialog
 from .webconf import WebconfForm
 
+if hglib.TYPE_CHECKING:
+    from typing import (
+        Any,
+        List,
+        Text,
+        Tuple,
+        Optional,
+    )
+    from .qtgui import (
+        QWidget,
+    )
+    from mercurial import (
+        ui as uimod,
+    )
+    from ..util.typelib import (
+        IniConfig,
+    )
+
+
 class ServeDialog(QDialog):
     """Dialog for serving repositories via web"""
     def __init__(self, ui, webconf, parent=None):
+        # type: (uimod.ui, Optional[IniConfig], Optional[QWidget]) -> None
         super(ServeDialog, self).__init__(parent)
         self.setWindowFlags((self.windowFlags() | Qt.WindowMinimizeButtonHint)
                             & ~Qt.WindowContextHelpButtonHint)
@@ -56,6 +76,7 @@ class ServeDialog(QDialog):
         self._updateform()
 
     def _initcmd(self, ui):
+        # type: (uimod.ui) -> None
         # TODO: forget old logs?
         self._log_edit = cmdui.LogWidget(self)
         self._qui.details_tabs.addTab(self._log_edit, _('Log'))
@@ -65,15 +86,18 @@ class ServeDialog(QDialog):
         self._agent.busyChanged.connect(self._updateform)
 
     def _initwebconf(self, webconf):
+        # type: (Optional[IniConfig]) -> None
         self._webconf_form = WebconfForm(webconf=webconf, parent=self)
         self._qui.details_tabs.addTab(self._webconf_form, _('Repositories'))
 
     def _initactions(self):
+        # type: () -> None
         self._qui.start_button.clicked.connect(self.start)
         self._qui.stop_button.clicked.connect(self.stop)
 
     @pyqtSlot()
     def _updateform(self):
+        # type: () -> None
         """update form availability and status text"""
         self._updatestatus()
         self._qui.start_button.setEnabled(not self.isstarted())
@@ -83,6 +107,7 @@ class ServeDialog(QDialog):
         self._webconf_form.setEnabled(not self.isstarted())
 
     def _updatestatus(self):
+        # type: () -> None
         if self.isstarted():
             # TODO: escape special chars
             link = '<a href="%s">%s</a>' % (self.rooturl, self.rooturl)
@@ -94,14 +119,15 @@ class ServeDialog(QDialog):
 
     @pyqtSlot()
     def start(self):
+        # type: () -> None
         """Start web server"""
         if self.isstarted():
             return
 
-        self._agent.runCommand(pycompat.maplist(hglib.tounicode,
-                                                self._cmdargs()))
+        self._agent.runCommand(self._cmdargs())
 
     def _cmdargs(self):
+        # type: () -> List[Text]
         """Build command args to run server"""
         a = ['serve', '--port', str(self.port), '-v']
         if self._singlerepo:
@@ -111,59 +137,72 @@ class ServeDialog(QDialog):
         return a
 
     def _tempwebconf(self):
+        # type: () -> Text
         """Save current webconf to temporary file; return its path"""
         if not hasattr(self._webconf, 'write'):
-            return self._webconf.path
+            return hglib.tounicode(self._webconf.path)  # pytype: disable=attribute-error
+
+        assert isinstance(self._webconf, wconfig._wconfig)  # help pytype
 
         fd, fname = tempfile.mkstemp(prefix=b'webconf_',
                                      dir=qtlib.gettempdir())
         f = os.fdopen(fd, 'w')
         try:
             self._webconf.write(f)
-            return fname
+            return hglib.tounicode(fname)
         finally:
             f.close()
 
     @property
     def _webconf(self):
+        # type: () -> IniConfig
         """Selected webconf object"""
         return self._webconf_form.webconf
 
     @property
     def _singlerepo(self):
+        # type: () -> Optional[Text]
         """Return repository path if serving single repository"""
+        # TODO: The caller crashes if this returns None with:
+        #    `'ServeDialog' object has no attribute '_singlerepo'`
         # NOTE: we cannot use web-conf to serve single repository at '/' path
-        if len(self._webconf['paths']) != 1:
+        if len(self._webconf[b'paths']) != 1:
             return
-        path = self._webconf.get('paths', '/')
-        if path and '*' not in path:  # exactly a single repo (no wildcard)
-            return path
+        path = self._webconf.get(b'paths', b'/')
+        if path and b'*' not in path:  # exactly a single repo (no wildcard)
+            return hglib.tounicode(path)
 
     @pyqtSlot()
     def stop(self):
+        # type: () -> None
         """Stop web server"""
         self._agent.abortCommands()
 
     def reject(self):
+        # type: () -> None
         self.stop()
         super(ServeDialog, self).reject()
 
     def isstarted(self):
+        # type: () -> bool
         """Is the web server running?"""
         return self._agent.isBusy()
 
     @property
     def rooturl(self):
+        # type: () -> Text
         """Returns the root URL of the web server"""
         # TODO: scheme, hostname ?
         return 'http://localhost:%d' % self.port
 
     @property
     def port(self):
+        # type: () -> int
         """Port number of the web server"""
         return int(self._qui.port_edit.value())
 
     def setport(self, port):
+        # type: (int) -> None
         self._qui.port_edit.setValue(port)
 
     def keyPressEvent(self, event):
@@ -210,6 +249,7 @@ class ServeDialog(QDialog):
 
 
 def _asconfigliststr(value):
+    # type: (bytes) -> bytes
     r"""
     >>> _asconfigliststr(b'foo')
     b'foo'
@@ -227,6 +267,7 @@ def _asconfigliststr(value):
         return value
 
 def _readconfig(ui, repopath, webconfpath):
+    # type: (uimod.ui, Optional[bytes], Optional[bytes]) -> Tuple[uimod.ui, Optional[IniConfig]]
     """Create new ui and webconf object and read appropriate files"""
     lui = ui.copy()
     if webconfpath:
@@ -255,7 +296,9 @@ def _readconfig(ui, repopath, webconfpath):
         return lui, None
 
 def run(ui, *pats, **opts):
-    repopath = opts.get('root') or paths.find_root()
+    # type: (uimod.ui, Any, Any) -> ServeDialog
+    # TODO: No known caller provides **opts so bytes vs str is unknown
+    repopath = opts.get('root') or paths.find_root_bytes()
     webconfpath = opts.get('web_conf') or opts.get('webdir_conf')
 
     lui, webconf = _readconfig(ui, repopath, webconfpath)
