@@ -3,7 +3,7 @@
 # Copyright 2010 Steve Borho <steve@borho.org>
 #
 # This software may be used and distributed according to the terms of the
-# GNU General Public License version 2, incorporated herein by reference.
+# GNU General Public License version 2 or any later version.
 
 from __future__ import absolute_import, print_function
 
@@ -58,6 +58,10 @@ from mercurial import (
     util,
 )
 
+from mercurial.utils import (
+    stringutil,
+)
+
 from ..util import (
     editor,
     gpg,
@@ -204,10 +208,16 @@ class BoolRBGroup(QWidget):
 
     ## common APIs for all edit widgets
     def setValue(self, curvalue):
-        self.curvalue = curvalue
-        if curvalue == b'True':
+        # Store parsed value, so a compare of 1 to True isn't flagged as dirty
+        if curvalue:
+            value = pycompat.unicode(stringutil.parsebool(curvalue))
+            self.curvalue = hglib.fromunicode(value)
+        else:
+            self.curvalue = None
+
+        if self.curvalue == b'True':
             self.trueRB.setChecked(True)
-        elif curvalue == b'False':
+        elif self.curvalue == b'False':
             self.falseRB.setChecked(True)
         else:
             self.unspecRB.setChecked(True)
@@ -813,6 +823,10 @@ INFO = (
           '<b>Note</b>: This layouter colors edges using branch information '
           'and does not display graft edges, regardless of whether they are '
           'requested or not.')),
+    _fi(_('Copy Hash to Selection'), 'tortoisehg.copy_hash_selection',
+        genBoolRBGroup,
+        _('When a revision is selected in Workbench, copy hash to primary '
+          'selection clipboard (X11). Default: False')),
     )),
 ({'name': 'commit', 'label': _('Commit', 'config item'), 'icon': 'hg-commit'},
  (
@@ -1108,6 +1122,9 @@ INFO = (
     _fi(_('Diff Font'), 'tortoisehg.fontdiff', genFontEdit,
         _('Font used to display text differences. Default: monospace 10'),
         globalonly=True),
+    _fi(_('Editor Font'), 'tortoisehg.fonteditor', genFontEdit,
+        _('Font used for the text editor. Default: monospace 10'),
+        globalonly=True),
     _fi(_('ChangeLog Font'), 'tortoisehg.fontlog', genFontEdit,
         _('Font used to display changelog data. Default: monospace 10'),
         globalonly=True),
@@ -1581,7 +1598,7 @@ class SettingsForm(QWidget):
                         break
                 if val is None:
                     curvalue = False
-                elif len(val) and val[0] == '!':
+                elif val.startswith(b'!'):
                     curvalue = False
                 else:
                     curvalue = True
@@ -1816,7 +1833,6 @@ class SettingsForm(QWidget):
     def applyChangesForExtensions(self):
         assert 'extensions' in self.pages
         emitChanged = False
-        section = 'extensions'
         enabledexts = hglib.enabledextensions()
         for chk in self.pages['extensions'][2]:
             if (not emitChanged) and chk.isDirty():
@@ -1838,7 +1854,6 @@ class SettingsForm(QWidget):
 
     @pyqtSlot()
     def validateextensions(self):
-        section = 'extensions'
         enabledexts = hglib.enabledextensions()
         selectedexts = set(chk.opts['label']
                            for chk in self.pages['extensions'][2]
@@ -1846,17 +1861,17 @@ class SettingsForm(QWidget):
         invalidexts = hglib.validateextensions(selectedexts)
 
         def getinival(cpath):
-            if section not in self.ini:
+            if b'extensions' not in self.ini:
                 return None
             sect, key = pycompat.sysbytes(cpath).split(b'.', 1)
             try:
-                return self.ini[sect][key]
+                return self.ini.get(sect, key)
             except KeyError:
                 pass
 
         def changable(name, cpath):
             curval = getinival(cpath)
-            if curval not in ('', None):
+            if curval not in (b'', None):
                 # enabled or unspecified, official extensions only
                 return False
             elif name in enabledexts and curval is None:
@@ -1877,6 +1892,11 @@ class SettingsForm(QWidget):
                 sect, key = pycompat.sysbytes(cpath).split(b'.', 1)
                 if hglib.loadui().config(sect, key, None) is not None:
                     chk.setValue(True)
+                    # XXX: setValue(False) was called in refreshPage() prior to
+                    #      this.  SettingsCheckBox only sets its value if the
+                    #      current value is None, but this needs to be forced on
+                    #      because the dirty check looks at the checkbox state
+                    #      and will flag the item as dirty otherwise.
                     chk.curvalue = True
             else:
                 chk.setEnabled(True)
