@@ -61,6 +61,10 @@ from mercurial import (
     util,
 )
 
+from mercurial.utils import (
+    urlutil,
+)
+
 from ..util import (
     hglib,
     paths,
@@ -95,16 +99,17 @@ if hglib.TYPE_CHECKING:
 
 
 def parseurl(url):
-    # type: (pycompat.unicode) -> util.url
+    # _type: (pycompat.unicode) -> urlutil.url
+    # TODO: re-enable this after dropping 5.7 support
     assert isinstance(url, pycompat.unicode), repr(url)
-    return util.url(hglib.fromunicode(url))
+    return urlutil.url(hglib.fromunicode(url))
 
 def linkify(url):
     # type: (pycompat.unicode) -> pycompat.unicode
     assert isinstance(url, pycompat.unicode), repr(url)
-    u = util.url(hglib.fromunicode(url))
+    u = urlutil.url(hglib.fromunicode(url))
     if u.scheme in (b'http', b'https'):
-        safe = util.hidepassword(hglib.fromunicode(url))
+        safe = urlutil.hidepassword(hglib.fromunicode(url))
         return u'<a href="%s">%s</a>' % (url, hglib.tounicode(safe))
     elif u.scheme is None or u.scheme == b'file':
         return u'<a href="%s">%s</a>' % (url, hglib.tounicode(u.path))
@@ -389,8 +394,8 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
         fn = self.repo.vfs.join(b'hgrc')
         fn, cfg = hgrcutil.loadIniFile([fn], self)
         if b'paths' in cfg:
-            for alias in cfg[b'paths']:
-                self.paths[ alias ] = cfg[b'paths'][ alias ]
+            for alias, path in cfg.items(b'paths'):
+                self.paths[alias] = path
         tm = PathsModel(self.paths.items(), self)
         self.hgrctv.setModel(tm)
         sm = self.hgrctv.selectionModel()
@@ -405,7 +410,7 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
         known = set()  # type: Set[bytes]
         known.add(os.path.abspath(self.repo.root).lower())
         for path in self.paths.values():
-            if not util.hasscheme(path):
+            if not urlutil.hasscheme(path):
                 known.add(os.path.abspath(util.localpath(path)).lower())
             else:
                 known.add(path)
@@ -430,7 +435,7 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
             else:
                 continue
             for alias, path in ui.configitems(b'paths'):
-                if not util.hasscheme(path):
+                if not urlutil.hasscheme(path):
                     abs = os.path.abspath(util.localpath(path)).lower()
                 else:
                     abs = path
@@ -691,7 +696,7 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
             if self.repo.ui.configbool(b'insecurehosts', u.host):
                 cmdline.append('--insecure')
             if u.user:
-                cleanurl = util.removeauth(lurl)
+                cleanurl = urlutil.removeauth(lurl)
                 res = httpconnection.readauthforuri(self.repo.ui, cleanurl,
                                                     u.user)
                 if res:
@@ -1305,7 +1310,7 @@ class SaveDialog(QDialog):
                 if not os.path.exists(repo.wjoin(subname)):
                     continue
                 defaultsubpath = ctx.substate[subname][0]
-                pathurl = util.url(path)
+                pathurl = urlutil.url(path)
                 if pathurl.scheme:
                     subpath = bytes(pathurl).rstrip(b'/') + b'/' + subname
                 else:
@@ -1338,11 +1343,11 @@ class SaveDialog(QDialog):
     @pyqtSlot(bool)
     def _removeAuthData(self, showclean):
         if showclean:
-            cleanurl = hglib.tounicode(util.removeauth(self.origurl))
+            cleanurl = hglib.tounicode(urlutil.removeauth(self.origurl))
             self.urllabel.setText(cleanurl)
             self.urlentry.setText(cleanurl)
         else:
-            safeurl = hglib.tounicode(util.hidepassword(self.origurl))
+            safeurl = hglib.tounicode(urlutil.hidepassword(self.origurl))
             self.urllabel.setText(safeurl)
             self.urlentry.setText(hglib.tounicode(self.origurl))
 
@@ -1379,7 +1384,7 @@ class SecureDialog(QDialog):
                             ~Qt.WindowContextHelpButtonHint)
 
         # if the already user has an [auth] configuration for this URL, use it
-        cleanurl = util.removeauth(hglib.fromunicode(urlu))
+        cleanurl = urlutil.removeauth(hglib.fromunicode(urlu))
         res = httpconnection.readauthforuri(repo.ui, cleanurl, u.user)  # type: Optional[Tuple[bytes, Dict[bytes, bytes]]]
         self.schemes = None  # type: Optional[bytes]
         if res:
@@ -1416,8 +1421,7 @@ class SecureDialog(QDialog):
         self.fprintentry = le = QLineEdit(hglib.tounicode(fprint))
         self.fprintradio.toggled.connect(self.fprintentry.setEnabled)
         self.fprintentry.setEnabled(False)
-        if hasattr(le, 'setPlaceholderText'): # Qt >= 4.7
-            le.setPlaceholderText(_('### host certificate fingerprint ###'))
+        le.setPlaceholderText(_('### host certificate fingerprint ###'))
         hbox.addWidget(le)
         self._querybutton = qb = QPushButton(_('Query'))
         qb.clicked.connect(self._queryFingerprint)
@@ -1648,7 +1652,7 @@ class PathsModel(QAbstractTableModel):
         self.headers = (_('Alias'), _('URL'))
         self.rows = []
         for alias, path in sorted(pathlist):
-            safepath = util.hidepassword(path)
+            safepath = urlutil.hidepassword(path)
             ualias = hglib.tounicode(alias)
             usafepath = hglib.tounicode(safepath)
             self.rows.append([ualias, usafepath, path])
