@@ -3,7 +3,7 @@
 # Copyright 2010 Yuki KODAMA <endflow.net@gmail.com>
 #
 # This software may be used and distributed according to the terms of the
-# GNU General Public License version 2, incorporated herein by reference.
+# GNU General Public License version 2 or any later version.
 
 from __future__ import absolute_import
 
@@ -27,10 +27,6 @@ from .qtgui import (
     QWidget,
 )
 
-from mercurial import (
-    scmutil,
-)
-
 from ..util import (
     hglib,
     i18n,
@@ -41,11 +37,27 @@ from . import (
     qtlib,
 )
 
+if hglib.TYPE_CHECKING:
+    from typing import (
+        Any,
+        Dict,
+        Optional,
+        Text,
+        Union,
+    )
+    from .qtgui import (
+        QIcon,
+    )
+    from .thgrepo import (
+        RepoAgent,
+    )
+
 keep = i18n.keepgettext()
 
 class TagDialog(QDialog):
 
-    def __init__(self, repoagent, tag='', rev='tip', parent=None, opts=None):
+    def __init__(self, repoagent, tag=b'', rev='tip', parent=None, opts=None):
+        # type: (RepoAgent, bytes, Text, Optional[QWidget], Dict[Text, Union[bool, bytes]]) -> None
         super(TagDialog, self).__init__(parent)
         if opts is None:
             opts = {}
@@ -71,9 +83,9 @@ class TagDialog(QDialog):
         base.addWidget(formwidget)
 
         repo = repoagent.rawRepo()
-        ctx = scmutil.revsymbol(repo, hglib.fromunicode(rev))
+        ctx = hglib.revsymbol(repo, hglib.fromunicode(rev))
         form.addRow(_('Revision:'), QLabel('%d (%s)' % (ctx.rev(), ctx)))
-        self.rev = ctx.rev()
+        self.rev = ctx.rev()  # type: int
 
         ### tag combo
         self.tagCombo = QComboBox()
@@ -159,6 +171,7 @@ class TagDialog(QDialog):
 
     @pyqtSlot()
     def refresh(self):
+        # type: () -> None
         """ update display on dialog with recent repo data """
         cur = self.tagCombo.currentText()
 
@@ -175,6 +188,7 @@ class TagDialog(QDialog):
 
     @pyqtSlot()
     def updateStates(self):
+        # type: () -> None
         """ update bottom button sensitives based on rev and tag """
         tagu = self.tagCombo.currentText()
         tag = hglib.fromunicode(tagu)
@@ -184,10 +198,10 @@ class TagDialog(QDialog):
             exists = tag in self.repo.tags()
             if exists:
                 tagtype = self.repo.tagtype(tag)
-                islocal = 'local' == tagtype
+                islocal = b'local' == tagtype
                 try:
                     ctx = self.repo[self.repo.tags()[tag]]
-                    trev = ctx.rev()
+                    trev = ctx.rev()  # type: int
                     thash = str(ctx)
                 except:
                     trev, thash, local = 0, '????????', ''
@@ -222,11 +236,13 @@ class TagDialog(QDialog):
             self.tagRevLabel.clear()
 
     def customMessageToggle(self, checked):
+        # type: (bool) -> None
         self.customTextLineEdit.setEnabled(checked)
         if checked:
             self.customTextLineEdit.setFocus()
 
     def show_options(self, visible):
+        # type: (bool) -> None
         self.localCheckBox.setVisible(visible)
         self.replaceCheckBox.setVisible(visible)
         self.englishCheckBox.setVisible(visible)
@@ -234,15 +250,18 @@ class TagDialog(QDialog):
         self.customTextLineEdit.setVisible(visible)
 
     def set_status(self, text, icon):
+        # type: (Text, Optional[Union[bool, Text, QIcon]]) -> None
         self.status.setVisible(True)
         self.sep.setVisible(True)
         self.status.set_status(text, icon)
 
     def clear_status(self):
+        # type: () -> None
         self.status.setHidden(True)
         self.sep.setHidden(True)
 
     def _runTag(self, tagname, **opts):
+        # type: (Text, Any) -> None
         if not self._cmdsession.isFinished():
             self.set_status(_('Repository command still running'), False)
             return
@@ -254,12 +273,14 @@ class TagDialog(QDialog):
 
     @pyqtSlot(int)
     def _onTagFinished(self, ret):
+        # type: (int) -> None
         if ret == 0:
             self.set_status(self._finishmsg, True)
         else:
             self.set_status(self._cmdsession.errorString(), False)
 
     def onAddTag(self):
+        # type: () -> None
         tagu = self.tagCombo.currentText()
         tag = hglib.fromunicode(tagu)
         local = self.localCheckBox.isChecked()
@@ -293,11 +314,12 @@ class TagDialog(QDialog):
         user = qtlib.getCurrentUsername(self, self.repo)
         if not user:
             return
-        self._runTag(tagu, rev=self.rev, user=hglib.tounicode(user),
+        self._runTag(tagu, rev=self.rev, user=user,
                      local=local, force=force, message=message,
                      finishmsg=finishmsg)
 
     def onRemoveTag(self):
+        # type: () -> None
         tagu = self.tagCombo.currentText()
         local = self.localCheckBox.isChecked()
         force = self.replaceCheckBox.isChecked()
@@ -318,6 +340,7 @@ class TagDialog(QDialog):
                      finishmsg=finishmsg)
 
     def reject(self):
+        # type: () -> None
         if not self._cmdsession.isFinished():
             self.set_status(_('Repository command still running'), False)
             return
