@@ -55,6 +55,7 @@ if hglib.TYPE_CHECKING:
         List,
         Optional,
         Text,
+        Tuple,
         Union,
     )
     from .qtgui import (
@@ -67,9 +68,9 @@ _lfregex = re.compile(br'^\.hglf/')
 
 # thgrepo.repository() will be deprecated
 def repository(_ui=None, path=b''):
-    '''Returns a subclassed Mercurial repository to which new
+    """Returns a subclassed Mercurial repository to which new
     THG-specific methods have been added. The repository object
-    is obtained using mercurial.hg.repository()'''
+    is obtained using mercurial.hg.repository()"""
     assert isinstance(path, bytes), repr(path)
     if path not in _repocache:
         if _ui is None:
@@ -195,15 +196,15 @@ class RepoWatcher(QObject):
         self._pollFsChanges()
 
     def _pollFsChanges(self):
-        '''Catch writes or deletions of files, or writes to .hg/ folder,
-        most importantly lock files'''
+        """Catch writes or deletions of files, or writes to .hg/ folder,
+        most importantly lock files"""
         self._pollStatus()
         # filesystem monitor may be stopped inside _pollStatus()
         if self.isMonitoring():
             self._addMissingPaths()
 
     def _addMissingPaths(self):
-        'Add files to watcher that may have been added or replaced'
+        """Add files to watcher that may have been added or replaced"""
         existing = [f for f, (_flag, watched) in self._filesmap.items()
                     if watched and f in self._laststats]
         files = [pycompat.unicode(f) for f in self._fswatcher.files()]
@@ -258,7 +259,6 @@ class RepoWatcher(QObject):
     def _fixState(self):
         """Update paths to be checked and record state of new paths"""
         repo = self._repo
-        q = getattr(repo, 'mq', None)
         newfilesmap = {
             repo.vfs.join(b'bookmarks'): (LogChanged, False),
             repo.vfs.join(b'bookmarks.current'): (LogChanged, False),
@@ -270,6 +270,7 @@ class RepoWatcher(QObject):
             repo.svfs.join(b'obsstore'): (LogChanged, False),
             repo.svfs.join(b'phaseroots'): (LogChanged, False),
             }
+        q = getattr(repo, 'mq', None)
         if q:
             newfilesmap.update({
                 q.join(b'guards'): (LogChanged, True),
@@ -450,21 +451,27 @@ class RepoAgent(QObject):
         return hglib.tounicode(self._repo.root)
 
     def configBool(self, section, name, default=hgconfig.UNSET_DEFAULT):
+        # type: (Text, Text, bool) -> bool
         return self._config.configBool(section, name, default)
 
     def configInt(self, section, name, default=hgconfig.UNSET_DEFAULT):
+        # type: (Text, Text, int) -> int
         return self._config.configInt(section, name, default)
 
     def configString(self, section, name, default=hgconfig.UNSET_DEFAULT):
+        # type: (Text, Text, Text) -> Text
         return self._config.configString(section, name, default)
 
     def configStringList(self, section, name, default=hgconfig.UNSET_DEFAULT):
+        # type: (Text, Text, List[Text]) -> List[Text]
         return self._config.configStringList(section, name, default)
 
     def configStringItems(self, section):
+        # type: (Text) -> List[Tuple[Text, Text]]
         return self._config.configStringItems(section)
 
     def hasConfig(self, section, name):
+        # type: (Text, Text) -> bool
         return self._config.hasConfig(section, name)
 
     def displayName(self):
@@ -559,6 +566,10 @@ class RepoAgent(QObject):
     @pyqtSlot(int)
     def _onRepositoryChanged(self, flags):
         self._repo.thginvalidate()
+        if flags & LogChanged:
+            # only call thg_reload_hggit_data() when there are new elements in
+            # the commit log
+            self._repo.thg_reload_hggit_data()
         # ignore signal that just contains internal flags
         if flags & ~WorkingStateChanged:
             self._emitRepositoryChanged(flags)
@@ -789,12 +800,12 @@ def _extendrepo(repo):
     class thgrepository(repo.__class__):
 
         def __getitem__(self, changeid):
-            '''Extends Mercurial's standard __getitem__() method to
+            """Extends Mercurial's standard __getitem__() method to
             a) return a thgchangectx with additional methods
             b) return a patchctx if changeid is the name of an MQ
             unapplied patch
             c) return a patchctx if changeid is an absolute patch path
-            '''
+            """
 
             # Mercurial's standard changectx() (rather, lookup())
             # implies that tags and branch names live in the same namespace.
@@ -818,7 +829,7 @@ def _extendrepo(repo):
             return changectx
 
         def hgchangectx(self, changeid):
-            '''Returns unwrapped changectx or workingctx object'''
+            """Returns unwrapped changectx or workingctx object"""
             # This provides temporary workaround for troubles caused by class
             # extension: e.g. changectx(n) != thgchangectx(n).
             # thgrepository and thgchangectx should be removed in some way.
@@ -831,9 +842,9 @@ def _extendrepo(repo):
 
         @localrepo.unfilteredpropertycache
         def thgmqunappliedpatches(self):
-            '''Returns a list of (patch name, patch path) of all self's
+            """Returns a list of (patch name, patch path) of all self's
             unapplied MQ patches, in patch series order, first unapplied
-            patch first.'''
+            patch first."""
             if not hasattr(self, 'mq'): return []
 
             q = self.mq
@@ -843,22 +854,31 @@ def _extendrepo(repo):
 
         @localrepo.unfilteredpropertycache
         def _thgmqpatchnames(self):
-            '''Returns all tag names used by MQ patches. Returns []
-            if MQ not in use.'''
+            """Returns all tag names used by MQ patches. Returns []
+            if MQ not in use."""
             return hglib.getmqpatchtags(self)
 
         @property
         def thgactivemqname(self):
-            '''Currenty-active qqueue name (see hgext/mq.py:qqueue)'''
+            """Currenty-active qqueue name (see hgext/mq.py:qqueue)"""
             return hglib.getcurrentqqueue(self)
 
         @localrepo.unfilteredpropertycache
         def _uifiles(self):
             cfg = self.ui._ucfg
             files = set()
-            for line in cfg._source.values():
-                f = line.rsplit(b':', 1)[0]
-                files.add(f)
+
+            for section in cfg:
+                for item in cfg.items(section):
+                    src = cfg.source(section, item[0])
+
+                    # Skip sourceless items like ui.{verbose,debug}
+                    if not src:
+                        continue
+
+                    f = src.rsplit(b':', 1)[0]
+                    files.add(f)
+
             files.add(self.vfs.join(b'hgrc'))
             return files
 
@@ -929,15 +949,15 @@ def _extendrepo(repo):
             return installed
 
         def uifiles(self):
-            'Returns complete list of config files'
+            """Returns complete list of config files"""
             return self._uifiles
 
         def extensions(self):
-            'Returns list of extensions enabled in this repository'
+            """Returns list of extensions enabled in this repository"""
             return self._exts
 
         def thgmqtag(self, tag):
-            'Returns true if `tag` marks an applied MQ patch'
+            """Returns true if `tag` marks an applied MQ patch"""
             return tag in self._thgmqpatchnames
 
         def thgshelves(self):
@@ -961,7 +981,7 @@ def _extendrepo(repo):
             f.close()
 
         def thginvalidate(self):
-            'Should be called when mtime of repo store/dirstate are changed'
+            """Should be called when mtime of repo store/dirstate are changed"""
             self.invalidatedirstate()
 
             if not isinstance(repo, bundlerepo.bundlerepository):
@@ -974,8 +994,30 @@ def _extendrepo(repo):
                 if localrepo.hasunfilteredcache(self, a):
                     delattr(self.unfiltered(), a)
 
+        def thg_reload_hggit_data(self):
+            # type: () -> None
+            """Reload hg-git specific data structures.
+
+            This function forces the reload of hg-git specific files, so that
+            the UI can show updated values (for example, the git hashes of newly
+            pulled commits, or the fake local tags representing remote refs).
+
+            The data structures that are reloaded are:
+            - the map between mercurial and git commit hashes
+            - the git tags
+            - the remote git refs
+
+            If the hg-git extension is not loaded, nothing is done.
+            """
+            if b'hggit' not in self.extensions():
+                return
+            self.ui.debug(b'reloading hg-git specific data\n')
+            self.githandler.load_map()
+            self.githandler.load_tags()
+            self.githandler.load_remote_refs()
+
         def invalidateui(self):
-            'Should be called when mtime of ui files are changed'
+            """Should be called when mtime of ui files are changed"""
             origui = self.ui
             self.ui = hglib.loadui()
             self.ui.readconfig(self.vfs.join(b'hgrc'))
@@ -985,8 +1027,8 @@ def _extendrepo(repo):
                     delattr(self.unfiltered(), a)
 
         def thgbackup(self, path):
-            '''Make a backup of the given file in the directory "Trashcan" if
-            it exists'''
+            """Make a backup of the given file in the directory "Trashcan" if
+            it exists"""
             # The backup name will be similar to the orginal file name plus
             # '.bak' and characters to make it unique
             trashcan = hglib.tounicode(self.vfs.join(b'Trashcan'))
@@ -1045,12 +1087,12 @@ def _createchangectxcls(parentcls):
             return srepo
 
         def thgtags(self):
-            '''Returns all unhidden tags for self'''
+            """Returns all unhidden tags for self"""
             htlist = self._repo._thghiddentags
             return [tag for tag in self.tags() if tag not in htlist]
 
         def _thgmqpatchtags(self):
-            '''Returns the set of self's tags which are MQ patch names'''
+            """Returns the set of self's tags which are MQ patch names"""
             mytags = set(self.tags())
             patchtags = self._repo._thgmqpatchnames
             result = mytags.intersection(patchtags)
@@ -1058,20 +1100,20 @@ def _createchangectxcls(parentcls):
             return result
 
         def thgmqappliedpatch(self):
-            '''True if self is an MQ applied patch'''
+            """True if self is an MQ applied patch"""
             return self.rev() is not None and bool(self._thgmqpatchtags())
 
         def thgmqunappliedpatch(self):
             return False
 
         def thgmqpatchname(self):
-            '''Return self's MQ patch name. AssertionError if self not an MQ patch'''
+            """Return self's MQ patch name. AssertionError if self not an MQ patch"""
             patchtags = self._thgmqpatchtags()
             assert len(patchtags) == 1, "thgmqpatchname: called on non-mq patch"
             return list(patchtags)[0]
 
         def thgmqoriginalparent(self):
-            '''The revisionid of the original patch parent'''
+            """The revisionid of the original patch parent"""
             if not self.thgmqunappliedpatch() and not self.thgmqappliedpatch():
                 return ''
             try:
@@ -1144,7 +1186,7 @@ def recursiveMergeStatus(repo):
         pass
 
 def relatedRepositories(repoid):
-    'Yields root paths for local related repositories'
+    """Yields root paths for local related repositories"""
     from tortoisehg.hgqt import reporegistry, repotreemodel
     if repoid == node.nullid:  # empty repositories shouldn't be related
         return
