@@ -5,7 +5,7 @@
 # Copyright 2010 Yuki KODAMA <endflow.net@gmail.com>
 #
 # This software may be used and distributed according to the terms of the
-# GNU General Public License version 2, incorporated herein by reference.
+# GNU General Public License version 2 or any later version.
 
 from __future__ import absolute_import
 
@@ -25,7 +25,6 @@ from .qtgui import (
 from mercurial import (
     error,
     pycompat,
-    scmutil,
 )
 
 from ..util import hglib
@@ -38,9 +37,27 @@ from . import (
     resolve,
 )
 
+if hglib.TYPE_CHECKING:
+    from typing import (
+        Any,
+        Dict,
+        Optional,
+        Text,
+        Union,
+    )
+    from mercurial import context
+    from .qtcore import QSettings
+    from .qtgui import QWidget
+    from .thgrepo import RepoAgent
+
+    UpdateOpts = Dict[Text, Any]
+
+
 class UpdateWidget(cmdui.AbstractCmdWidget):
 
     def __init__(self, repoagent, rev=None, parent=None, opts=None):
+        # type: (RepoAgent, Optional[Union[Text, pycompat.unicode]], Optional[QWidget], Optional[UpdateOpts]) -> None
+        # TODO: unify the `rev` type
         super(UpdateWidget, self).__init__(parent)
         if opts is None:
             opts = {}
@@ -63,9 +80,9 @@ class UpdateWidget(cmdui.AbstractCmdWidget):
 
         # always include integer revision
         if rev:
-            assert isinstance(rev, pycompat.unicode), repr(rev)
+            assert isinstance(rev, (pycompat.unicode, str)), repr(rev)
             try:
-                ctx = scmutil.revsymbol(self.repo, hglib.fromunicode(rev))
+                ctx = hglib.revsymbol(self.repo, hglib.fromunicode(rev))
                 if isinstance(ctx.rev(), int):  # could be patch name
                     combo.addItem(str(ctx.rev()))
             except error.RepoLookupError:
@@ -156,6 +173,7 @@ class UpdateWidget(cmdui.AbstractCmdWidget):
             self.rev_combo.lineEdit().selectAll()
 
     def readSettings(self, qs):
+        # type: (QSettings) -> None
         self.merge_chk.setChecked(qtlib.readBool(qs, 'merge'))
         self.autoresolve_chk.setChecked(
             self._repoagent.configBool('tortoisehg', 'autoresolve',
@@ -166,6 +184,7 @@ class UpdateWidget(cmdui.AbstractCmdWidget):
         self.optexpander.set_expanded(self.hiddenSettingIsChecked())
 
     def writeSettings(self, qs):
+        # type: (QSettings) -> None
         qs.setValue('merge', self.merge_chk.isChecked())
         qs.setValue('autoresolve', self.autoresolve_chk.isChecked())
         qs.setValue('verbose', self.verbose_chk.isChecked())
@@ -175,22 +194,25 @@ class UpdateWidget(cmdui.AbstractCmdWidget):
         return self._repoagent.rawRepo()
 
     def hiddenSettingIsChecked(self):
+        # type: () -> bool
         return (self.merge_chk.isChecked()
                 or self.autoresolve_chk.isChecked())
 
     @pyqtSlot()
     def update_info(self):
+        # type: () -> None
         self.p1_info.update(self.ctxs[0].node())
         merge = len(self.ctxs) == 2
         if merge:
             self.p2_info.update(self.ctxs[1].node())
         new_rev = hglib.fromunicode(self.rev_combo.currentText())
-        if new_rev == 'null':
+        if new_rev == b'null':
             self.target_info.setText(_('remove working directory'))
             self.commandChanged.emit()
             return
         try:
-            new_ctx = scmutil.revsymbol(self.repo, new_rev)
+            new_ctx = hglib.revsymbol(self.repo, new_rev)
+
             if not merge and new_ctx.rev() == self.ctxs[0].rev() \
                     and not new_ctx.bookmarks():
                 self.target_info.setText(_('(same as parent)'))
@@ -199,7 +221,7 @@ class UpdateWidget(cmdui.AbstractCmdWidget):
             # only show the path combo when there are multiple paths
             # and the target revision has subrepos
             showpathcombo = self.path_combo.count() > 1 and \
-                '.hgsubstate' in new_ctx
+                b'.hgsubstate' in new_ctx
             self.path_combo_label.setVisible(showpathcombo)
             self.path_combo.setVisible(showpathcombo)
         except (error.LookupError, error.RepoError, EnvironmentError):
@@ -207,29 +229,34 @@ class UpdateWidget(cmdui.AbstractCmdWidget):
         self.commandChanged.emit()
 
     def canRunCommand(self):
+        # type: () -> bool
         new_rev = hglib.fromunicode(self.rev_combo.currentText())
         try:
-            new_ctx = scmutil.revsymbol(self.repo, new_rev)
+            new_ctx = hglib.revsymbol(self.repo, new_rev)
         except (error.LookupError, error.RepoError, EnvironmentError):
             return False
+
         return (self.discard_chk.isChecked()
                 or len(self.ctxs) == 2
                 or new_ctx.rev() != self.ctxs[0].rev()
                 or bool(new_ctx.bookmarks()))
 
     def runCommand(self):
+        # type: () -> cmdcore.CmdSession
         cmdline = ['update']
         if self.verbose_chk.isChecked():
             cmdline += ['--verbose']
         cmdline += ['--config', 'ui.merge=internal:' +
                     (self.autoresolve_chk.isChecked() and 'merge' or 'fail')]
-        rev = hglib.fromunicode(self.rev_combo.currentText())
+        rev = self.rev_combo.currentText()  # type: Text
 
         activatebookmarkmode = self._repoagent.configString(
             'tortoisehg', 'activatebookmarks')
         if activatebookmarkmode != 'never':
-            bookmarks = scmutil.revsymbol(self.repo, rev).bookmarks()
-            if bookmarks and rev not in bookmarks:
+            brev = hglib.fromunicode(rev)  # type: bytes
+            bookmarks = hglib.revsymbol(self.repo, brev).bookmarks()
+
+            if bookmarks and brev not in bookmarks:
                 # The revision that we are updating into has bookmarks,
                 # but the user did not refer to the revision by one of them
                 # (probably used a revision number or hash)
@@ -247,10 +274,9 @@ class UpdateWidget(cmdui.AbstractCmdWidget):
                               'it?<br></b>'
                               '<i>You can disable this prompt by configuring '
                               'Settings/Workbench/Activate Bookmarks</i>') \
-                            % (hglib.tounicode(rev),
-                               hglib.tounicode(bookmarks[0])))
+                            % (rev, hglib.tounicode(bookmarks[0])))
                     if activatebookmark:
-                        selectedbookmark = bookmarks[0]
+                        selectedbookmark = hglib.tounicode(bookmarks[0])
                 else:
                     # Even in auto mode, when there is more than one bookmark
                     # we must ask the user which one must be activated
@@ -263,14 +289,15 @@ class UpdateWidget(cmdui.AbstractCmdWidget):
                           'activate any of them.<p><p>'
                           '<i>You can disable this prompt by configuring '
                           'Settings/Workbench/Activate Bookmarks</i><p>') \
-                        % (hglib.tounicode(rev), len(bookmarks)),
-                        self, bookmarks, hglib.activebookmark(self.repo)).run()
+                        % (rev, len(bookmarks)),
+                        self, [hglib.tounicode(b) for b in bookmarks],
+                        hglib.tounicode(hglib.activebookmark(self.repo))).run()
                 if selectedbookmark:
                     rev = selectedbookmark
                 else:
                     activebookmark = hglib.activebookmark(self.repo)
-                    if (activebookmark and scmutil.revsymbol(self.repo, rev)
-                        == scmutil.revsymbol(self.repo, activebookmark)):
+                    if (activebookmark and hglib.revsymbol(self.repo, brev)
+                        == hglib.revsymbol(self.repo, activebookmark)):
                         deactivatebookmark = qtlib.QuestionMsgBox(
                             _('Deactivate current bookmark?'),
                             _('Do you really want to deactivate the <i>%s</i> '
@@ -294,21 +321,21 @@ class UpdateWidget(cmdui.AbstractCmdWidget):
             # We must tell mercurial to pull any missing repository
             # revisions from the selected path. The only way to do so is
             # to temporarily set the default path to the selected path URL
-            pullpath = hglib.fromunicode(
-                self.path_combo.itemData(self.path_combo.currentIndex()))
+            pullpath = self.path_combo.itemData(self.path_combo.currentIndex())
             cmdline.append('--config')
             cmdline.append('paths.default=%s' % pullpath)
 
         if self.discard_chk.isChecked():
             cmdline.append('--clean')
         else:
-            cur = self.repo.hgchangectx(b'.')
+            cur = self.repo.hgchangectx(b'.')  # type: context.changectx
             try:
                 node = self.repo.hgchangectx(
-                    scmutil.revsymbol(self.repo, rev).rev())
+                    hglib.revsymbol(self.repo, hglib.fromunicode(rev)).rev())
             except (error.LookupError, error.RepoError, EnvironmentError):
                 return cmdcore.nullCmdSession()
             def isclean():
+                # type: () -> bool
                 '''whether WD is changed'''
                 try:
                     wc = self.repo[None]
@@ -321,16 +348,19 @@ class UpdateWidget(cmdui.AbstractCmdWidget):
                     return False
                 return True
             def ismergedchange():
+                # type: () -> bool
                 '''whether the local changes are merged (have 2 parents)'''
                 wc = self.repo[None]
                 return len(wc.parents()) == 2
             def islocalmerge(p1, p2, clean=None):
+                # type: (context.changectx, context.changectx, Optional[bool]) -> bool
                 if clean is None:
                     clean = isclean()
                 pa = p1.ancestor(p2)
                 return not clean \
                     and (p1.rev() == pa.rev() or p2.rev() == pa.rev())
             def confirmupdate():
+                # type: () -> Optional[Text]
                 msg = _('Detected uncommitted local changes in working tree.\n'
                         'Please select to continue:\n')
                 data = {'discard': (_('&Discard'),
@@ -360,8 +390,7 @@ class UpdateWidget(cmdui.AbstractCmdWidget):
                 dlg.setDefaultButton(QMessageBox.Cancel)
                 dlg.setText(msg)
                 dlg.exec_()
-                clicked = buttonnames.get(dlg.clickedButton())
-                return clicked
+                return buttonnames.get(dlg.clickedButton())
 
             # If merge-by-default, we want to merge whenever possible,
             # without prompting user (similar to command-line behavior)
@@ -386,22 +415,24 @@ class UpdateWidget(cmdui.AbstractCmdWidget):
             elif not islocalmerge(cur, node, clean):
                 cmdline.append('--merge')
 
-        cmdline = pycompat.maplist(hglib.tounicode, cmdline)
         return self._repoagent.runCommand(cmdline, self)
 
     @pyqtSlot(bool)
     def show_options(self, visible):
+        # type: (bool) -> None
         self.merge_chk.setVisible(visible)
         self.autoresolve_chk.setVisible(visible)
 
     @pyqtSlot(int)
     def _updatePathComboTooltip(self, idx):
+        # type: (int) -> None
         self.path_combo.setToolTip(self.path_combo.itemData(idx))
 
 
 class UpdateDialog(cmdui.CmdControlDialog):
 
     def __init__(self, repoagent, rev=None, parent=None, opts=None):
+        # type: (RepoAgent, Optional[Union[Text, pycompat.unicode]], Optional[QWidget], Optional[UpdateOpts]) -> None
         super(UpdateDialog, self).__init__(parent)
         if opts is None:
             opts = {}
@@ -416,6 +447,7 @@ class UpdateDialog(cmdui.CmdControlDialog):
 
     @pyqtSlot(int)
     def _checkMergeConflicts(self, ret):
+        # type: (int) -> None
         if ret != 1:
             return
         qtlib.InfoMsgBox(_('Merge caused file conflicts'),
