@@ -3,7 +3,7 @@
 # Copyright 2010 Yuya Nishihara <yuya@tcha.org>
 #
 # This software may be used and distributed according to the terms of the
-# GNU General Public License version 2, incorporated herein by reference.
+# GNU General Public License version 2 or any later version.
 
 from __future__ import absolute_import
 
@@ -40,12 +40,30 @@ from ..util.i18n import _
 from . import qtlib
 from .webconf_ui import Ui_WebconfForm
 
+if hglib.TYPE_CHECKING:
+    from typing import (
+        Any,
+        Iterable,
+        List,
+        Optional,
+        Text,
+        Tuple,
+    )
+    from .qtcore import (
+        QObject,
+    )
+    from ..util.typelib import (
+        IniConfig
+    )
+
+
 _FILE_FILTER = ';;'.join([_('Config files (*.conf *.config *.ini)'),
                           _('All files (*)')])
 
 class WebconfForm(QWidget):
     """Widget to show/edit webconf"""
     def __init__(self, parent=None, webconf=None):
+        # type: (Optional[QWidget], Optional[IniConfig]) -> None
         super(WebconfForm, self).__init__(parent, acceptDrops=True)
         self._qui = Ui_WebconfForm()
         self._qui.setupUi(self)
@@ -58,6 +76,7 @@ class WebconfForm(QWidget):
         self._updateform()
 
     def _initicons(self):
+        # type: () -> None
         def setstdicon(w, name):
             w.setIcon(self.style().standardIcon(name))
 
@@ -86,6 +105,7 @@ class WebconfForm(QWidget):
             return pycompat.unicode(m.urls()[0].toLocalFile())
 
     def setwebconf(self, webconf):
+        # type: (IniConfig) -> None
         """set current webconf object"""
         path = hglib.tounicode(getattr(webconf, 'path', None) or '')
         i = self._qui.path_edit.findText(path)
@@ -96,6 +116,7 @@ class WebconfForm(QWidget):
 
     @property
     def webconf(self):
+        # type: () -> IniConfig
         """current webconf object"""
         def curconf(w):
             i = w.currentIndex()
@@ -114,12 +135,14 @@ class WebconfForm(QWidget):
 
     @pyqtSlot()
     def _updateview(self):
+        # type: () -> None
         m = WebconfModel(config=self.webconf, parent=self)
         self._qui.repos_view.setModel(m)
         self._qui.repos_view.selectionModel().currentChanged.connect(
             self._updateform)
 
     def _updateform(self):
+        # type: () -> None
         """Update availability of each widget"""
         self._qui.repos_view.setEnabled(hasattr(self.webconf, 'write'))
         self._qui.add_button.setEnabled(hasattr(self.webconf, 'write'))
@@ -132,6 +155,7 @@ class WebconfForm(QWidget):
 
     @pyqtSlot()
     def on_open_button_clicked(self):
+        # type: () -> None
         path, _filter = QFileDialog.getOpenFileName(
             self, _('Open hgweb config'),
             getattr(self.webconf, 'path', None) or '', _FILE_FILTER)
@@ -139,6 +163,7 @@ class WebconfForm(QWidget):
             self.openwebconf(path)
 
     def openwebconf(self, path):
+        # type: (Text) -> None
         """load the specified webconf file"""
         path = hglib.fromunicode(path)
         c = wconfig.readfile(path)
@@ -147,6 +172,7 @@ class WebconfForm(QWidget):
 
     @pyqtSlot()
     def on_save_button_clicked(self):
+        # type: () -> None
         path, _filter = QFileDialog.getSaveFileName(
             self, _('Save hgweb config'),
             getattr(self.webconf, 'path', None) or '', _FILE_FILTER)
@@ -154,13 +180,14 @@ class WebconfForm(QWidget):
             self.savewebconf(path)
 
     def savewebconf(self, path):
+        # type: (Text) -> None
         """save current webconf to the specified file"""
-        path = hglib.fromunicode(path)
-        wconfig.writefile(self.webconf, path)
+        wconfig.writefile(self.webconf, hglib.fromunicode(path))
         self.openwebconf(path)  # reopen in case file path changed
 
     @pyqtSlot()
     def _addpathmap(self, path=None, localpath=None):
+        # type: (Optional[Text], Optional[Text]) -> None
         path, localpath = _PathDialog.getaddpathmap(
             self, path=path, localpath=localpath,
             invalidpaths=self._webconfmodel.paths)
@@ -169,10 +196,12 @@ class WebconfForm(QWidget):
 
     @pyqtSlot()
     def on_edit_button_clicked(self):
+        # type: () -> None
         self.on_repos_view_doubleClicked(self._qui.repos_view.currentIndex())
 
     @pyqtSlot(QModelIndex)
     def on_repos_view_doubleClicked(self, index):
+        # type: (QModelIndex) -> None
         assert index.isValid()
         origpath, origlocalpath = self._webconfmodel.getpathmapat(index.row())
         path, localpath = _PathDialog.geteditpathmap(
@@ -189,6 +218,7 @@ class WebconfForm(QWidget):
 
     @pyqtSlot()
     def on_remove_button_clicked(self):
+        # type: () -> None
         index = self._qui.repos_view.currentIndex()
         assert index.isValid()
         path, _localpath = self._webconfmodel.getpathmapat(index.row())
@@ -198,6 +228,7 @@ class _PathDialog(QDialog):
     """Dialog to add/edit path mapping"""
     def __init__(self, title, acceptlabel, path=None, localpath=None,
                  invalidpaths=None, parent=None):
+        # type: (Text, Text, Optional[Text], Optional[Text], Optional[Iterable[Text]], Optional[QWidget]) -> None
         super(_PathDialog, self).__init__(parent)
         self.setWindowFlags((self.windowFlags() | Qt.WindowMinimizeButtonHint)
                             & ~Qt.WindowContextHelpButtonHint)
@@ -212,6 +243,7 @@ class _PathDialog(QDialog):
         self._updateform()
 
     def _initfields(self):
+        # type: () -> None
         """initialize input fields"""
         def addfield(key, label, *extras):
             edit = QLineEdit(self)
@@ -233,6 +265,7 @@ class _PathDialog(QDialog):
         self._localpath_browse_button.clicked.connect(self._browse_localpath)
 
     def _initbuttons(self, acceptlabel):
+        # type: (Text) -> None
         """initialize dialog buttons"""
         self._buttons = QDialogButtonBox(self)
         self._accept_button = self._buttons.addButton(QDialogButtonBox.Ok)
@@ -244,16 +277,19 @@ class _PathDialog(QDialog):
 
     @property
     def path(self):
+        # type: () -> Text
         """value of path field"""
         return pycompat.unicode(self._path_edit.text())
 
     @property
     def localpath(self):
+        # type: () -> Text
         """value of localpath field"""
         return pycompat.unicode(self._localpath_edit.text())
 
     @pyqtSlot()
     def _browse_localpath(self):
+        # type: () -> None
         path = QFileDialog.getExistingDirectory(self, _('Select Repository'),
                                                 self.localpath)
         if not path:
@@ -269,15 +305,18 @@ class _PathDialog(QDialog):
 
     @pyqtSlot()
     def _updateform(self):
+        # type: () -> None
         """update availability of form elements"""
         self._accept_button.setEnabled(self._isacceptable())
 
     def _isacceptable(self):
+        # type: () -> bool
         return bool(self.path and self.localpath
                     and self.path not in self._invalidpaths)
 
     @classmethod
     def getaddpathmap(cls, parent, path=None, localpath=None, invalidpaths=None):
+        # type: (Optional[QWidget], Optional[Text], Optional[Text], Optional[Iterable[Text]]) -> Tuple[Optional[Text], Optional[Text]]
         d = cls(title=_('Add Path to Serve'), acceptlabel=_('Add'),
                 path=path, localpath=localpath,
                 invalidpaths=invalidpaths, parent=parent)
@@ -288,6 +327,7 @@ class _PathDialog(QDialog):
 
     @classmethod
     def geteditpathmap(cls, parent, path=None, localpath=None, invalidpaths=None):
+        # type: (Optional[QWidget], Optional[Text], Optional[Text], Optional[Iterable[Text]]) -> Tuple[Optional[Text], Optional[Text]]
         d = cls(title=_('Edit Path to Serve'), acceptlabel=_('Edit'),
                 path=path, localpath=localpath,
                 invalidpaths=invalidpaths, parent=parent)
@@ -302,10 +342,12 @@ class WebconfModel(QAbstractTableModel):
                 (_('Local Path'),)]
 
     def __init__(self, config, parent=None):
+        # type: (IniConfig, Optional[QObject]) -> None
         super(WebconfModel, self).__init__(parent)
         self._config = config
 
     def data(self, index, role):
+        # type: (QModelIndex, int) -> Any
         if not index.isValid():
             return None
         if role == Qt.DisplayRole:
@@ -314,32 +356,38 @@ class WebconfModel(QAbstractTableModel):
         return None
 
     def rowCount(self, parent=QModelIndex()):
+        # type: (QModelIndex) -> int
         if parent.isValid():
             return 0  # no child
         return len(self._config[b'paths'])
 
     def columnCount(self, parent=QModelIndex()):
+        # type: (QModelIndex) -> int
         if parent.isValid():
             return 0  # no child
         return len(self._COLUMNS)
 
     def headerData(self, section, orientation, role):
+        # type: (int, int, int) -> Any
         if role != Qt.DisplayRole or orientation != Qt.Horizontal:
             return None
         return self._COLUMNS[section][0]
 
     @property
     def paths(self):
+        # type: () -> List[Text]
         """return list of known paths"""
         return [hglib.tounicode(e) for e in self._config[b'paths']]
 
     def getpathmapat(self, row):
+        # type: (int) -> Tuple[Text, ...]
         """return pair of (path, localpath) at the specified index"""
         assert 0 <= row and row < self.rowCount(), row
         return tuple(hglib.tounicode(e)
                      for e in self._config.items(b'paths')[row])
 
     def addpathmap(self, path, localpath):
+        # type: (Text, Text) -> None
         """add path mapping to serve"""
         assert path not in self.paths, path
         self.beginInsertRows(QModelIndex(), self.rowCount(), self.rowCount())
@@ -350,6 +398,7 @@ class WebconfModel(QAbstractTableModel):
             self.endInsertRows()
 
     def setpathmap(self, path, localpath):
+        # type: (Text, Text) -> None
         """change path mapping at the specified index"""
         self._config.set(b'paths', hglib.fromunicode(path),
                          hglib.fromunicode(localpath))
@@ -358,6 +407,7 @@ class WebconfModel(QAbstractTableModel):
                               self.index(row, self.columnCount()))
 
     def removepathmap(self, path):
+        # type: (Text) -> None
         """remove path from mapping"""
         row = self._indexofpath(path)
         self.beginRemoveRows(QModelIndex(), row, row)
@@ -367,6 +417,7 @@ class WebconfModel(QAbstractTableModel):
             self.endRemoveRows()
 
     def _indexofpath(self, path):
+        # type: (Text) -> int
         path = hglib.fromunicode(path)
         assert path in self._config[b'paths'], path
         return list(self._config[b'paths']).index(path)
