@@ -3,7 +3,7 @@
 # Copyright 2008 Steve Borho <steve@borho.org>
 #
 # This software may be used and distributed according to the terms of the
-# GNU General Public License version 2, incorporated herein by reference.
+# GNU General Public License version 2 or any later version.
 
 import os
 import sys
@@ -11,11 +11,23 @@ import sys
 from mercurial import hg, node, error, pycompat, scmutil
 from tortoisehg.util import paths, debugthg, hglib
 
+if hglib.TYPE_CHECKING:
+    from typing import (
+        Dict,
+        List,
+        Optional,
+        Text,
+    )
+    from mercurial import (
+        localrepo,
+    )
+
+
 debugging = False
 enabled = True
 localonly = False
-includepaths = []
-excludepaths = []
+includepaths = []  # type: List[Text]
+excludepaths = []  # type: List[Text]
 
 try:
     from mercurial.windows import winreg
@@ -23,7 +35,7 @@ try:
     OpenKey = winreg.OpenKey
     QueryValueEx = winreg.QueryValueEx
 
-    from win32api import GetTickCount
+    from win32api import GetTickCount  # pytype: disable=import-error
     CACHE_TIMEOUT = 5000
     try:
         hkey = OpenKey(HKEY_CURRENT_USER, r"Software\TortoiseHg")
@@ -63,16 +75,17 @@ ROOT = "r"
 UNRESOLVED = 'U'
 
 # file status cache
-overlay_cache = {}
+overlay_cache = {}  # type: Dict[Optional[Text], Optional[Text]]
 cache_tick_count = 0
-cache_root = None
-cache_pdir = None
+cache_root = None  # type: Optional[Text]
+cache_pdir = None  # type: Optional[Text]
 
 
 def add_dirs(list):
+    # type: (List[bytes]) -> None
     dirs = set()
     if list:
-        dirs.add('')
+        dirs.add(b'')
     for f in list:
         pdir = os.path.dirname(f)
         if pdir in dirs:
@@ -84,6 +97,7 @@ def add_dirs(list):
 
 
 def get_state(upath, repo=None):
+    # type: (Text, Optional[localrepo.localrepository]) -> Text
     """
     Get the state of a given path in source control.
     """
@@ -92,6 +106,7 @@ def get_state(upath, repo=None):
 
 
 def get_states(path, repo=None):
+    # type: (Text, Optional[localrepo.localrepository]) -> Text
     """
     Get the states of a given path in source control.
     """
@@ -160,7 +175,7 @@ def get_states(path, repo=None):
             cache_tick_count = GetTickCount()
             debugf("overlayicons disabled")
             return NOT_IN_REPO
-        if localonly and not paths.is_on_fixed_drive(path):
+        if localonly and not paths.is_on_fixed_drive(hglib.fromunicode(path)):
             debugf("%s: is a network drive", path)
             overlay_cache = {None: None}
             cache_tick_count = GetTickCount()
@@ -210,7 +225,6 @@ def get_states(path, repo=None):
         return UNKNOWN
 
     debugf("status() took %g ticks", (GetTickCount() - tc1))
-    mergestate = repo.dirstate.parents()[1] != node.nullid
 
     # cached file info
     tc = GetTickCount()
@@ -218,9 +232,10 @@ def get_states(path, repo=None):
     add(root, ROOT)
     add(os.path.join(root, '.hg'), NOT_IN_REPO)
     states = STATUS_STATES
-    if mergestate:
-        mstate = hglib.readmergestate(repo)
-        unresolved = [f for f in mstate if mstate[f] == b'u']
+
+    mstate = hglib.readmergestate(repo)
+    if mstate.active():
+        unresolved = [f for f in mstate.unresolved()]
         if unresolved:
             repostate = [unresolved,
                          set(repostate.modified) - set(unresolved),
@@ -245,4 +260,5 @@ def get_states(path, repo=None):
 
 
 def add(path, state):
+    # type: (Text, Text) -> None
     overlay_cache[path] = overlay_cache.get(path, '') + state
