@@ -18,6 +18,9 @@ from tortoisehg.util import (
 configtable = {}
 configitem = registrar.configitem(configtable)
 
+# special value for case where the default is derived from other values
+dynamicdefault = configitem.dynamicdefault  # pytype: disable=attribute-error
+
 testedwith = hgversion.testedwith
 
 configitem(b'experimental', b'graph-group-branches', default=False)
@@ -41,16 +44,18 @@ configitem(b'tortoisehg', b'cmdserver.readtimeout', default=30)
 configitem(b'tortoisehg', b'confirmaddfiles', default=True)
 configitem(b'tortoisehg', b'confirmdeletefiles', default=True)
 configitem(b'tortoisehg', b'confirmpush', default=True)
+configitem(b'tortoisehg', b'copy_hash_selection', default=False)
 configitem(b'tortoisehg', b'deadbranch', default=b'')
 configitem(b'tortoisehg', b'defaultclonedest', default=None)
 configitem(b'tortoisehg', b'defaultpush', default=b'all')
 configitem(b'tortoisehg', b'defaultwidget', default=None)
 configitem(b'tortoisehg', b'editor', default=None)
 configitem(b'tortoisehg', b'engmsg', default=False)
-configitem(b'tortoisehg', b'fontcomment', default=configitem.dynamicdefault)
-configitem(b'tortoisehg', b'fontdiff', default=configitem.dynamicdefault)
-configitem(b'tortoisehg', b'fontlog', default=configitem.dynamicdefault)
-configitem(b'tortoisehg', b'fontoutputlog', default=configitem.dynamicdefault)
+configitem(b'tortoisehg', b'fontcomment', default=dynamicdefault)
+configitem(b'tortoisehg', b'fontdiff', default=dynamicdefault)
+configitem(b'tortoisehg', b'fonteditor', default=dynamicdefault)
+configitem(b'tortoisehg', b'fontlog', default=dynamicdefault)
+configitem(b'tortoisehg', b'fontoutputlog', default=dynamicdefault)
 configitem(b'tortoisehg', b'forcerepotab', default=False)
 configitem(b'tortoisehg', b'forcevdiffwin', default=False)
 configitem(b'tortoisehg', b'fullauthorname', default=False)
@@ -71,6 +76,7 @@ configitem(b'tortoisehg', b'issue.linkmandatory', default=False)
 configitem(b'tortoisehg', b'issue.regex', default=None)
 configitem(b'tortoisehg', b'longsummary', default=False)
 configitem(b'tortoisehg', b'maxdiff', default=None)
+configitem(b'tortoisehg', b'mergecommitmessage', default=None)
 configitem(b'tortoisehg', b'monitorrepo', default=b'localonly')
 configitem(b'tortoisehg', b'opentabsaftercurrent', default=True)
 configitem(b'tortoisehg', b'postpull', default=None)
