@@ -6,10 +6,22 @@
 # GNU General Public License version 2 or any later version.
 import os
 
+from . import hglib
+
+if hglib.TYPE_CHECKING:
+    from typing import (
+        List,
+        Text,
+    )
+    from mercurial import (
+        ui as uimod,
+    )
+
 if os.name == 'nt':
     from mercurial.windows import winreg
 
     def findgpg(ui):
+        # type: (uimod.ui) -> List[Text]
         path = []
         for key in (r"Software\GNU\GnuPG", r"Software\Wow6432Node\GNU\GnuPG"):
             try:
@@ -19,7 +31,7 @@ if os.name == 'nt':
                     for f in fileNames:
                         if f == 'gpg.exe':
                             path.append(os.path.join(dirPath, f))
-            except WindowsError:
+            except WindowsError:  # pytype: disable=name-error
                 pass
             except EnvironmentError:
                 pass
@@ -28,4 +40,5 @@ if os.name == 'nt':
 
 else:
     def findgpg(ui):
+        # type: (uimod.ui) -> List[Text]
         return []
