@@ -3,7 +3,7 @@
 # Copyright 2007 Steve Borho <steve@borho.org>
 #
 # This software may be used and distributed according to the terms of the
-# GNU General Public License version 2, incorporated herein by reference.
+# GNU General Public License version 2 or any later version.
 
 from __future__ import absolute_import
 
@@ -19,6 +19,7 @@ from hgext import mq as mqmod
 from mercurial import (
     cmdutil,
     config,
+    context,
     dispatch as dispatchmod,
     encoding,
     error,
@@ -36,6 +37,7 @@ from mercurial import (
     revsetlang,
     scmutil,
     state as statemod,
+    subrepo,
     subrepoutil,
     ui as uimod,
     util,
@@ -72,9 +74,12 @@ if TYPE_CHECKING:
         overload,
     )
     from mercurial import (
+        context,
         localrepo,
     )
     from .typelib import (
+        DiffTools,
+        HgContext,
         IniConfig,
     )
 
@@ -245,6 +250,12 @@ def _firstchangectx(repo):
         return repo[rev]
     return repo[nullrev]
 
+def revsymbol(repo, rev):
+    # type: (localrepo.localrepository, bytes) -> HgContext
+    ctx = scmutil.revsymbol(repo, rev)
+    assert isinstance(ctx, (context.changectx, context.workingctx))
+    return ctx
+
 def shortrepoid(repo):
     # type: (...) -> str
     """Short hash of the first root changeset; can be used for settings key"""
@@ -285,7 +296,7 @@ def getrevisionlabel(repo, rev):
 
     ctx = repo[rev]
     label = _getfirstrevisionlabel(repo, ctx)
-    if label and ctx == scmutil.revsymbol(repo, label):
+    if label and ctx == revsymbol(repo, label):
         return tounicode(label)
 
     return '%d' % rev
@@ -309,23 +320,33 @@ def getcurrentqqueue(repo):
         cur = cur[8:]
     return cur
 
-def gitcommit(ctx):
-    # type: (...) -> Optional[bytes]
+def gitcommit_full(ctx):
+    # type: (...) -> Optional[Text]
     """
     If the hggit extension is loaded, and the repository is a git clone,
-    returns the git commit hash of the current revision
+    returns the complete git commit hash of the current revision
     """
 
-    repo = ctx._repo
+    repo = ctx.repo()
 
     if b'hggit' not in repo.extensions():
         return None
 
-    fullgitnode = repo.githandler.map_git_get(ctx.hex())
+    fullgitnode = tounicode(repo.githandler.map_git_get(ctx.hex()))
     if fullgitnode is None:
         return None
 
-    return fullgitnode[:12]
+    return fullgitnode
+
+def gitcommit_short(ctx):
+    # type: (...) -> Optional[Text]
+    """
+    If the hggit extension is loaded, and the repository is a git clone,
+    returns the short (12 digits) git commit hash of the current revision
+    """
+
+    fullgitnode = gitcommit_full(ctx)
+    return None if fullgitnode is None else fullgitnode[:12]
 
 def getqqueues(repo):
     # type: (...) -> List[Text]
@@ -353,6 +374,28 @@ def readgraftstate(repo):
 mergestate = mergestatemod.mergestate
 readmergestate = mergestate.read
 
+
+def operation_in_progress(repo, statename):
+    # type: (localrepo.localrepository, bytes) -> bool
+    """Indicates if a long term operation is in progress"""
+    return statemod.cmdstate(repo, statename).exists()
+
+
+def graft_in_progress(repo):
+    # type: (localrepo.localrepository) -> bool
+    return operation_in_progress(repo, b"graftstate")
+
+
+def pick_in_progress(repo):
+    # type: (localrepo.localrepository) -> bool
+    return operation_in_progress(repo, b"pickstate")
+
+
+def rebase_in_progress(repo):
+    # type: (localrepo.localrepository) -> bool
+    return operation_in_progress(repo, b"rebasestate")
+
+
 def readundodesc(repo):
     # type: (...) -> Tuple[Text, int]
     """Read short description and changelog size of last transaction"""
@@ -402,7 +445,19 @@ def allextensions():
             exts['hgsubversion'] = _('hgsubversion packaged with thg')
         if 'hggit' not in exts:
             exts['hggit'] = _('hggit packaged with thg')
-        exts.pop('mercurial_extension_utils', None)  # Part of keyring extension
+        if 'evolve' not in exts:
+            exts['evolve'] = _('evolve packaged with thg')
+        if 'topic' not in exts:
+            exts['topic'] = _('topic packaged with thg')
+
+    exts.pop('mercurial_extension_utils', None)  # Part of keyring extension
+    exts.pop('hooklib', None)  # Internal, but also needs [hooks] setup
+
+    # The core git extension requires pygit2, which doesn't support py2, so no
+    # sense in advertising it.
+    if not pycompat.ispy3:
+        exts.pop('git', None)
+
     return exts
 
 def validateextensions(enabledexts):
@@ -483,6 +538,14 @@ def canonpaths(list):
                 canonpats.append(f)
     return canonpats
 
+
+def normpath(path):
+    # type: (Text) -> Text
+    """Normalize a path in the same manner as mercurial.util.normpath()."""
+    # TODO: Do a unicode port of util.normpath() here.
+    return tounicode(util.normpath(fromunicode(path)))
+
+
 def normreporoot(path):
     # type: (Text) -> Text
     """Normalize repo root path in the same manner as localrepository"""
@@ -514,9 +577,9 @@ def mergetools(ui):
     return values
 
 
-_difftools = None
+_difftools = None  # type: Optional[DiffTools]
 def difftools(ui):
-    # TODO: type annotation
+    # type: (uimod.ui) -> DiffTools
     '''Return mapping from tool name to tuples with tool name,
     diff args, and merge args, all as bytecode strings'''
     global _difftools
@@ -534,7 +597,7 @@ def difftools(ui):
             mergeopts = []
         return diffopts, mergeopts
 
-    tools = {}
+    tools = {}  # type: DiffTools
     for cmd, path in ui.configitems(b'extdiff'):
         if cmd.startswith(b'cmd.'):
             cmd = cmd[4:]
@@ -543,7 +606,7 @@ def difftools(ui):
             diffopts = ui.config(b'extdiff', b'opts.' + cmd)
             diffopts = pycompat.shlexsplit(diffopts)
             diffopts, mergeopts = fixup_extdiff(diffopts)
-            tools[cmd] = [path, diffopts, mergeopts]
+            tools[cmd] = (path, diffopts, mergeopts)
         elif cmd.startswith(b'opts.'):
             continue
         else:
@@ -554,7 +617,7 @@ def difftools(ui):
             else:
                 path, diffopts = cmd, []
             diffopts, mergeopts = fixup_extdiff(diffopts)
-            tools[cmd] = [path, diffopts, mergeopts]
+            tools[cmd] = (path, diffopts, mergeopts)
     for t in mergetools(ui):
         if t.startswith(b'internal:'):
             continue
@@ -793,11 +856,11 @@ def extractchoices(prompttext):
     return msg, pycompat.ziplist(resps, choices)
 
 def displaytime(date):
-    # type: (Optional[Tuple[int, int]]) -> bytes
+    # type: (Optional[Tuple[float, int]]) -> bytes
     return dateutil.datestr(date, b'%Y-%m-%d %H:%M:%S %1%2')
 
 def utctime(date):
-    # type: (Tuple[int, int]) -> str
+    # type: (Tuple[float, int]) -> str
     return time.strftime("%Y-%m-%d %H:%M:%S", time.gmtime(date[0]))
 
 agescales = [
@@ -873,13 +936,20 @@ def user(ctx):
 def getestimatedsize(fctx):
     # type: (...) -> int
     """Return the size of the given fctx without loading the revision text"""
+    FILE_SIZE_UNKNOWN = 2147483647
+
     if fctx.rev() is None:
         return fctx.size()
     else:
         # fctx.size() can read all data into memory in rename cases so
         # we read the size directly from the filelog, this is deeper
         # under the API than I prefer to go, but seems necessary
-        return fctx._filelog._revlog.rawsize(fctx.filerev())
+        try:
+            return fctx._filelog._revlog.rawsize(fctx.filerev())
+        except AttributeError:
+            # _filelog is not available when using the remotefilelog extension
+            # on a shallow repo
+            return FILE_SIZE_UNKNOWN
 
 def get_revision_desc(fctx, curpath=None):
     # type: (Any, Optional[bytes]) -> pycompat.unicode
@@ -921,7 +991,7 @@ def longsummary(description, limit=None):
     return summary
 
 def getDeepestSubrepoContainingFile(wfile, ctx):
-    # TODO: type annotation
+    # type: (bytes, context.changectx) -> Tuple[Optional[bytes], bytes, context.changectx]
     """
     Given a filename and context, get the deepest subrepo that contains the file
 
@@ -929,18 +999,23 @@ def getDeepestSubrepoContainingFile(wfile, ctx):
     its containing subrepo
     """
     if wfile in ctx:
-        return '', wfile, ctx
-    for wsub in ctx.substate:
+        return b'', wfile, ctx
+
+    # TODO: figure out how to annotate ctx.substate so that pytype sees the
+    #   correct type instead of Any.
+    for wsub in ctx.substate:  # type: bytes
         if wfile.startswith(wsub):
-            srev = ctx.substate[wsub][1]
-            stype = ctx.substate[wsub][2]
-            if stype != 'hg':
+            srev = ctx.substate[wsub][1]  # type: bytes
+            stype = ctx.substate[wsub][2]  # type: bytes
+            if stype != b'hg':
                 continue
-            if not os.path.exists(ctx._repo.wjoin(wsub)):
+            if not os.path.exists(ctx.repo().wjoin(wsub)):
                 # Maybe the repository does not exist in the working copy?
                 continue
             try:
-                sctx = ctx.sub(wsub)._repo[srev]
+                s = ctx.sub(wsub)
+                assert isinstance(s, subrepo.hgsubrepo)  # help pytype
+                sctx = s._repo[srev]  # type: context.changectx
             except:
                 # The selected revision does not exist in the working copy
                 continue
@@ -1213,6 +1288,8 @@ def buildcmdargs(name, *args, **opts):
     ['qpush']
     >>> buildcmdargs('update', '')
     ['update', '']
+    >>> buildcmdargs('update', 25)
+    ['update', '25']
 
     type conversion to string:
 
