@@ -3,7 +3,7 @@
 # Copyright 2009 Steve Borho <steve@borho.org>
 #
 # This software may be used and distributed according to the terms of the
-# GNU General Public License version 2, incorporated herein by reference.
+# GNU General Public License version 2 or any later version.
 
 from __future__ import print_function
 
@@ -14,8 +14,25 @@ from mercurial import hg, node, error
 from tortoisehg.util.i18n import _ as gettext
 from tortoisehg.util import cachethg, paths, hglib
 
+if hglib.TYPE_CHECKING:
+    from typing import (
+        Dict,
+        List,
+        Optional,
+        Text,
+        Union,
+    )
+    from mercurial import (
+        localrepo,
+        ui as uimod,
+    )
+
+    MenuT = List[Union["TortoiseMenu", "TortoiseMenuSep"]]
+
+
 def _(msgid):
-    return {'id': msgid, 'str': gettext(msgid).encode('utf-8')}
+    # type: (Text) -> Dict[Text, Text]
+    return {'id': msgid, 'str': gettext(msgid)}
 
 thgcmenu = {
     'commit':     { 'label': _('Commit...'),
@@ -99,6 +116,7 @@ _ALWAYS_DEMOTE_ = ('about', 'userconf', 'repoconf')
 class TortoiseMenu(object):
 
     def __init__(self, menutext, helptext, hgcmd, icon=None, state=True):
+        # type: (Text, Text, Optional[Text], Optional[Text], bool) -> None
         self.menutext = menutext
         self.helptext = helptext
         self.hgcmd = hgcmd
@@ -115,12 +133,14 @@ class TortoiseMenu(object):
 class TortoiseSubmenu(TortoiseMenu):
 
     def __init__(self, menutext, helptext, menus=None, icon=None):
+        # type: (Text, Text, Optional[MenuT], Optional[Text]) -> None
         TortoiseMenu.__init__(self, menutext, helptext, None, icon)
         if menus is None:
-            menus = []
+            menus = []  # type: MenuT
         self.menus = menus[:]
 
     def add_menu(self, menutext, helptext, hgcmd, icon=None, state=True):
+        # type: (Text, Text, Optional[Text], Optional[Text], bool) -> None
         self.menus.append(TortoiseMenu(menutext, helptext,
                 hgcmd, icon, state))
 
@@ -151,13 +171,15 @@ class TortoiseMenuSep(object):
 class thg_menu(object):
 
     def __init__(self, ui, promoted, name = "TortoiseHg"):
-        self.menus = [[]]
+        # type: (uimod.ui, List[Text], Text) -> None
+        self.menus = [[]]  # type: List[MenuT]
         self.ui = ui
         self.name = name
         self.sep = [False]
         self.promoted = promoted
 
     def add_menu(self, hgcmd, icon=None, state=True):
+        # type: (Text, Optional[Text], bool) -> None
         if hgcmd in self.promoted:
             pos = 0
         else:
@@ -169,8 +191,8 @@ class thg_menu(object):
             self.sep[pos] = False
             self.menus[pos].append(TortoiseMenuSep())
         self.menus[pos].append(TortoiseMenu(
-                hglib.tounicode(thgcmenu[hgcmd]['label']['str']),
-                hglib.tounicode(thgcmenu[hgcmd]['help']['str']),
+                thgcmenu[hgcmd]['label']['str'],
+                thgcmenu[hgcmd]['help']['str'],
                 hgcmd,
                 thgcmenu[hgcmd]['icon'], state))
 
@@ -178,6 +200,7 @@ class thg_menu(object):
         self.sep = [True for _s in self.sep]
 
     def get(self):
+        # type: () -> MenuT
         menu = self.menus[0][:]
         for submenu in self.menus[1:]:
             menu.append(TortoiseSubmenu(self.name, 'Mercurial', submenu, "hg.ico"))
@@ -189,10 +212,11 @@ class thg_menu(object):
 
 
 def open_repo(path):
+    # type: (Text) -> Optional[localrepo.localrepository]
     root = paths.find_root(path)
     if root:
         try:
-            repo = hg.repository(hglib.loadui(), path=root)
+            repo = hg.repository(hglib.loadui(), path=hglib.fromunicode(root))
             return repo
         except error.RepoError:
             pass
@@ -207,9 +231,11 @@ class menuThg:
     """shell extension that adds context menu items"""
 
     def __init__(self, internal=False):
+        # type: (bool) -> None
         self.name = "TortoiseHg"
         promoted = []
         pl = hglib.loadui().config(b'tortoisehg', b'promoteditems', b'commit,log')
+        assert pl is not None
         for item in pl.split(b','):
             item = hglib.tounicode(item.strip())
             if item:
@@ -224,6 +250,7 @@ class menuThg:
 
 
     def get_commands_dragdrop(self, srcfiles, destfolder):
+        # type: (List[Text], Text) -> Union[List[Text], thg_menu]
         """
         Get a list of commands valid for the current selection.
 
@@ -255,6 +282,7 @@ class menuThg:
         return menu
 
     def get_norepo_commands(self, cwd, files):
+        # type: (Text, List[Text]) -> thg_menu
         menu = thg_menu(hglib.loadui(), self.promoted, self.name)
         menu.add_menu('clone')
         menu.add_menu('init')
@@ -265,6 +293,7 @@ class menuThg:
         return menu
 
     def get_commands(self, repo, cwd, files):
+        # type: (localrepo.localrepository, Text, List[Text]) -> thg_menu
         """
         Get a list of commands valid for the current selection.
 
