@@ -15,6 +15,7 @@ from mercurial import (
     error,
     node,
     patch,
+    phases,
     pycompat,
     util,
 )
@@ -55,7 +56,7 @@ class patchctx(object):
         self._mtime = None
         self._fsize = 0
         self._parseerror = None
-        self._phase = b'draft'
+        self._phase = phases.draft
 
         try:
             self._mtime = os.path.getmtime(patchpath)
@@ -70,7 +71,7 @@ class patchctx(object):
             self._branch = ph.branch or b''
             self._node = binascii.unhexlify(ph.nodeid)
             if self._repo.ui.configbool(b'mq', b'secret'):
-                self._phase = b'secret'
+                self._phase = phases.secret
         except TypeError:
             pass
         except AttributeError:
@@ -96,7 +97,7 @@ class patchctx(object):
         # ensure the patch contents are re-read
         self._mtime = 0
 
-    @property
+    @propertycache
     def substate(self):
         return {}  # unapplied patch won't include .hgsubstate
 
@@ -201,9 +202,12 @@ class patchctx(object):
             return buf.getvalue()
         return b''
 
-    def phasestr(self):
+    def phase(self):
         return self._phase
 
+    def phasestr(self):
+        return phases.phasenames[self.phase()]
+
     def hidden(self):
         return False
 
