@@ -10,6 +10,7 @@ import re
 
 from mercurial import (
     config as config_mod,
+    encoding,
     error,
     pycompat,
     util,
@@ -17,34 +18,51 @@ from mercurial import (
 
 from tortoisehg.util import hglib
 
+if hglib.TYPE_CHECKING:
+    from typing import (
+        Optional,
+        Union,
+    )
+
 if pycompat.ispy3:
     import configparser
 else:
-    import ConfigParser as configparser
+    import ConfigParser as configparser  # pytype: disable=import-error
 
 try:
-    from iniparse import INIConfig
+    from iniparse import INIConfig  # pytype: disable=import-error
     _hasiniparse = True
 except ImportError:
     _hasiniparse = False
 
 if _hasiniparse:
     try:
-        from iniparse import change_comment_syntax  # iniparse>=0.3.2
+        # iniparse>=0.3.2
+        from iniparse import change_comment_syntax  # pytype: disable=import-error
         change_comment_syntax(allow_rem=False)
     except (ImportError, TypeError):
         # TODO: yet need to care about iniparse<0.3.2 ??
-        from iniparse.ini import CommentLine
+        from iniparse.ini import CommentLine  # pytype: disable=import-error
         # Monkypatch this regex to prevent iniparse from considering
         # 'rem' as a comment
         CommentLine.regex = re.compile(r'^(?P<csep>[%;#])(?P<comment>.*)$')
 
     # allow :suboption in <name>
-    from iniparse.ini import OptionLine
+    from iniparse.ini import OptionLine  # pytype: disable=import-error
     OptionLine.regex = re.compile(r'^(?P<name>[^:=\s[][^=]*)'
                                   r'(?P<sep>=\s*)'
                                   r'(?P<value>.*)$')
 
+# Since hg 5.8 (a3dced4b7b04), config dict is no longer a plain
+# {key: value} dict, but has metadata for each key.
+
+def _packvalue(config, value, source):
+    return (value, source, config._current_source_level)
+
+def _unpackvalue(packed):
+    value, _source, _level = packed
+    return value
+
 class _wsortdict(object):
     """Wrapper for config.sortdict to record set/del operations"""
     def __init__(self, dict):
@@ -61,7 +79,7 @@ class _wsortdict(object):
 
     def __setitem__(self, key, val):
         self._setdict(key, val)
-        self._logset(key, val)
+        self._logset(key, _unpackvalue(val))
 
     def _logset(self, key, val):
         """Record set operation to log; called also by _wconfig"""
@@ -101,7 +119,7 @@ class _wsortdict(object):
     def _logupdate(self, src):
         """Record update operation to log; called also by _wconfig"""
         for k in src:
-            self._logset(k, src[k])
+            self._logset(k, _unpackvalue(src[k]))
 
     def __delitem__(self, key):
         del self._dict[key]
@@ -140,6 +158,7 @@ class _wconfig(object):
     """
 
     def __init__(self, data=None):
+        # type: (Optional[Union[_wconfig, config_mod.config]]) -> None
         self._config = config_mod.config(data)
         self._readfiles = []  # list of read (path, fp, sections, remap)
         self._sections = {}
@@ -194,7 +213,7 @@ class _wconfig(object):
             # need to handle 'source'
             self._config.set(section, item, value, source)
         else:
-            self[section][item] = value
+            self[section][item] = _packvalue(self._config, value, source)
 
     def remove(self, section, item):
         del self[section][item]
@@ -215,19 +234,30 @@ class _wconfig(object):
             raise NotImplementedError("wconfig does not support read() more "
                                       "than once")
 
+        def _filename(err):
+            if pycompat.ispy3:
+                return err.source
+            else:
+                return err.filename
+
         def newini(fp=None):
             try:
                 # TODO: optionxformvalue isn't used by INIConfig ?
                 return INIConfig(fp=fp, optionxformvalue=None)
             except configparser.MissingSectionHeaderError as err:
-                raise error.ParseError(err.message.splitlines()[0],
-                                       '%s:%d' % (err.filename, err.lineno))
+                raise error.ParseError(
+                    encoding.strtolocal(err.message.splitlines()[0]),
+                    encoding.strtolocal('%s:%d' % (_filename(err), err.lineno))
+                )
             except configparser.ParsingError as err:
                 if err.errors:
-                    loc = '%s:%d' % (err.filename, err.errors[0][0])
+                    loc = '%s:%d' % (_filename(err), err.errors[0][0])
                 else:
-                    loc = err.filename
-                raise error.ParseError(err.message.splitlines()[0], loc)
+                    loc = _filename(err)
+                raise error.ParseError(
+                    encoding.strtolocal(err.message.splitlines()[0]),
+                    encoding.strtolocal(loc)
+                )
 
         if not self._readfiles:
             return newini()
